@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  *  Author: aCaB <acab@clamav.net>
@@ -41,7 +41,7 @@
 #include "clamav.h"
 #include "default.h"
 
-// shared
+// common
 #include "output.h"
 #include "optparser.h"
 #include "misc.h"
@@ -49,7 +49,7 @@
 #include "connpool.h"
 #include "netcode.h"
 #include "clamfi.h"
-#include "whitelist.h"
+#include "allow_list.h"
 
 #ifndef _WIN32
 #include <sys/wait.h>
@@ -62,25 +62,25 @@ static void milter_exit(int sig)
 {
     const struct optstruct *opt;
 
-    logg("*clamav-milter: milter_exit, signal %d\n", sig);
+    logg(LOGG_DEBUG, "clamav-milter: milter_exit, signal %d\n", sig);
 
 #ifndef _WIN32
     if ((opt = optget(opts, "MilterSocket"))) {
         if (unlink(opt->strarg) == -1)
-            logg("!Can't unlink the socket file %s\n", opt->strarg);
+            logg(LOGG_ERROR, "Can't unlink the socket file %s\n", opt->strarg);
         else
-            logg("Socket file removed.\n");
+            logg(LOGG_INFO, "Socket file removed.\n");
     }
 #endif
 
-    logg("clamav-milter: stopped\n");
+    logg(LOGG_INFO, "clamav-milter: stopped\n");
 
     optfree(opts);
 
     logg_close();
     cpool_free();
     localnets_free();
-    whitelist_free();
+    allow_list_free();
 }
 
 int main(int argc, char **argv)
@@ -96,7 +96,7 @@ int main(int argc, char **argv)
 
     sigset_t sigset;
     struct sigaction act;
-    const char * user_name = NULL;
+    const char *user_name = NULL;
 
     cl_initialize_crypto();
 
@@ -114,7 +114,7 @@ int main(int argc, char **argv)
 
     opts = optparse(NULL, argc, argv, 1, OPT_MILTER, 0, NULL);
     if (!opts) {
-        mprintf("!Can't parse command line options\n");
+        mprintf(LOGG_ERROR, "Can't parse command line options\n");
         return 1;
     }
 
@@ -122,7 +122,7 @@ int main(int argc, char **argv)
         printf("\n");
         printf("                       Clam AntiVirus: Milter Mail Scanner %s\n", get_version());
         printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-        printf("           (C) 2021 Cisco Systems, Inc.\n");
+        printf("           (C) 2022 Cisco Systems, Inc.\n");
         printf("\n");
         printf("    %s [-c <config-file>]\n\n", argv[0]);
         printf("\n");
@@ -137,7 +137,7 @@ int main(int argc, char **argv)
     if (opts->filename) {
         int x;
         for (x = 0; opts->filename[x]; x++)
-            mprintf("^Ignoring option %s\n", opts->filename[x]);
+            mprintf(LOGG_WARNING, "Ignoring option %s\n", opts->filename[x]);
     }
 
     if (optget(opts, "version")->enabled) {
@@ -158,17 +158,17 @@ int main(int argc, char **argv)
     }
     free(pt);
 
-    if ((opt = optget(opts, "User"))->enabled){
+    if ((opt = optget(opts, "User"))->enabled) {
         user_name = opt->strarg;
     }
 
     if ((opt = optget(opts, "Chroot"))->enabled) {
         if (chdir(opt->strarg) != 0) {
-            logg("!Cannot change directory to %s\n", opt->strarg);
+            logg(LOGG_ERROR, "Cannot change directory to %s\n", opt->strarg);
             return 1;
         }
         if (chroot(opt->strarg) != 0) {
-            logg("!chroot to %s failed. Are you root?\n", opt->strarg);
+            logg(LOGG_ERROR, "chroot to %s failed. Are you root?\n", opt->strarg);
             return 1;
         }
     }
@@ -201,20 +201,20 @@ int main(int argc, char **argv)
     }
 
     if (!(my_socket = optget(opts, "MilterSocket")->strarg)) {
-        logg("!Please configure the MilterSocket directive\n");
+        logg(LOGG_ERROR, "Please configure the MilterSocket directive\n");
         logg_close();
         optfree(opts);
         return 1;
     }
 
     if (smfi_setconn(my_socket) == MI_FAILURE) {
-        logg("!smfi_setconn failed\n");
+        logg(LOGG_ERROR, "smfi_setconn failed\n");
         logg_close();
         optfree(opts);
         return 1;
     }
     if (smfi_register(descr) == MI_FAILURE) {
-        logg("!smfi_register failed\n");
+        logg(LOGG_ERROR, "smfi_register failed\n");
         logg_close();
         optfree(opts);
         return 1;
@@ -222,7 +222,7 @@ int main(int argc, char **argv)
     opt  = optget(opts, "FixStaleSocket");
     umsk = umask(0777); /* socket is created with 000 to avoid races */
     if (smfi_opensocket(opt->enabled) == MI_FAILURE) {
-        logg("!Failed to create socket %s\n", my_socket);
+        logg(LOGG_ERROR, "Failed to create socket %s\n", my_socket);
         logg_close();
         optfree(opts);
         return 1;
@@ -245,7 +245,7 @@ int main(int argc, char **argv)
             if (*end) {
                 struct group *pgrp = getgrnam(gname);
                 if (!pgrp) {
-                    logg("!Unknown group %s\n", gname);
+                    logg(LOGG_ERROR, "Unknown group %s\n", gname);
                     logg_close();
                     optfree(opts);
                     return 1;
@@ -253,7 +253,7 @@ int main(int argc, char **argv)
                 sock_gid = pgrp->gr_gid;
             }
             if (chown(sock_name, -1, sock_gid)) {
-                logg("!Failed to change socket ownership to group %s\n", gname);
+                logg(LOGG_ERROR, "Failed to change socket ownership to group %s\n", gname);
                 logg_close();
                 optfree(opts);
                 return 1;
@@ -263,7 +263,7 @@ int main(int argc, char **argv)
         if (NULL != user_name) {
             struct passwd *user;
             if ((user = getpwnam(user_name)) == NULL) {
-                logg("ERROR: Can't get information about user %s.\n",
+                logg(LOGG_INFO, "ERROR: Can't get information about user %s.\n",
                      user_name);
                 logg_close();
                 optfree(opts);
@@ -271,7 +271,7 @@ int main(int argc, char **argv)
             }
 
             if (chown(sock_name, user->pw_uid, -1)) {
-                logg("!Failed to change socket ownership to user %s\n", user->pw_name);
+                logg(LOGG_ERROR, "Failed to change socket ownership to user %s\n", user->pw_name);
                 optfree(opts);
                 logg_close();
                 return 1;
@@ -282,7 +282,7 @@ int main(int argc, char **argv)
             char *end;
             sock_mode = strtol(optget(opts, "MilterSocketMode")->strarg, &end, 8);
             if (*end) {
-                logg("!Invalid MilterSocketMode %s\n", optget(opts, "MilterSocketMode")->strarg);
+                logg(LOGG_ERROR, "Invalid MilterSocketMode %s\n", optget(opts, "MilterSocketMode")->strarg);
                 logg_close();
                 optfree(opts);
                 return 1;
@@ -291,7 +291,7 @@ int main(int argc, char **argv)
             sock_mode = 0777 & ~umsk;
 
         if (chmod(sock_name, sock_mode & 0666)) {
-            logg("!Cannot set milter socket permission to %s\n", optget(opts, "MilterSocketMode")->strarg);
+            logg(LOGG_ERROR, "Cannot set milter socket permission to %s\n", optget(opts, "MilterSocketMode")->strarg);
             logg_close();
             optfree(opts);
             return 1;
@@ -322,7 +322,7 @@ int main(int argc, char **argv)
 
         opt = optget(opts, "LogFacility");
         if ((fac = logg_facility(opt->strarg)) == -1) {
-            logg("!LogFacility: %s: No such facility.\n", opt->strarg);
+            logg(LOGG_ERROR, "LogFacility: %s: No such facility.\n", opt->strarg);
             logg_close();
             optfree(opts);
             return 1;
@@ -334,7 +334,7 @@ int main(int argc, char **argv)
 #endif
 
     time(&currtime);
-    if (logg("#+++ Started at %s", ctime(&currtime))) {
+    if (logg(LOGG_INFO_NF, "+++ Started at %s", ctime(&currtime))) {
         fprintf(stderr, "ERROR: Can't initialize the internal logger\n");
         logg_close();
         optfree(opts);
@@ -349,7 +349,7 @@ int main(int argc, char **argv)
         return 1;
     }
 
-    if ((opt = optget(opts, "Whitelist"))->enabled && whitelist_init(opt->strarg)) {
+    if (((opt = optget(opts, "Whitelist"))->enabled || (opt = optget(opts, "AllowList"))->enabled) && allow_list_init(opt->strarg)) {
         localnets_free();
         logg_close();
         optfree(opts);
@@ -358,7 +358,7 @@ int main(int argc, char **argv)
 
     if ((opt = optget(opts, "SkipAuthenticated"))->enabled && smtpauth_init(opt->strarg)) {
         localnets_free();
-        whitelist_free();
+        allow_list_free();
         logg_close();
         optfree(opts);
         return 1;
@@ -369,16 +369,16 @@ int main(int argc, char **argv)
 #ifndef _WIN32
     if (!optget(opts, "Foreground")->enabled) {
         if (-1 == daemonize_parent_wait(user_name, logg_file)) {
-            logg("!daemonize() failed\n");
+            logg(LOGG_ERROR, "daemonize() failed\n");
             localnets_free();
-            whitelist_free();
+            allow_list_free();
             cpool_free();
             logg_close();
             optfree(opts);
             return 1;
         }
         if (chdir("/") == -1) {
-            logg("^Can't change current working directory to root\n");
+            logg(LOGG_WARNING, "Can't change current working directory to root\n");
         }
     }
 
@@ -401,16 +401,16 @@ int main(int argc, char **argv)
 
     maxfilesize = optget(opts, "MaxFileSize")->numarg;
     if (!maxfilesize) {
-        logg("^Invalid MaxFileSize, using default (%d)\n", CLI_DEFAULT_MAXFILESIZE);
+        logg(LOGG_WARNING, "Invalid MaxFileSize, using default (%d)\n", CLI_DEFAULT_MAXFILESIZE);
         maxfilesize = CLI_DEFAULT_MAXFILESIZE;
     }
     readtimeout = optget(opts, "ReadTimeout")->numarg;
 
     cpool_init(opts);
     if (!cp) {
-        logg("!Failed to init the socket pool\n");
+        logg(LOGG_ERROR, "Failed to init the socket pool\n");
         localnets_free();
-        whitelist_free();
+        allow_list_free();
         logg_close();
         optfree(opts);
         return 1;
@@ -418,15 +418,15 @@ int main(int argc, char **argv)
 
     if ((opt = optget(opts, "PidFile"))->enabled) {
         FILE *fd;
-        mode_t old_umask = umask(0002);
-        int err = 0;
+        mode_t old_umask = umask(0022);
+        int err          = 0;
 
         if ((fd = fopen(opt->strarg, "w")) == NULL) {
-            logg("!Can't save PID in file %s\n", opt->strarg);
+            logg(LOGG_ERROR, "Can't save PID in file %s\n", opt->strarg);
             err = 1;
         } else {
             if (fprintf(fd, "%u\n", (unsigned int)getpid()) < 0) {
-                logg("!Can't save PID in file %s\n", opt->strarg);
+                logg(LOGG_ERROR, "Can't save PID in file %s\n", opt->strarg);
                 err = 1;
             }
             fclose(fd);
@@ -434,24 +434,24 @@ int main(int argc, char **argv)
         umask(old_umask);
 
 #ifndef _WIN32
-        if (0 == err){
+        if (0 == err) {
             /*If the file has already been created by a different user, it will just be
              * rewritten by us, but not change the ownership, so do that explicitly.
              */
-            if (0 == geteuid()){
-                struct passwd * pw = getpwuid(0);
-                int ret = lchown(opt->strarg, pw->pw_uid, pw->pw_gid);
-                if (ret){
-                    logg("!Can't change ownership of PID file %s '%s'\n", opt->strarg, strerror(errno));
+            if (0 == geteuid()) {
+                struct passwd *pw = getpwuid(0);
+                int ret           = lchown(opt->strarg, pw->pw_uid, pw->pw_gid);
+                if (ret) {
+                    logg(LOGG_ERROR, "Can't change ownership of PID file %s '%s'\n", opt->strarg, strerror(errno));
                     err = 1;
                 }
             }
         }
 #endif /*_WIN32*/
 
-        if (err){
+        if (err) {
             localnets_free();
-            whitelist_free();
+            allow_list_free();
             logg_close();
             optfree(opts);
             return 2;
@@ -460,7 +460,7 @@ int main(int argc, char **argv)
 
 #ifndef _WIN32
     dropPrivRet = drop_privileges(user_name, logg_file);
-    if (dropPrivRet){
+    if (dropPrivRet) {
         optfree(opts);
         return dropPrivRet;
     }
@@ -468,7 +468,7 @@ int main(int argc, char **argv)
     /* We have been daemonized, and initialization is done.  Signal
      * the parent process so that it can exit cleanly.
      */
-    if (parentPid != getpid()){ //we have been daemonized
+    if (parentPid != getpid()) { // we have been daemonized
         daemonize_signal_parent(parentPid);
     }
 #endif
