@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -45,6 +45,7 @@
 #endif
 #include <signal.h>
 #include <errno.h>
+#include <locale.h>
 
 #if defined(USE_SYSLOG) && !defined(C_AIX)
 #include <syslog.h>
@@ -56,22 +57,33 @@
 
 #include "target.h"
 
-#include "libclamav/clamav.h"
-#include "libclamav/others.h"
-#include "libclamav/matcher-ac.h"
-#include "libclamav/readdb.h"
+// libclamav
+#include "clamav.h"
+#include "others.h"
+#include "matcher-ac.h"
+#include "readdb.h"
 
-#include "shared/output.h"
-#include "shared/optparser.h"
-#include "shared/misc.h"
+// common
+#include "output.h"
+#include "optparser.h"
+#include "misc.h"
 
 #include "server.h"
 #include "tcpserver.h"
 #include "localserver.h"
-#include "others.h"
+#include "clamd_others.h"
 #include "shared.h"
 #include "scanner.h"
 
+#ifdef _WIN32
+#include "service.h"
+#endif
+
+#include <sys/types.h>
+#ifndef WIN32
+#include <sys/wait.h>
+#endif
+
 short debug_mode = 0, logok = 0;
 short foreground = -1;
 
@@ -80,14 +92,19 @@ static void help(void)
     printf("\n");
     printf("                      Clam AntiVirus: Daemon %s\n", get_version());
     printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-    printf("           (C) 2020 Cisco Systems, Inc.\n");
+    printf("           (C) 2022 Cisco Systems, Inc.\n");
     printf("\n");
     printf("    clamd [options]\n");
     printf("\n");
     printf("    --help                   -h             Show this help\n");
     printf("    --version                -V             Show version number\n");
+#ifdef _WIN32
+    printf("    --install-service                       Install Windows Service\n");
+    printf("    --uninstall-service                     Uninstall Windows Service\n");
+#endif
     printf("    --foreground             -F             Run in foreground; do not daemonize\n");
     printf("    --debug                                 Enable debug mode\n");
+    printf("    --log=FILE               -l FILE        Log into FILE\n");
     printf("    --config-file=FILE       -c FILE        Read configuration from FILE\n");
     printf("\n");
     printf("Pass in - as the filename for stdin.\n");
@@ -117,6 +134,9 @@ int main(int argc, char **argv)
 #ifndef _WIN32
     struct passwd *user = NULL;
     struct sigaction sa;
+    int dropPrivRet = 0;
+#endif
+#if defined(C_LINUX) || (defined(RLIMIT_DATA) && defined(C_BSD))
     struct rlimit rlim;
 #endif
     time_t currtime;
@@ -130,9 +150,13 @@ int main(int argc, char **argv)
     unsigned int i;
     int j;
     int num_fd;
+    pid_t parentPid = getpid();
 #ifdef C_LINUX
     STATBUF sb;
 #endif
+    pid_t mainpid         = 0;
+    mode_t old_umask      = 0;
+    const char *user_name = NULL;
 
     if (check_flevel())
         exit(1);
@@ -142,10 +166,13 @@ int main(int argc, char **argv)
     sa.sa_handler = SIG_IGN;
     sigaction(SIGHUP, &sa, NULL);
     sigaction(SIGUSR2, &sa, NULL);
+    if (!setlocale(LC_CTYPE, "")) {
+        mprintf(LOGG_WARNING, "Failed to set locale\n");
+    }
 #endif
 
     if ((opts = optparse(NULL, argc, argv, 1, OPT_CLAMD, 0, NULL)) == NULL) {
-        mprintf("!Can't parse command line options\n");
+        mprintf(LOGG_ERROR, "Can't parse command line options\n");
         return 1;
     }
 
@@ -197,49 +224,16 @@ int main(int argc, char **argv)
     }
     free(pt);
 
+    if ((opt = optget(opts, "User"))->enabled) {
+        user_name = opt->strarg;
+    }
+
     if (optget(opts, "version")->enabled) {
         print_version(optget(opts, "DatabaseDirectory")->strarg);
         optfree(opts);
         return 0;
     }
 
-    /* drop privileges */
-#ifndef _WIN32
-    if (geteuid() == 0 && (opt = optget(opts, "User"))->enabled) {
-        if ((user = getpwnam(opt->strarg)) == NULL) {
-            fprintf(stderr, "ERROR: Can't get information about user %s.\n", opt->strarg);
-            optfree(opts);
-            return 1;
-        }
-
-#ifdef HAVE_INITGROUPS
-        if (initgroups(opt->strarg, user->pw_gid)) {
-            fprintf(stderr, "ERROR: initgroups() failed.\n");
-            optfree(opts);
-            return 1;
-        }
-#elif HAVE_SETGROUPS
-        if (setgroups(1, &user->pw_gid)) {
-            fprintf(stderr, "ERROR: setgroups() failed.\n");
-            optfree(opts);
-            return 1;
-        }
-#endif
-
-        if (setgid(user->pw_gid)) {
-            fprintf(stderr, "ERROR: setgid(%d) failed.\n", (int)user->pw_gid);
-            optfree(opts);
-            return 1;
-        }
-
-        if (setuid(user->pw_uid)) {
-            fprintf(stderr, "ERROR: setuid(%d) failed.\n", (int)user->pw_uid);
-            optfree(opts);
-            return 1;
-        }
-    }
-#endif
-
     /* initialize logger */
     logg_lock    = !optget(opts, "LogFileUnlock")->enabled;
     logg_time    = optget(opts, "LogTime")->enabled;
@@ -250,30 +244,135 @@ int main(int argc, char **argv)
         logg_rotate = optget(opts, "LogRotate")->enabled;
     mprintf_send_timeout = optget(opts, "SendBufTimeout")->numarg;
 
-    do { /* logger initialized */
-        if ((opt = optget(opts, "LogFile"))->enabled) {
-            char timestr[32];
-            logg_file = opt->strarg;
-            if (!cli_is_abspath(logg_file)) {
-                fprintf(stderr, "ERROR: LogFile requires full path.\n");
-                ret = 1;
-                break;
-            }
-            time(&currtime);
-            if (logg("#+++ Started at %s", cli_ctime(&currtime, timestr, sizeof(timestr)))) {
-                fprintf(stderr, "ERROR: Can't initialize the internal logger\n");
-                ret = 1;
-                break;
-            }
-        } else {
-            logg_file = NULL;
+    if ((opt = optget(opts, "LogFile"))->enabled) {
+        char timestr[32];
+        logg_file = opt->strarg;
+        if (!cli_is_abspath(logg_file)) {
+            fprintf(stderr, "ERROR: LogFile requires full path.\n");
+            ret = 1;
+            return ret;
         }
+        time(&currtime);
+        if (logg(LOGG_INFO_NF, "+++ Started at %s", cli_ctime(&currtime, timestr, sizeof(timestr)))) {
+            fprintf(stderr, "ERROR: Can't initialize the internal logger\n");
+            ret = 1;
+            return ret;
+        }
+    } else {
+        logg_file = NULL;
+    }
+
+#ifndef WIN32
+    /* fork into background */
+    if (foreground == -1) {
+        if (optget(opts, "Foreground")->enabled) {
+            foreground = 1;
+        } else {
+            foreground = 0;
+        }
+    }
+    if (foreground == 0) {
+        int daemonizeRet = 0;
+#ifdef C_BSD
+        /* workaround for OpenBSD bug, see https://wwws.clamav.net/bugzilla/show_bug.cgi?id=885 */
+        for (ret = 0; (unsigned int)ret < nlsockets; ret++) {
+            if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) | O_NONBLOCK) == -1) {
+                logg(LOGG_ERROR, "fcntl for lsockets[] failed\n");
+                close(lsockets[ret]);
+                ret = 1;
+                break;
+            }
+        }
+#endif
+        gengine = engine;
+        atexit(free_engine);
+        daemonizeRet = daemonize_parent_wait(user_name, logg_file);
+        if (daemonizeRet < 0) {
+            logg(LOGG_ERROR, "daemonize() failed: %s\n", strerror(errno));
+            return 1;
+        }
+        gengine = NULL;
+#ifdef C_BSD
+        for (ret = 0; (unsigned int)ret < nlsockets; ret++) {
+            if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) & ~O_NONBLOCK) == -1) {
+                logg(LOGG_ERROR, "fcntl for lsockets[] failed\n");
+                close(lsockets[ret]);
+                ret = 1;
+                break;
+            }
+        }
+#endif
+    }
+
+#endif
+
+    /* save the PID */
+    mainpid = getpid();
+    if ((opt = optget(opts, "PidFile"))->enabled) {
+        FILE *fd;
+        old_umask = umask(0022);
+        if ((fd = fopen(opt->strarg, "w")) == NULL) {
+            // logg(LOGG_ERROR, "Can't save PID in file %s\n", opt->strarg);
+            logg(LOGG_ERROR, "Can't save PID to file %s: %s\n", opt->strarg, strerror(errno));
+            exit(2);
+        } else {
+            if (fprintf(fd, "%u\n", (unsigned int)mainpid) < 0) {
+                logg(LOGG_ERROR, "Can't save PID to file %s: %s\n", opt->strarg, strerror(errno));
+                // logg(LOGG_ERROR, "Can't save PID in file %s\n", opt->strarg);
+                fclose(fd);
+                exit(2);
+            }
+            fclose(fd);
+        }
+        umask(old_umask);
+
+#ifndef _WIN32
+        /*If the file has already been created by a different user, it will just be
+         * rewritten by us, but not change the ownership, so do that explicitly.
+         */
+        if (0 == geteuid()) {
+            struct passwd *pw = getpwuid(0);
+            int ret           = lchown(opt->strarg, pw->pw_uid, pw->pw_gid);
+            if (ret) {
+                logg(LOGG_ERROR, "Can't change ownership of PID file %s '%s'\n", opt->strarg, strerror(errno));
+                exit(2);
+            }
+        }
+#endif /* _WIN32 */
+    }
+
+#ifdef _WIN32
+
+    if (optget(opts, "install-service")->enabled) {
+        svc_install("clamd", "ClamAV ClamD",
+                    "Provides virus scanning facilities for ClamAV");
+        optfree(opts);
+        return 0;
+    }
+
+    if (optget(opts, "uninstall-service")->enabled) {
+        svc_uninstall("clamd", 1);
+        optfree(opts);
+        return 0;
+    }
+#endif
+
+    /* drop privileges */
+#ifndef _WIN32
+    dropPrivRet = drop_privileges(user_name, logg_file);
+    if (dropPrivRet) {
+        optfree(opts);
+        return dropPrivRet;
+    }
+#endif /* _WIN32 */
+
+    do { /* logger initialized */
 
         if (optget(opts, "DevLiblog")->enabled)
             cl_set_clcb_msg(msg_callback);
 
         if ((ret = cl_init(CL_INIT_DEFAULT))) {
-            logg("!Can't initialize libclamav: %s\n", cl_strerror(ret));
+            logg(LOGG_ERROR, "Can't initialize libclamav: %s\n", cl_strerror(ret));
             ret = 1;
             break;
         }
@@ -290,7 +389,7 @@ int main(int argc, char **argv)
 
             opt = optget(opts, "LogFacility");
             if ((fac = logg_facility(opt->strarg)) == -1) {
-                logg("!LogFacility: %s: No such facility.\n", opt->strarg);
+                logg(LOGG_ERROR, "LogFacility: %s: No such facility.\n", opt->strarg);
                 ret = 1;
                 break;
             }
@@ -314,54 +413,54 @@ int main(int argc, char **argv)
         if (optget(opts, "LocalSocket")->enabled)
             localsock = 1;
 
-        logg("#Received %d file descriptor(s) from systemd.\n", num_fd);
+        logg(LOGG_INFO_NF, "Received %d file descriptor(s) from systemd.\n", num_fd);
 
         if (!tcpsock && !localsock && num_fd == 0) {
-            logg("!Please define server type (local and/or TCP).\n");
+            logg(LOGG_ERROR, "Please define server type (local and/or TCP).\n");
             ret = 1;
             break;
         }
 
-        logg("#clamd daemon %s (OS: " TARGET_OS_TYPE ", ARCH: " TARGET_ARCH_TYPE ", CPU: " TARGET_CPU_TYPE ")\n", get_version());
+        logg(LOGG_INFO_NF, "clamd daemon %s (OS: " TARGET_OS_TYPE ", ARCH: " TARGET_ARCH_TYPE ", CPU: " TARGET_CPU_TYPE ")\n", get_version());
 
 #ifndef _WIN32
         if (user)
-            logg("#Running as user %s (UID %u, GID %u)\n", user->pw_name, user->pw_uid, user->pw_gid);
+            logg(LOGG_INFO_NF, "Running as user %s (UID %u, GID %u)\n", user->pw_name, user->pw_uid, user->pw_gid);
 #endif
 
 #if defined(RLIMIT_DATA) && defined(C_BSD)
         if (getrlimit(RLIMIT_DATA, &rlim) == 0) {
             /* bb #1941.
-            * On 32-bit FreeBSD if you set ulimit -d to >2GB then mmap() will fail
-            * too soon (after ~120 MB).
-            * Set limit lower than 2G if on 32-bit */
+             * On 32-bit FreeBSD if you set ulimit -d to >2GB then mmap() will fail
+             * too soon (after ~120 MB).
+             * Set limit lower than 2G if on 32-bit */
             uint64_t lim = rlim.rlim_cur;
             if (sizeof(void *) == 4 &&
                 lim > (1ULL << 31)) {
                 rlim.rlim_cur = 1ULL << 31;
                 if (setrlimit(RLIMIT_DATA, &rlim) < 0)
-                    logg("!setrlimit(RLIMIT_DATA) failed: %s\n", strerror(errno));
+                    logg(LOGG_ERROR, "setrlimit(RLIMIT_DATA) failed: %s\n", strerror(errno));
                 else
-                    logg("Running on 32-bit system, and RLIMIT_DATA > 2GB, lowering to 2GB!\n");
+                    logg(LOGG_INFO, "Running on 32-bit system, and RLIMIT_DATA > 2GB, lowering to 2GB!\n");
             }
         }
 #endif
 
         if (logg_size)
-            logg("#Log file size limited to %lld bytes.\n", (long long int)logg_size);
+            logg(LOGG_INFO_NF, "Log file size limited to %lld bytes.\n", (long long int)logg_size);
         else
-            logg("#Log file size limit disabled.\n");
+            logg(LOGG_INFO_NF, "Log file size limit disabled.\n");
 
         min_port = optget(opts, "StreamMinPort")->numarg;
         max_port = optget(opts, "StreamMaxPort")->numarg;
         if (min_port < 1024 || min_port > max_port || max_port > 65535) {
-            logg("!Invalid StreamMinPort/StreamMaxPort: %d, %d\n", min_port, max_port);
+            logg(LOGG_ERROR, "Invalid StreamMinPort/StreamMaxPort: %d, %d\n", min_port, max_port);
             ret = 1;
             break;
         }
 
         if (!(engine = cl_engine_new())) {
-            logg("!Can't initialize antivirus engine\n");
+            logg(LOGG_ERROR, "Can't initialize antivirus engine\n");
             ret = 1;
             break;
         }
@@ -371,7 +470,7 @@ int main(int argc, char **argv)
 
         /* load the database(s) */
         dbdir = optget(opts, "DatabaseDirectory")->strarg;
-        logg("#Reading databases from %s\n", dbdir);
+        logg(LOGG_INFO_NF, "Reading databases from %s\n", dbdir);
 
         if (optget(opts, "DetectPUA")->enabled) {
             dboptions |= CL_DB_PUA;
@@ -379,17 +478,17 @@ int main(int argc, char **argv)
             if ((opt = optget(opts, "ExcludePUA"))->enabled) {
                 dboptions |= CL_DB_PUA_EXCLUDE;
                 i = 0;
-                logg("#Excluded PUA categories:");
+                logg(LOGG_INFO_NF, "Excluded PUA categories:");
 
                 while (opt) {
                     if (!(pua_cats = realloc(pua_cats, i + strlen(opt->strarg) + 3))) {
-                        logg("!Can't allocate memory for pua_cats\n");
+                        logg(LOGG_ERROR, "Can't allocate memory for pua_cats\n");
                         cl_engine_free(engine);
                         ret = 1;
                         break;
                     }
 
-                    logg("# %s", opt->strarg);
+                    logg(LOGG_INFO_NF, " %s", opt->strarg);
 
                     sprintf(pua_cats + i, ".%s", opt->strarg);
                     i += strlen(opt->strarg) + 1;
@@ -400,14 +499,14 @@ int main(int argc, char **argv)
                 if (ret)
                     break;
 
-                logg("#\n");
+                logg(LOGG_INFO_NF, "\n");
                 pua_cats[i]     = '.';
                 pua_cats[i + 1] = 0;
             }
 
             if ((opt = optget(opts, "IncludePUA"))->enabled) {
                 if (pua_cats) {
-                    logg("!ExcludePUA and IncludePUA cannot be used at the same time\n");
+                    logg(LOGG_ERROR, "ExcludePUA and IncludePUA cannot be used at the same time\n");
                     free(pua_cats);
                     ret = 1;
                     break;
@@ -415,15 +514,15 @@ int main(int argc, char **argv)
 
                 dboptions |= CL_DB_PUA_INCLUDE;
                 i = 0;
-                logg("#Included PUA categories:");
+                logg(LOGG_INFO_NF, "Included PUA categories:");
                 while (opt) {
                     if (!(pua_cats = realloc(pua_cats, i + strlen(opt->strarg) + 3))) {
-                        logg("!Can't allocate memory for pua_cats\n");
+                        logg(LOGG_ERROR, "Can't allocate memory for pua_cats\n");
                         ret = 1;
                         break;
                     }
 
-                    logg("# %s", opt->strarg);
+                    logg(LOGG_INFO_NF, " %s", opt->strarg);
 
                     sprintf(pua_cats + i, ".%s", opt->strarg);
                     i += strlen(opt->strarg) + 1;
@@ -434,14 +533,14 @@ int main(int argc, char **argv)
                 if (ret)
                     break;
 
-                logg("#\n");
+                logg(LOGG_INFO_NF, "\n");
                 pua_cats[i]     = '.';
                 pua_cats[i + 1] = 0;
             }
 
             if (pua_cats) {
                 if ((ret = cl_engine_set_str(engine, CL_ENGINE_PUA_CATEGORIES, pua_cats))) {
-                    logg("!cli_engine_set_str(CL_ENGINE_PUA_CATEGORIES) failed: %s\n", cl_strerror(ret));
+                    logg(LOGG_ERROR, "cli_engine_set_str(CL_ENGINE_PUA_CATEGORIES) failed: %s\n", cl_strerror(ret));
                     free(pua_cats);
                     ret = 1;
                     break;
@@ -449,18 +548,18 @@ int main(int argc, char **argv)
                 free(pua_cats);
             }
         } else {
-            logg("#Not loading PUA signatures.\n");
+            logg(LOGG_INFO_NF, "Not loading PUA signatures.\n");
         }
 
         if (optget(opts, "OfficialDatabaseOnly")->enabled) {
             dboptions |= CL_DB_OFFICIAL_ONLY;
-            logg("#Only loading official signatures.\n");
+            logg(LOGG_INFO_NF, "Only loading official signatures.\n");
         }
 
         /* set the temporary dir */
         if ((opt = optget(opts, "TemporaryDirectory"))->enabled) {
             if ((ret = cl_engine_set_str(engine, CL_ENGINE_TMPDIR, opt->strarg))) {
-                logg("!cli_engine_set_str(CL_ENGINE_TMPDIR) failed: %s\n", cl_strerror(ret));
+                logg(LOGG_ERROR, "cli_engine_set_str(CL_ENGINE_TMPDIR) failed: %s\n", cl_strerror(ret));
                 ret = 1;
                 break;
             }
@@ -479,7 +578,7 @@ int main(int argc, char **argv)
         if (optget(opts, "PhishingSignatures")->enabled)
             dboptions |= CL_DB_PHISHING;
         else
-            logg("#Not loading phishing signatures.\n");
+            logg(LOGG_INFO_NF, "Not loading phishing signatures.\n");
 
         if (optget(opts, "Bytecode")->enabled) {
             dboptions |= CL_DB_BYTECODE;
@@ -488,26 +587,26 @@ int main(int argc, char **argv)
 
                 if (!strcmp(opt->strarg, "TrustSigned")) {
                     s = CL_BYTECODE_TRUST_SIGNED;
-                    logg("#Bytecode: Security mode set to \"TrustSigned\".\n");
+                    logg(LOGG_INFO_NF, "Bytecode: Security mode set to \"TrustSigned\".\n");
                 } else if (!strcmp(opt->strarg, "Paranoid")) {
                     s = CL_BYTECODE_TRUST_NOTHING;
-                    logg("#Bytecode: Security mode set to \"Paranoid\".\n");
+                    logg(LOGG_INFO_NF, "Bytecode: Security mode set to \"Paranoid\".\n");
                 } else {
-                    logg("!Unable to parse bytecode security setting:%s\n",
+                    logg(LOGG_ERROR, "Unable to parse bytecode security setting:%s\n",
                          opt->strarg);
                     ret = 1;
                     break;
                 }
 
                 if ((ret = cl_engine_set_num(engine, CL_ENGINE_BYTECODE_SECURITY, s))) {
-                    logg("^Invalid bytecode security setting %s: %s\n", opt->strarg, cl_strerror(ret));
+                    logg(LOGG_WARNING, "Invalid bytecode security setting %s: %s\n", opt->strarg, cl_strerror(ret));
                     ret = 1;
                     break;
                 }
             }
             if ((opt = optget(opts, "BytecodeUnsigned"))->enabled) {
                 dboptions |= CL_DB_BYTECODE_UNSIGNED;
-                logg("#Bytecode: Enabled support for unsigned bytecode.\n");
+                logg(LOGG_INFO_NF, "Bytecode: Enabled support for unsigned bytecode.\n");
             }
 
             if ((opt = optget(opts, "BytecodeMode"))->enabled) {
@@ -528,32 +627,39 @@ int main(int argc, char **argv)
                 cl_engine_set_num(engine, CL_ENGINE_BYTECODE_TIMEOUT, opt->numarg);
             }
         } else {
-            logg("#Bytecode support disabled.\n");
+            logg(LOGG_INFO_NF, "Bytecode support disabled.\n");
         }
 
         if (optget(opts, "PhishingScanURLs")->enabled)
             dboptions |= CL_DB_PHISHING_URLS;
         else
-            logg("#Disabling URL based phishing detection.\n");
+            logg(LOGG_INFO_NF, "Disabling URL based phishing detection.\n");
 
         if (optget(opts, "DevACOnly")->enabled) {
-            logg("#Only using the A-C matcher.\n");
+            logg(LOGG_INFO_NF, "Only using the A-C matcher.\n");
             cl_engine_set_num(engine, CL_ENGINE_AC_ONLY, 1);
         }
 
         if ((opt = optget(opts, "DevACDepth"))->enabled) {
             cl_engine_set_num(engine, CL_ENGINE_AC_MAXDEPTH, opt->numarg);
-            logg("#Max A-C depth set to %u\n", (unsigned int)opt->numarg);
+            logg(LOGG_INFO_NF, "Max A-C depth set to %u\n", (unsigned int)opt->numarg);
         }
 
+#ifdef _WIN32
+        if (optget(opts, "daemon")->enabled) {
+            cl_engine_set_clcb_sigload(engine, svc_checkpoint, NULL);
+            svc_register("clamd");
+        }
+#endif
+
         if ((ret = cl_load(dbdir, engine, &sigs, dboptions))) {
-            logg("!%s\n", cl_strerror(ret));
+            logg(LOGG_ERROR, "%s\n", cl_strerror(ret));
             ret = 1;
             break;
         }
 
-        if ((ret = statinidir_th(dbdir))) {
-            logg("!%s\n", cl_strerror(ret));
+        if ((ret = statinidir(dbdir))) {
+            logg(LOGG_ERROR, "%s\n", cl_strerror(ret));
             ret = 1;
             break;
         }
@@ -561,12 +667,12 @@ int main(int argc, char **argv)
         if (optget(opts, "DisableCertCheck")->enabled)
             cl_engine_set_num(engine, CL_ENGINE_DISABLE_PE_CERTS, 1);
 
-        logg("#Loaded %u signatures.\n", sigs);
+        logg(LOGG_INFO_NF, "Loaded %u signatures.\n", sigs);
 
         /* pcre engine limits - required for cl_engine_compile */
         if ((opt = optget(opts, "PCREMatchLimit"))->active) {
             if ((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_MATCH_LIMIT, opt->numarg))) {
-                logg("!cli_engine_set_num(PCREMatchLimit) failed: %s\n", cl_strerror(ret));
+                logg(LOGG_ERROR, "cli_engine_set_num(PCREMatchLimit) failed: %s\n", cl_strerror(ret));
                 cl_engine_free(engine);
                 return 1;
             }
@@ -574,14 +680,14 @@ int main(int argc, char **argv)
 
         if ((opt = optget(opts, "PCRERecMatchLimit"))->active) {
             if ((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_RECMATCH_LIMIT, opt->numarg))) {
-                logg("!cli_engine_set_num(PCRERecMatchLimit) failed: %s\n", cl_strerror(ret));
+                logg(LOGG_ERROR, "cli_engine_set_num(PCRERecMatchLimit) failed: %s\n", cl_strerror(ret));
                 cl_engine_free(engine);
                 return 1;
             }
         }
 
         if ((ret = cl_engine_compile(engine)) != 0) {
-            logg("!Database initialization error: %s\n", cl_strerror(ret));
+            logg(LOGG_ERROR, "Database initialization error: %s\n", cl_strerror(ret));
             ret = 1;
             break;
         }
@@ -639,7 +745,7 @@ int main(int argc, char **argv)
                     struct group *pgrp = getgrnam(gname);
 
                     if (!pgrp) {
-                        logg("!Unknown group %s\n", gname);
+                        logg(LOGG_ERROR, "Unknown group %s\n", gname);
                         ret = 1;
                         break;
                     }
@@ -647,7 +753,7 @@ int main(int argc, char **argv)
                     sock_gid = pgrp->gr_gid;
                 }
                 if (chown(optget(opts, "LocalSocket")->strarg, -1, sock_gid)) {
-                    logg("!Failed to change socket ownership to group %s\n", gname);
+                    logg(LOGG_ERROR, "Failed to change socket ownership to group %s\n", gname);
                     ret = 1;
                     break;
                 }
@@ -658,7 +764,7 @@ int main(int argc, char **argv)
                 sock_mode = strtol(optget(opts, "LocalSocketMode")->strarg, &end, 8);
 
                 if (*end) {
-                    logg("!Invalid LocalSocketMode %s\n", optget(opts, "LocalSocketMode")->strarg);
+                    logg(LOGG_ERROR, "Invalid LocalSocketMode %s\n", optget(opts, "LocalSocketMode")->strarg);
                     ret = 1;
                     break;
                 }
@@ -667,7 +773,7 @@ int main(int argc, char **argv)
             }
 
             if (chmod(optget(opts, "LocalSocket")->strarg, sock_mode & 0666)) {
-                logg("!Cannot set socket permission to %s\n", optget(opts, "LocalSocketMode")->strarg);
+                logg(LOGG_ERROR, "Cannot set socket permission for %s to %3o\n", optget(opts, "LocalSocket")->strarg, sock_mode & 0666);
                 ret = 1;
                 break;
             }
@@ -694,62 +800,45 @@ int main(int argc, char **argv)
             }
         }
 
-        /* fork into background */
-        if (foreground == -1) {
-            if (optget(opts, "Foreground")->enabled) {
-                foreground = 1;
-            } else {
-                foreground = 0;
+        if (0 == foreground) {
+            if (!debug_mode) {
+                if (chdir("/") == -1) {
+                    logg(LOGG_WARNING, "Can't change current working directory to root\n");
+                }
             }
+
+#ifndef _WIN32
+
+            /*Since some of the logging is written to stderr, and some of it
+             * is written to a log file, close stdin, stderr, and stdout
+             * now, since everything is initialized.*/
+
+            /*signal the parent process.*/
+            if (parentPid != getpid()) {
+                daemonize_signal_parent(parentPid);
+            }
+#endif
         }
-        if (foreground == 0) {
-#ifdef C_BSD
-            /* workaround for OpenBSD bug, see https://wwws.clamav.net/bugzilla/show_bug.cgi?id=885 */
-            for (ret = 0; (unsigned int)ret < nlsockets; ret++) {
-                if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) | O_NONBLOCK) == -1) {
-                    logg("!fcntl for lsockets[] failed\n");
-                    close(lsockets[ret]);
-                    ret = 1;
-                    break;
-                }
-            }
-#endif
-            gengine = engine;
-            atexit(free_engine);
-            if (daemonize() == -1) {
-                logg("!daemonize() failed: %s\n", strerror(errno));
-                ret = 1;
-                break;
-            }
-            gengine = NULL;
-#ifdef C_BSD
-            for (ret = 0; (unsigned int)ret < nlsockets; ret++) {
-                if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) & ~O_NONBLOCK) == -1) {
-                    logg("!fcntl for lsockets[] failed\n");
-                    close(lsockets[ret]);
-                    ret = 1;
-                    break;
-                }
-            }
-#endif
-            if (!debug_mode)
-                if (chdir("/") == -1)
-                    logg("^Can't change current working directory to root\n");
+
+#elif defined(_WIN32)
+        if (optget(opts, "service-mode")->enabled) {
+            cl_engine_set_clcb_sigload(engine, NULL, NULL);
+            svc_ready();
         }
 #endif
 
         if (nlsockets == 0) {
-            logg("!Not listening on any interfaces\n");
+            logg(LOGG_ERROR, "Not listening on any interfaces\n");
             ret = 1;
             break;
         }
 
-        ret = recvloop_th(lsockets, nlsockets, engine, dboptions, opts);
+        ret = recvloop(lsockets, nlsockets, engine, dboptions, opts);
 
     } while (0);
 
     if (num_fd == 0) {
-        logg("*Closing the main socket%s.\n", (nlsockets > 1) ? "s" : "");
+        logg(LOGG_DEBUG, "Closing the main socket%s.\n", (nlsockets > 1) ? "s" : "");
 
         for (i = 0; i < nlsockets; i++) {
             closesocket(lsockets[i]);
@@ -759,9 +848,9 @@ int main(int argc, char **argv)
             opt = optget(opts, "LocalSocket");
 
             if (unlink(opt->strarg) == -1)
-                logg("!Can't unlink the socket file %s\n", opt->strarg);
+                logg(LOGG_ERROR, "Can't unlink the socket file %s\n", opt->strarg);
             else
-                logg("Socket file removed.\n");
+                logg(LOGG_INFO, "Socket file removed.\n");
         }
 #endif
     }
