@@ -73,7 +73,7 @@
 #include "scanners.h"
 #include "others.h"
 
-// shared
+// common
 #include "optparser.h"
 #include "output.h"
 #include "misc.h"
@@ -93,7 +93,7 @@ void virusaction(const char *filename, const char *virname,
                  const struct optstruct *opts)
 {
     if (optget(opts, "VirusEvent")->enabled)
-        logg("^VirusEvent is not supported on this platform"); /* Yet */
+        logg(LOGG_WARNING, "VirusEvent is not supported on this platform"); /* Yet */
 }
 
 #else
@@ -101,6 +101,8 @@ void virusaction(const char *filename, const char *virname,
 #define VE_FILENAME "CLAM_VIRUSEVENT_FILENAME"
 #define VE_VIRUSNAME "CLAM_VIRUSEVENT_VIRUSNAME"
 
+#define FILENAME_DISABLED_MESSAGE "The filename format character has been disabled due to security concerns, use the 'CLAM_VIRUSEVENT_FILENAME' environment variable instead."
+
 void virusaction(const char *filename, const char *virname,
                  const struct optstruct *opts)
 {
@@ -108,7 +110,7 @@ void virusaction(const char *filename, const char *virname,
     const struct optstruct *opt;
     char *buffer_file, *buffer_vir, *buffer_cmd, *path;
     const char *pt;
-    size_t i, j, v = 0, len;
+    size_t i, j, v = 0, f = 0, len;
     char *env[4];
 
     if (!(opt = optget(opts, "VirusEvent"))->enabled)
@@ -138,9 +140,14 @@ void virusaction(const char *filename, const char *virname,
         pt += 2;
         v++;
     }
+    pt = opt->strarg;
+    while ((pt = strstr(pt, "%f"))) {
+        pt += 2;
+        f++;
+    }
     len = strlen(opt->strarg);
     buffer_cmd =
-        (char *)calloc(len + v * strlen(virname) + 1, sizeof(char));
+        (char *)calloc(len + v * strlen(virname) + f * strlen(FILENAME_DISABLED_MESSAGE) + 1, sizeof(char));
     if (!buffer_cmd) {
         if (path)
             xfree(env[0]);
@@ -154,6 +161,10 @@ void virusaction(const char *filename, const char *virname,
             strcat(buffer_cmd, virname);
             j += strlen(virname);
             i++;
+        } else if (i + 1 < len && opt->strarg[i] == '%' && opt->strarg[i + 1] == 'f') {
+            strcat(buffer_cmd, FILENAME_DISABLED_MESSAGE);
+            j += strlen(FILENAME_DISABLED_MESSAGE);
+            i++;
         } else {
             buffer_cmd[j++] = opt->strarg[i];
         }
@@ -169,7 +180,7 @@ void virusaction(const char *filename, const char *virname,
         while (waitpid(pid, NULL, 0) == -1 && errno == EINTR) continue;
     } else {
         pthread_mutex_unlock(&virusaction_lock);
-        logg("!VirusEvent: fork failed.\n");
+        logg(LOGG_ERROR, "VirusEvent: fork failed.\n");
     }
     if (path)
         xfree(env[0]);
@@ -181,7 +192,7 @@ void virusaction(const char *filename, const char *virname,
 #endif /* _WIN32 */
 
 /* Function: writen
-	Try hard to write the specified number of bytes
+        Try hard to write the specified number of bytes
 */
 int writen(int fd, void *buff, unsigned int count)
 {
@@ -217,7 +228,7 @@ realloc_polldata(struct fd_data *data)
         free(data->poll_data);
     data->poll_data = malloc(data->nfds * sizeof(*data->poll_data));
     if (!data->poll_data) {
-        logg("!realloc_polldata: Memory allocation failed for poll_data\n");
+        logg(LOGG_ERROR, "realloc_polldata: Memory allocation failed for poll_data\n");
         return -1;
     }
     data->poll_data_nfds = data->nfds;
@@ -259,7 +270,7 @@ void fds_cleanup(struct fd_data *data)
     for (i = j; i < data->nfds; i++)
         data->buf[i].fd = -1;
     data->nfds = j;
-    logg("$Number of file descriptors polled: %u fds\n",
+    logg(LOGG_DEBUG_NV, "Number of file descriptors polled: %u fds\n",
          (unsigned)data->nfds);
     /* Shrink buffer */
     newbuf = realloc(data->buf, j * sizeof(*newbuf));
@@ -282,9 +293,9 @@ read_fd_data(struct fd_buf *buf)
         return -1;
 
         /* Read the pending packet, it may contain more than one command, but
-     * that is to the cmdparser to handle.
-     * It will handle 1st command, and then move leftover to beginning of buffer
-     */
+         * that is to the cmdparser to handle.
+         * It will handle 1st command, and then move leftover to beginning of buffer
+         */
 #ifdef HAVE_FD_PASSING
     {
         struct msghdr msg;
@@ -296,7 +307,7 @@ read_fd_data(struct fd_buf *buf)
         struct iovec iov[1];
 
         if (buf->recvfd != -1) {
-            logg("$Closing unclaimed FD: %d\n", buf->recvfd);
+            logg(LOGG_DEBUG_NV, "Closing unclaimed FD: %d\n", buf->recvfd);
             close(buf->recvfd);
             buf->recvfd = -1;
         }
@@ -312,18 +323,18 @@ read_fd_data(struct fd_buf *buf)
         if (n < 0)
             return -1;
         if (msg.msg_flags & MSG_TRUNC) {
-            logg("^Message truncated at %d bytes\n", (int)n);
+            logg(LOGG_WARNING, "Message truncated at %d bytes\n", (int)n);
             return -1;
         }
         if (msg.msg_flags & MSG_CTRUNC) {
             if (msg.msg_controllen > 0)
-                logg("^Control message truncated at %d bytes, %d data read\n", (int)msg.msg_controllen, (int)n);
+                logg(LOGG_WARNING, "Control message truncated at %d bytes, %d data read\n", (int)msg.msg_controllen, (int)n);
             else
-                logg("^Control message truncated, no control data received, %d bytes read"
+                logg(LOGG_WARNING, "Control message truncated, no control data received, %d bytes read"
 #ifdef C_LINUX
-                     "(Is SELinux/AppArmor enabled, and blocking file descriptor passing?)"
+                                   "(Is SELinux/AppArmor enabled, and blocking file descriptor passing?)"
 #endif
-                     "\n",
+                                   "\n",
                      (int)n);
             return -1;
         }
@@ -334,11 +345,11 @@ read_fd_data(struct fd_buf *buf)
                     cmsg->cmsg_level == SOL_SOCKET &&
                     cmsg->cmsg_type == SCM_RIGHTS) {
                     if (buf->recvfd != -1) {
-                        logg("$Unclaimed file descriptor received. closing: %d\n", buf->recvfd);
+                        logg(LOGG_DEBUG_NV, "Unclaimed file descriptor received. closing: %d\n", buf->recvfd);
                         close(buf->recvfd);
                     }
                     buf->recvfd = *(int *)CMSG_DATA(cmsg);
-                    logg("$Receveived a file descriptor: %d\n", buf->recvfd);
+                    logg(LOGG_DEBUG_NV, "Receveived a file descriptor: %d\n", buf->recvfd);
                 }
             }
         }
@@ -372,7 +383,7 @@ buf_init(struct fd_buf *buf, int listen_only, int timeout)
             /* plus extra space for a \0 so we can make sure every command is \0
              * terminated */
             if (!(buf->buffer = malloc(buf->bufsize + 1))) {
-                logg("!add_fd: Memory allocation failed for command buffer\n");
+                logg(LOGG_ERROR, "add_fd: Memory allocation failed for command buffer\n");
                 return -1;
             }
         }
@@ -396,7 +407,7 @@ int fds_add(struct fd_data *data, int fd, int listen_only, int timeout)
     struct fd_buf *buf;
     unsigned n;
     if (fd < 0) {
-        logg("!add_fd: invalid fd passed to add_fd\n");
+        logg(LOGG_ERROR, "add_fd: invalid fd passed to add_fd\n");
         return -1;
     }
     /* we may already have this fd, if
@@ -412,7 +423,7 @@ int fds_add(struct fd_data *data, int fd, int listen_only, int timeout)
     n++;
     buf = realloc(data->buf, n * sizeof(*buf));
     if (!buf) {
-        logg("!add_fd: Memory allocation failed for fd_buf\n");
+        logg(LOGG_ERROR, "add_fd: Memory allocation failed for fd_buf\n");
         return -1;
     }
     data->buf               = buf;
@@ -507,7 +518,7 @@ int fds_poll_recv(struct fd_data *data, int timeout, int check_signals,
     else
         timeout = -1;
     if (timeout > 0)
-        logg("$fds_poll_recv: timeout after %d seconds\n", timeout);
+        logg(LOGG_DEBUG_NV, "fds_poll_recv: timeout after %d seconds\n", timeout);
 #ifdef HAVE_POLL
     /* Use poll() if available, preferred because:
      *  - can poll any number of FDs
@@ -549,12 +560,12 @@ int fds_poll_recv(struct fd_data *data, int timeout, int check_signals,
                     continue;
                 if (data->buf[i].fd != data->poll_data[i].fd) {
                     /* should never happen */
-                    logg("!poll_recv_fds FD mismatch\n");
+                    logg(LOGG_ERROR, "poll_recv_fds FD mismatch\n");
                     continue;
                 }
                 revents = data->poll_data[i].revents;
                 if (revents & (POLLIN | POLLHUP)) {
-                    logg("$Received POLLIN|POLLHUP on fd %d\n",
+                    logg(LOGG_DEBUG_NV, "Received POLLIN|POLLHUP on fd %d\n",
                          data->poll_data[i].fd);
                 }
 #ifndef _WIN32
@@ -577,11 +588,11 @@ int fds_poll_recv(struct fd_data *data, int timeout, int check_signals,
                 if (revents & (POLLHUP | POLLERR | POLLNVAL)) {
                     if (revents & (POLLHUP | POLLNVAL)) {
                         /* remote disconnected */
-                        logg("*Client disconnected (FD %d)\n",
+                        logg(LOGG_DEBUG, "Client disconnected (FD %d)\n",
                              data->poll_data[i].fd);
                     } else {
                         /* error on file descriptor */
-                        logg("^Error condition on fd %d\n",
+                        logg(LOGG_WARNING, "Error condition on fd %d\n",
                              data->poll_data[i].fd);
                     }
                     data->buf[i].got_newdata = -1;
@@ -600,7 +611,7 @@ int fds_poll_recv(struct fd_data *data, int timeout, int check_signals,
         for (i = 0; i < data->nfds; i++) {
             int fd = data->buf[i].fd;
             if (fd >= FD_SETSIZE) {
-                logg("!File descriptor is too high for FD_SET\n");
+                logg(LOGG_ERROR, "File descriptor is too high for FD_SET\n");
                 return -1;
             }
 
@@ -633,14 +644,14 @@ int fds_poll_recv(struct fd_data *data, int timeout, int check_signals,
                         int ret = read_fd_data(&data->buf[i]);
                         if (ret == -1 || !ret) {
                             if (ret == -1)
-                                logg("!Error condition on fd %d\n",
+                                logg(LOGG_ERROR, "Error condition on fd %d\n",
                                      data->buf[i].fd);
                             else {
                                 /* avoid SHUT_WR problem on Mac OS X */
                                 int ret = send(data->buf[i].fd, &i, 0, 0);
                                 if (!ret || (ret == -1 && errno == EINTR))
                                     continue;
-                                logg("*Client disconnected\n");
+                                logg(LOGG_DEBUG, "Client disconnected\n");
                             }
                             data->buf[i].got_newdata = -1;
                         }
@@ -680,10 +691,10 @@ int fds_poll_recv(struct fd_data *data, int timeout, int check_signals,
     if (retval == -1 && errno != EINTR) {
         char err[128];
 #ifdef HAVE_POLL
-        logg("!poll_recv_fds: poll failed: %s\n",
+        logg(LOGG_ERROR, "poll_recv_fds: poll failed: %s\n",
              cli_strerror(errno, err, sizeof(err)));
 #else
-        logg("!poll_recv_fds: select failed: %s\n",
+        logg(LOGG_ERROR, "poll_recv_fds: select failed: %s\n",
              cli_strerror(errno, err, sizeof(err)));
 #endif
     }
