@@ -38,7 +38,7 @@
 // libclamav
 #include "clamav.h"
 
-// shared
+// common
 #include "output.h"
 #include "misc.h"
 #include "optparser.h"
@@ -73,7 +73,7 @@ int main(int argc, char **argv)
 #endif
 
     if ((opts = optparse(NULL, argc, argv, 1, OPT_CLAMDSCAN, OPT_CLAMSCAN, NULL)) == NULL) {
-        mprintf("!Can't parse command line options\n");
+        mprintf(LOGG_ERROR, "Can't parse command line options\n");
         exit(2);
     }
 
@@ -83,7 +83,7 @@ int main(int argc, char **argv)
     }
 
     if ((clamdopts = optparse(optget(opts, "config-file")->strarg, 0, NULL, 1, OPT_CLAMD, 0, NULL)) == NULL) {
-        logg("!Can't parse clamd configuration file %s\n", optget(opts, "config-file")->strarg);
+        logg(LOGG_ERROR, "Can't parse clamd configuration file %s\n", optget(opts, "config-file")->strarg);
         optfree(opts);
         exit(2);
     }
@@ -131,8 +131,8 @@ int main(int argc, char **argv)
 
     if ((opt = optget(opts, "log"))->enabled) {
         logg_file = opt->strarg;
-        if (logg("--------------------------------------\n")) {
-            mprintf("!Problem with internal logger.\n");
+        if (logg(LOGG_INFO, "--------------------------------------\n")) {
+            mprintf(LOGG_ERROR, "Problem with internal logger.\n");
             optfree(opts);
             optfree(clamdopts);
             exit(2);
@@ -179,37 +179,37 @@ int main(int argc, char **argv)
         dms = t2.tv_usec - t1.tv_usec;
         ds -= (dms < 0) ? (1) : (0);
         dms += (dms < 0) ? (1000000) : (0);
-        logg("\n----------- SCAN SUMMARY -----------\n");
-        logg("Infected files: %d\n", infected);
+        logg(LOGG_INFO, "\n----------- SCAN SUMMARY -----------\n");
+        logg(LOGG_INFO, "Infected files: %d\n", infected);
         if (err)
-            logg("Total errors: %d\n", err);
+            logg(LOGG_INFO, "Total errors: %d\n", err);
         if (notremoved) {
-            logg("Not removed: %d\n", notremoved);
+            logg(LOGG_INFO, "Not removed: %d\n", notremoved);
         }
         if (notmoved) {
-            logg("Not moved: %d\n", notmoved);
+            logg(LOGG_INFO, "Not moved: %d\n", notmoved);
         }
-        logg("Time: %d.%3.3d sec (%d m %d s)\n", ds, dms / 1000, ds / 60, ds % 60);
+        logg(LOGG_INFO, "Time: %d.%3.3d sec (%d m %d s)\n", ds, dms / 1000, ds / 60, ds % 60);
 
 #ifdef _WIN32
         if (0 != localtime_s(&tmp, &date_start)) {
 #else
         if (!localtime_r(&date_start, &tmp)) {
 #endif
-            logg("!Failed to get local time for Start Date.\n");
+            logg(LOGG_ERROR, "Failed to get local time for Start Date.\n");
         }
         strftime(buffer, sizeof(buffer), "%Y:%m:%d %H:%M:%S", &tmp);
-        logg("Start Date: %s\n", buffer);
+        logg(LOGG_INFO, "Start Date: %s\n", buffer);
 
 #ifdef _WIN32
         if (0 != localtime_s(&tmp, &date_end)) {
 #else
         if (!localtime_r(&date_end, &tmp)) {
 #endif
-            logg("!Failed to get local time for End Date.\n");
+            logg(LOGG_ERROR, "Failed to get local time for End Date.\n");
         }
         strftime(buffer, sizeof(buffer), "%Y:%m:%d %H:%M:%S", &tmp);
-        logg("End Date:   %s\n", buffer);
+        logg(LOGG_INFO, "End Date:   %s\n", buffer);
     }
 
     logg_close();
@@ -222,35 +222,40 @@ void help(void)
 {
     mprintf_stdout = 1;
 
-    mprintf("\n");
-    mprintf("                      Clam AntiVirus: Daemon Client %s\n", get_version());
-    mprintf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-    mprintf("           (C) 2022 Cisco Systems, Inc.\n");
-    mprintf("\n");
-    mprintf("    clamdscan [options] [file/directory/-]\n");
-    mprintf("\n");
-    mprintf("    --help              -h             Show this help\n");
-    mprintf("    --version           -V             Print version number and exit\n");
-    mprintf("    --verbose           -v             Be verbose\n");
-    mprintf("    --quiet                            Be quiet, only output error messages\n");
-    mprintf("    --stdout                           Write to stdout instead of stderr. Does not affect 'debug' messages.\n");
-    mprintf("                                       (this help is always written to stdout)\n");
-    mprintf("    --log=FILE          -l FILE        Save scan report in FILE\n");
-    mprintf("    --file-list=FILE    -f FILE        Scan files from FILE\n");
-    mprintf("    --ping              -p A[:I]       Ping clamd up to [A] times at optional interval [I] until it responds.\n");
-    mprintf("    --wait              -w             Wait up to 30 seconds for clamd to start. Optionally use alongside --ping to set attempts [A] and interval [I] to check clamd.\n");
-    mprintf("    --remove                           Remove infected files. Be careful!\n");
-    mprintf("    --move=DIRECTORY                   Move infected files into DIRECTORY\n");
-    mprintf("    --copy=DIRECTORY                   Copy infected files into DIRECTORY\n");
-    mprintf("    --config-file=FILE                 Read configuration from FILE.\n");
-    mprintf("    --allmatch            -z           Continue scanning within file after finding a match.\n");
-    mprintf("    --multiscan           -m           Force MULTISCAN mode\n");
-    mprintf("    --infected            -i           Only print infected files\n");
-    mprintf("    --no-summary                       Disable summary at end of scanning\n");
-    mprintf("    --reload                           Request clamd to reload virus database\n");
-    mprintf("    --fdpass                           Pass filedescriptor to clamd (useful if clamd is running as a different user)\n");
-    mprintf("    --stream                           Force streaming files to clamd (for debugging and unit testing)\n");
-    mprintf("\n");
+    mprintf(LOGG_INFO, "\n");
+    mprintf(LOGG_INFO, "                      Clam AntiVirus: Daemon Client %s\n", get_version());
+    mprintf(LOGG_INFO, "           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
+    mprintf(LOGG_INFO, "           (C) 2022 Cisco Systems, Inc.\n");
+    mprintf(LOGG_INFO, "\n");
+    mprintf(LOGG_INFO, "    clamdscan [options] [file/directory/-]\n");
+    mprintf(LOGG_INFO, "\n");
+    mprintf(LOGG_INFO, "    --help              -h             Show this help\n");
+    mprintf(LOGG_INFO, "    --version           -V             Print version number and exit\n");
+    mprintf(LOGG_INFO, "    --verbose           -v             Be verbose\n");
+    mprintf(LOGG_INFO, "    --quiet                            Be quiet, only output error messages\n");
+    mprintf(LOGG_INFO, "    --stdout                           Write to stdout instead of stderr. Does not affect 'debug' messages.\n");
+    mprintf(LOGG_INFO, "                                       (this help is always written to stdout)\n");
+    mprintf(LOGG_INFO, "    --log=FILE          -l FILE        Save scan report in FILE\n");
+    mprintf(LOGG_INFO, "    --file-list=FILE    -f FILE        Scan files from FILE\n");
+    mprintf(LOGG_INFO, "    --ping              -p A[:I]       Ping clamd up to [A] times at optional interval [I] until it responds.\n");
+    mprintf(LOGG_INFO, "    --wait              -w             Wait up to 30 seconds for clamd to start. Optionally use alongside --ping to set attempts [A] and interval [I] to check clamd.\n");
+    mprintf(LOGG_INFO, "    --remove                           Remove infected files. Be careful!\n");
+    mprintf(LOGG_INFO, "    --move=DIRECTORY                   Move infected files into DIRECTORY\n");
+    mprintf(LOGG_INFO, "    --copy=DIRECTORY                   Copy infected files into DIRECTORY\n");
+    mprintf(LOGG_INFO, "    --config-file=FILE                 Read configuration from FILE.\n");
+#ifdef _WIN32
+    mprintf(LOGG_INFO, "    --memory                           Scan loaded executable modules\n");
+    mprintf(LOGG_INFO, "    --kill                             Kill/Unload infected loaded modules\n");
+    mprintf(LOGG_INFO, "    --unload                           Unload infected modules from processes\n");
+#endif
+    mprintf(LOGG_INFO, "    --allmatch            -z           Continue scanning within file after finding a match.\n");
+    mprintf(LOGG_INFO, "    --multiscan           -m           Force MULTISCAN mode\n");
+    mprintf(LOGG_INFO, "    --infected            -i           Only print infected files\n");
+    mprintf(LOGG_INFO, "    --no-summary                       Disable summary at end of scanning\n");
+    mprintf(LOGG_INFO, "    --reload                           Request clamd to reload virus database\n");
+    mprintf(LOGG_INFO, "    --fdpass                           Pass filedescriptor to clamd (useful if clamd is running as a different user)\n");
+    mprintf(LOGG_INFO, "    --stream                           Force streaming files to clamd (for debugging and unit testing)\n");
+    mprintf(LOGG_INFO, "\n");
 
     exit(0);
 }
