@@ -23,8 +23,6 @@
 #define PROTO_H
 #include "misc.h"
 
-int dconnect(void);
 int serial_client_scan(char *file, int scantype, int *infected, int *err, int maxlevel, int flags);
 int parallel_client_scan(char *file, int scantype, int *infected, int *err, int maxlevel, int flags);
-int dsresult(int sockd, int scantype, const char *filename, int *printok, int *errors);
 #endif
