@@ -128,6 +128,7 @@ const char *autoit_functions[] = {
     "DRIVESPACEFREE",
     "DRIVESPACETOTAL",
     "DRIVESTATUS",
+    "DUMMYSPEEDTEST",
     "ENVGET",
     "ENVSET",
     "ENVUPDATE",
@@ -135,7 +136,6 @@ const char *autoit_functions[] = {
     "EXECUTE",
     "EXP",
     "FILECHANGEDIR",
-    "UNKNOWN_89",
     "FILECLOSE",
     "FILECOPY",
     "FILECREATENTFSLINK",
@@ -275,17 +275,17 @@ const char *autoit_functions[] = {
     "ISHWND",
     "ISINT",
     "ISKEYWORD",
-    "UNKNOWN_229",
+    "ISMAP",
     "ISNUMBER",
     "ISOBJ",
     "ISPTR",
     "ISSTRING",
     "LOG",
+    "MAPAPPEND",
+    "MAPEXISTS",
+    "MAPKEYS",
+    "MAPREMOVE",
     "MEMGETSTATS",
-    "UNKNOWN_235",
-    "UNKNOWN_236",
-    "UNKNOWN_237",
-    "UNKNOWN_238",
     "MOD",
     "MOUSECLICK",
     "MOUSECLICKDRAG",
@@ -393,8 +393,8 @@ const char *autoit_functions[] = {
     "TCPNAMETOIP",
     "TCPRECV",
     "TCPSEND",
-    "TCPSHUTDOWN,",
-    "TCPSTARTUP,",
+    "TCPSHUTDOWN",
+    "TCPSTARTUP",
     "TIMERDIFF",
     "TIMERINIT",
     "TOOLTIP",
@@ -421,8 +421,8 @@ const char *autoit_functions[] = {
     "UDPOPEN",
     "UDPRECV",
     "UDPSEND",
-    "UNKNOWN_375",
-    "UNKNOWN_376",
+    "UDPSHUTDOWN",
+    "UDPSTARTUP",
     "VARGETTYPE",
     "WINACTIVATE",
     "WINACTIVE",
@@ -454,7 +454,7 @@ const char *autoit_functions[] = {
     "WINWAITNOTACTIVE"};
 
 const char *autoit_keywords[] = {
-    "UNKNOWN_0",
+    "UNKNOWN_0", // "".
     "AND",
     "OR",
     "NOT",
@@ -497,7 +497,7 @@ const char *autoit_keywords[] = {
     "FALSE",
     "DEFAULT",
     "NULL",
-    "UNKNOWN_43",
+    "VOLATILE",
     "ENUM",
 };
 
@@ -613,7 +613,7 @@ struct UNP {
 
 static uint32_t getbits(struct UNP *UNP, uint32_t size)
 {
-    //cli_dbgmsg("In getbits, (size: %u, bits_avail: %u, UNP->cur_input: %u)\n", size, UNP->bits_avail, UNP->cur_input);
+    // cli_dbgmsg("In getbits, (size: %u, bits_avail: %u, UNP->cur_input: %u)\n", size, UNP->bits_avail, UNP->cur_input);
     UNP->bitmap.half.h = 0;
     if (size > UNP->bits_avail && ((size - UNP->bits_avail - 1) / 16 + 1) * 2 > UNP->csize - UNP->cur_input) {
         cli_dbgmsg("autoit: getbits() - not enough bits available\n");
@@ -622,7 +622,7 @@ static uint32_t getbits(struct UNP *UNP, uint32_t size)
     }
     while (size) {
         if (!UNP->bits_avail) {
-            //cli_dbgmsg("cur_input: %u (size: %u)\n", UNP->cur_input, size);
+            // cli_dbgmsg("cur_input: %u (size: %u)\n", UNP->cur_input, size);
             UNP->bitmap.half.l |= UNP->inputbuf[UNP->cur_input++] << 8;
             UNP->bitmap.half.l |= UNP->inputbuf[UNP->cur_input++];
             UNP->bits_avail = 16;
@@ -638,39 +638,49 @@ static uint32_t getbits(struct UNP *UNP, uint32_t size)
  autoit3 EA05 handler
 *********************/
 
-static int ea05(cli_ctx *ctx, const uint8_t *base, char *tmpd)
+static cl_error_t ea05(cli_ctx *ctx, const uint8_t *base, char *tmpd)
 {
-    uint8_t b[300], comp;
+    cl_error_t status = CL_SUCCESS;
+    cl_error_t ret;
+    uint8_t b[300];
+    uint8_t comp;
     uint32_t s, m4sum = 0;
-    int i, ret, det = 0;
-    unsigned int files = 0;
-    char tempfile[1024];
-    struct UNP UNP;
-    fmap_t *map = ctx->fmap;
+    int i;
+    unsigned int files  = 0;
+    char tempfile[1024] = {0};
+    int tempfd          = -1;
+    struct UNP UNP      = {0};
+    fmap_t *map         = ctx->fmap;
 
-    if (!fmap_need_ptr_once(map, base, 16))
-        return CL_CLEAN;
+    if (!fmap_need_ptr_once(map, base, 16)) {
+        goto done;
+    }
 
     for (i = 0; i < 16; i++)
         m4sum += *base++;
 
-    while ((ret = cli_checklimits("autoit", ctx, 0, 0, 0)) == CL_CLEAN) {
-        if (!fmap_need_ptr_once(map, base, 8))
-            return (det ? CL_VIRUS : CL_CLEAN);
+    // While we have not exceeded the max files limit or the max time limit...
+    while (CL_SUCCESS == (status = cli_checklimits("autoit", ctx, 0, 0, 0))) {
+        if (!fmap_need_ptr_once(map, base, 8)) {
+            goto done;
+        }
 
         /*     MT_decrypt(buf,4,0x16fa);  waste of time */
         if ((uint32_t)cli_readint32(base) != 0xceb06dff) {
             cli_dbgmsg("autoit: no FILE magic found, extraction complete\n");
-            return (det ? CL_VIRUS : CL_CLEAN);
+            goto done;
         }
 
         s = cli_readint32(base + 4) ^ 0x29bc;
-        if ((int32_t)s < 0)
-            return (det ? CL_VIRUS : CL_CLEAN); /* the original code wouldn't seek back here */
+        if ((int32_t)s < 0) {
+            /* the original code wouldn't seek back here */
+            goto done;
+        }
         base += 8;
         if (cli_debug_flag && s < sizeof(b)) {
-            if (!fmap_need_ptr_once(map, base, s))
-                return (det ? CL_VIRUS : CL_CLEAN);
+            if (!fmap_need_ptr_once(map, base, s)) {
+                goto done;
+            }
             memcpy(b, base, s);
             MT_decrypt(b, s, s + 0xa25e);
             b[s] = '\0';
@@ -678,15 +688,20 @@ static int ea05(cli_ctx *ctx, const uint8_t *base, char *tmpd)
         }
         base += s;
 
-        if (!fmap_need_ptr_once(map, base, 4))
-            return (det ? CL_VIRUS : CL_CLEAN);
+        if (!fmap_need_ptr_once(map, base, 4)) {
+            goto done;
+        }
         s = cli_readint32(base) ^ 0x29ac;
-        if ((int32_t)s < 0)
-            return (det ? CL_VIRUS : CL_CLEAN); /* the original code wouldn't seek back here */
+        if ((int32_t)s < 0) {
+            /* the original code wouldn't seek back here */
+            goto done;
+        }
         base += 4;
         if (cli_debug_flag && s < sizeof(b)) {
-            if (!fmap_need_ptr_once(map, base, s))
-                return (det ? CL_VIRUS : CL_CLEAN);
+            if (!fmap_need_ptr_once(map, base, s)) {
+                goto done;
+            }
+
             memcpy(b, base, s);
             MT_decrypt(b, s, s + 0xf25e);
             b[s] = '\0';
@@ -694,13 +709,15 @@ static int ea05(cli_ctx *ctx, const uint8_t *base, char *tmpd)
         }
         base += s;
 
-        if (!fmap_need_ptr_once(map, base, 13))
-            return (det ? CL_VIRUS : CL_CLEAN);
+        if (!fmap_need_ptr_once(map, base, 13)) {
+            goto done;
+        }
+
         comp      = *base;
         UNP.csize = cli_readint32(base + 1) ^ 0x45aa;
         if ((int32_t)UNP.csize < 0) {
             cli_dbgmsg("autoit: bad file size - giving up\n");
-            return (det ? CL_VIRUS : CL_CLEAN);
+            goto done;
         }
 
         if (!UNP.csize) {
@@ -724,36 +741,50 @@ static int ea05(cli_ctx *ctx, const uint8_t *base, char *tmpd)
             continue;
         }
 
-        if (!(UNP.inputbuf = cli_malloc(UNP.csize)))
-            return CL_EMEM;
+        if (!(UNP.inputbuf = cli_malloc(UNP.csize))) {
+            status = CL_EMEM;
+            goto done;
+        }
         if (!fmap_need_ptr_once(map, base, UNP.csize)) {
             cli_dbgmsg("autoit: failed to read compressed stream. broken/truncated file?\n");
-            free(UNP.inputbuf);
-            return (det ? CL_VIRUS : CL_CLEAN);
+            goto done;
         }
+
         memcpy(UNP.inputbuf, base, UNP.csize);
         base += UNP.csize;
         MT_decrypt(UNP.inputbuf, UNP.csize, 0x22af + m4sum);
 
         if (comp == 1) {
+            /*
+             * File is compressed. Decompress!
+             */
             cli_dbgmsg("autoit: file is compressed\n");
             if (cli_readint32(UNP.inputbuf) != 0x35304145) {
                 cli_dbgmsg("autoit: bad magic or unsupported version\n");
+                // Free this inputbuf and set back to NULL.
                 free(UNP.inputbuf);
+                UNP.inputbuf = NULL;
+
                 continue;
             }
 
-            if (!(UNP.usize = be32_to_host(*(uint32_t *)(UNP.inputbuf + 4))))
+            if (!(UNP.usize = be32_to_host(*(uint32_t *)(UNP.inputbuf + 4)))) {
                 UNP.usize = UNP.csize; /* only a specifically crafted or badly corrupted sample should land here */
+            }
+
             if (cli_checklimits("autoit", ctx, UNP.usize, 0, 0) != CL_CLEAN) {
+                // Free this inputbuf and set back to NULL.
                 free(UNP.inputbuf);
+                UNP.inputbuf = NULL;
+
                 continue;
             }
 
             if (!(UNP.outputbuf = cli_malloc(UNP.usize))) {
-                free(UNP.inputbuf);
-                return CL_EMEM;
+                status = CL_EMEM;
+                goto done;
             }
+
             cli_dbgmsg("autoit: uncompressed size again: %x\n", UNP.usize);
 
             UNP.cur_output  = 0;
@@ -806,6 +837,8 @@ static int ea05(cli_ctx *ctx, const uint8_t *base, char *tmpd)
             }
 
             free(UNP.inputbuf);
+            UNP.inputbuf = NULL;
+
             /* Sometimes the autoit exe is in turn packed/lamed with a runtime compressor and similar shit.
              * However, since the autoit script doesn't compress a second time very well, chances are we're
              * still able to match the headers and unpack something (see sample 0811129)
@@ -818,14 +851,21 @@ static int ea05(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                 UNP.usize = UNP.cur_output;
             }
         } else {
+            /*
+             * File is NOT compressed.
+             */
             cli_dbgmsg("autoit: file is not compressed\n");
             UNP.outputbuf = UNP.inputbuf;
-            UNP.usize     = UNP.csize;
+            UNP.inputbuf  = NULL;
+
+            UNP.usize = UNP.csize;
         }
 
         if (UNP.usize < 4) {
             cli_dbgmsg("autoit: file is too short\n");
             free(UNP.outputbuf);
+            UNP.outputbuf = NULL;
+
             continue;
         }
 
@@ -836,41 +876,62 @@ static int ea05(cli_ctx *ctx, const uint8_t *base, char *tmpd)
 
         snprintf(tempfile, 1023, "%s" PATHSEP "autoit.%.3u", tmpd, files);
         tempfile[1023] = '\0';
-        if ((i = open(tempfile, O_RDWR | O_CREAT | O_TRUNC | O_BINARY, S_IRUSR | S_IWUSR)) < 0) {
+
+        tempfd = open(tempfile, O_RDWR | O_CREAT | O_TRUNC | O_BINARY, S_IRUSR | S_IWUSR);
+        if (tempfd < 0) {
             cli_dbgmsg("autoit: Can't create file %s\n", tempfile);
-            free(UNP.outputbuf);
-            return CL_ECREAT;
+            status = CL_ECREAT;
+            goto done;
         }
-        if (cli_writen(i, UNP.outputbuf, UNP.usize) != UNP.usize) {
+
+        if (cli_writen(tempfd, UNP.outputbuf, UNP.usize) != UNP.usize) {
             cli_dbgmsg("autoit: cannot write %d bytes\n", UNP.usize);
-            close(i);
-            free(UNP.outputbuf);
-            return CL_EWRITE;
+            status = CL_EWRITE;
+            goto done;
         }
+
         free(UNP.outputbuf);
-        if (ctx->engine->keeptmp)
+        UNP.outputbuf = NULL;
+
+        if (ctx->engine->keeptmp) {
             cli_dbgmsg("autoit: file extracted to %s\n", tempfile);
-        else
+        } else {
             cli_dbgmsg("autoit: file successfully extracted\n");
-        if (lseek(i, 0, SEEK_SET) == -1) {
+        }
+
+        if (lseek(tempfd, 0, SEEK_SET) == -1) {
             cli_dbgmsg("autoit: call to lseek() has failed\n");
-            close(i);
-            return CL_ESEEK;
+            status = CL_ESEEK;
+            goto done;
         }
-        if (cli_magic_scan_desc(i, tempfile, ctx, NULL) == CL_VIRUS) {
-            if (!SCAN_ALLMATCHES) {
-                close(i);
-                if (!ctx->engine->keeptmp)
-                    if (cli_unlink(tempfile)) return CL_EUNLINK;
-                return CL_VIRUS;
-            }
-            det = 1;
+
+        ret = cli_magic_scan_desc(tempfd, tempfile, ctx, NULL, LAYER_ATTRIBUTES_NONE);
+        if (CL_SUCCESS != ret) {
+            status = ret;
+            goto done;
+        }
+
+        close(tempfd);
+        tempfd = -1;
+        if (!ctx->engine->keeptmp) {
+            (void)cli_unlink(tempfile);
         }
-        close(i);
-        if (!ctx->engine->keeptmp)
-            if (cli_unlink(tempfile)) return CL_EUNLINK;
     }
-    return (det ? CL_VIRUS : ret);
+
+done:
+    if (NULL != UNP.inputbuf) {
+        free(UNP.inputbuf);
+    }
+    if (NULL != UNP.outputbuf) {
+        free(UNP.outputbuf);
+    }
+    if (tempfd >= 0) {
+        close(tempfd);
+        if (!ctx->engine->keeptmp) {
+            (void)cli_unlink(tempfile);
+        }
+    }
+    return status;
 }
 
 /*********************
@@ -964,12 +1025,13 @@ static void LAME_decrypt(uint8_t *cypher, uint32_t size, uint16_t seed)
 
 static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
 {
-    uint8_t b[600], comp, script, *buf;
+    cl_error_t ret;
+    uint8_t b[600], comp, *buf;
     uint32_t s;
-    int i, ret, det = 0;
+    int i;
     unsigned int files = 0;
     char tempfile[1024];
-    const char prefixes[] = {'\0', '\0', '@', '$', '\0', '.', '"', '#'};
+    const char prefixes[] = {'\0', '\0', '@', '$', '\0', '.', '"', '\0'};
     const char *opers[]   = {",", "=", ">", "<", "<>", ">=", "<=", "(", ")", "+", "-", "/", "*", "&", "[", "]", "==", "^", "+=", "-=", "/=", "*=", "&=", "?", ":"};
     struct UNP UNP;
     fmap_t *map = ctx->fmap;
@@ -981,60 +1043,81 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
     /*   buf+=0x10; */
     base += 16; /* for now we just skip the garbage */
 
-    while ((ret = cli_checklimits("cli_autoit", ctx, 0, 0, 0)) == CL_CLEAN) {
-        if (!fmap_need_ptr_once(map, base, 8))
-            return (det ? CL_VIRUS : CL_CLEAN);
+    while (CL_SUCCESS == (ret = cli_checklimits("cli_autoit", ctx, 0, 0, 0))) {
+        bool script = false;
+
+        if (!fmap_need_ptr_once(map, base, 8)) {
+            return CL_SUCCESS;
+        }
+
         /*     LAME_decrypt(buf, 4, 0x18ee); waste of time */
         if (cli_readint32(base) != 0x52ca436b) {
             cli_dbgmsg("autoit: no FILE magic found, giving up (got 0x%08x)\n", cli_readint32(base));
-            return (det ? CL_VIRUS : CL_CLEAN);
+            return CL_SUCCESS;
         }
 
-        script = 0;
-
         s = cli_readint32(base + 4) ^ 0xadbc;
-        if ((int32_t)(s * 2) < 0)
-            return (det ? CL_VIRUS : CL_CLEAN); /* the original code wouldn't seek back here */
+        if ((int32_t)(s * 2) < 0) {
+            return CL_SUCCESS; /* the original code wouldn't seek back here */
+        }
+
         base += 8;
+
         if (s < sizeof(b) / 2) {
-            if (!fmap_need_ptr_once(map, base, s * 2))
-                return (det ? CL_VIRUS : CL_CLEAN);
+            if (!fmap_need_ptr_once(map, base, s * 2)) {
+                return CL_SUCCESS;
+            }
+
             memcpy(b, base, s * 2);
             LAME_decrypt(b, s * 2, s + 0xb33f);
             u2a(b, s * 2);
             cli_dbgmsg("autoit: magic string '%s'\n", b);
-            if (s == 19 && !memcmp(">>>AUTOIT SCRIPT<<<", b, 19))
-                script = 1;
+
+            if (s == 19 && !memcmp(">>>AUTOIT SCRIPT<<<", b, 19)) {
+                script = true;
+            }
         } else {
             cli_dbgmsg("autoit: magic string too long to print\n");
         }
+
         base += s * 2;
 
-        if (!fmap_need_ptr_once(map, base, 4))
-            return (det ? CL_VIRUS : CL_CLEAN);
+        if (!fmap_need_ptr_once(map, base, 4)) {
+            return CL_SUCCESS;
+        }
+
         s = cli_readint32(base) ^ 0xf820;
-        if ((int32_t)(s * 2) < 0)
-            return (det ? CL_VIRUS : CL_CLEAN); /* the original code wouldn't seek back here */
+        if ((int32_t)(s * 2) < 0) {
+            return CL_SUCCESS; /* the original code wouldn't seek back here */
+        }
+
         base += 4;
+
         if (cli_debug_flag && s < sizeof(b) / 2) {
-            if (!fmap_need_ptr_once(map, base, s * 2))
-                return (det ? CL_VIRUS : CL_CLEAN);
+            if (!fmap_need_ptr_once(map, base, s * 2)) {
+                return CL_SUCCESS;
+            }
+
             memcpy(b, base, s * 2);
             LAME_decrypt(b, s * 2, s + 0xf479);
             b[s * 2]     = '\0';
             b[s * 2 + 1] = '\0';
             u2a(b, s * 2);
+
             cli_dbgmsg("autoit: original filename '%s'\n", b);
         }
+
         base += s * 2;
 
-        if (!fmap_need_ptr_once(map, base, 13))
-            return (det ? CL_VIRUS : CL_CLEAN);
+        if (!fmap_need_ptr_once(map, base, 13)) {
+            return CL_SUCCESS;
+        }
+
         comp      = *base;
         UNP.csize = cli_readint32(base + 1) ^ 0x87bc;
         if ((int32_t)UNP.csize < 0) {
             cli_dbgmsg("autoit: bad file size - giving up\n");
-            return (det ? CL_VIRUS : CL_CLEAN);
+            return CL_SUCCESS;
         }
 
         if (!UNP.csize) {
@@ -1042,6 +1125,7 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
             base += 13 + 16;
             continue;
         }
+
         cli_dbgmsg("autoit: compressed size: %x\n", UNP.csize);
         cli_dbgmsg("autoit: advertised uncompressed size %x\n", cli_readint32(base + 5) ^ 0x87bc);
         cli_dbgmsg("autoit: ref chksum: %x\n", cli_readint32(base + 9) ^ 0xa685);
@@ -1060,35 +1144,44 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
 
         files++;
 
-        if (!(UNP.inputbuf = cli_malloc(UNP.csize)))
+        if (!(UNP.inputbuf = cli_malloc(UNP.csize))) {
             return CL_EMEM;
+        }
+
         if (!fmap_need_ptr_once(map, base, UNP.csize)) {
             cli_dbgmsg("autoit: failed to read compressed stream. broken/truncated file?\n");
             free(UNP.inputbuf);
-            return (det ? CL_VIRUS : CL_CLEAN);
+            return CL_SUCCESS;
         }
+
         memcpy(UNP.inputbuf, base, UNP.csize);
         base += UNP.csize;
+
         LAME_decrypt(UNP.inputbuf, UNP.csize, 0x2477 /* + m4sum (broken by design) */);
 
         if (comp == 1) {
             cli_dbgmsg("autoit: file is compressed\n");
+
             if (cli_readint32(UNP.inputbuf) != 0x36304145) {
                 cli_dbgmsg("autoit: bad magic or unsupported version\n");
                 free(UNP.inputbuf);
                 continue;
             }
 
-            if (!(UNP.usize = be32_to_host(*(uint32_t *)(UNP.inputbuf + 4))))
+            if (!(UNP.usize = be32_to_host(*(uint32_t *)(UNP.inputbuf + 4)))) {
                 UNP.usize = UNP.csize; /* only a specifically crafted or badly corrupted sample should land here */
+            }
+
             if (cli_checklimits("autoit", ctx, UNP.usize, 0, 0) != CL_CLEAN) {
                 free(UNP.inputbuf);
                 continue;
             }
+
             if (!(UNP.outputbuf = cli_malloc(UNP.usize))) {
                 free(UNP.inputbuf);
                 return CL_EMEM;
             }
+
             cli_dbgmsg("autoit: uncompressed size again: %x\n", UNP.usize);
 
             UNP.cur_output  = 0;
@@ -1125,12 +1218,13 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                         break;
                     }
 
-                    //cli_dbgmsg("cur_output: %u, bs: %u, bb: %u\n", UNP.cur_output, bs, bb);
+                    // cli_dbgmsg("cur_output: %u, bs: %u, bb: %u\n", UNP.cur_output, bs, bb);
                     if (!CLI_ISCONTAINED(UNP.outputbuf, UNP.usize, &UNP.outputbuf[UNP.cur_output], bs) ||
                         !CLI_ISCONTAINED(UNP.outputbuf, UNP.usize, &UNP.outputbuf[UNP.cur_output - bb], bs)) {
                         UNP.error = 1;
                         break;
                     }
+
                     while (bs--) {
                         UNP.outputbuf[UNP.cur_output] = UNP.outputbuf[UNP.cur_output - bb];
                         UNP.cur_output++;
@@ -1166,10 +1260,12 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                 free(UNP.outputbuf);
                 return CL_EMEM;
             }
+
             UNP.cur_output = 0;
             UNP.cur_input  = 4;
             UNP.bits_avail = cli_readint32((char *)UNP.outputbuf);
             UNP.error      = 0;
+
             cli_dbgmsg("autoit: script has got %u lines\n", UNP.bits_avail);
 
             while (!UNP.error && UNP.bits_avail && UNP.cur_input < UNP.usize) {
@@ -1184,13 +1280,16 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             cli_dbgmsg("autoit: too few bytes present - expected enough for a keyword ID\n");
                             break;
                         }
+
                         keyword_id = cli_readint32((char *)&UNP.outputbuf[UNP.cur_input]);
                         if (keyword_id >= (sizeof(autoit_keywords) / sizeof(autoit_keywords[0]))) {
                             UNP.error = 1;
                             cli_dbgmsg("autoit: unknown AutoIT keyword ID: 0x%x\n", keyword_id);
                             break;
                         }
+
                         UNP.cur_input += 4;
+
                         keyword_len = strlen(autoit_keywords[keyword_id]);
                         if (UNP.cur_output + keyword_len + 2 >= UNP.csize) {
                             uint8_t *newout;
@@ -1201,11 +1300,13 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             }
                             buf = newout;
                         }
+
                         if (cli_debug_flag) {
                             if (0 == memcmp(autoit_keywords[keyword_id], "UNKNOWN", MIN(strlen("UNKNOWN"), keyword_len))) {
                                 cli_dbgmsg("autoit: encountered use of unknown keyword ID: %s\n", autoit_keywords[keyword_id]);
                             }
                         }
+
                         snprintf((char *)&buf[UNP.cur_output], keyword_len + 2, "%s ", autoit_keywords[keyword_id]);
                         UNP.cur_output += keyword_len + 1;
                         break;
@@ -1218,13 +1319,16 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             cli_dbgmsg("autoit: too few bytes present - expected enough for a function ID\n");
                             break;
                         }
+
                         function_id = cli_readint32((char *)&UNP.outputbuf[UNP.cur_input]);
                         if (function_id >= (sizeof(autoit_functions) / sizeof(autoit_functions[0]))) {
                             UNP.error = 1;
                             cli_dbgmsg("autoit: unknown AutoIT function ID: 0x%x\n", function_id);
                             break;
                         }
+
                         UNP.cur_input += 4;
+
                         function_len = strlen(autoit_functions[function_id]);
                         if (UNP.cur_output + function_len + 2 >= UNP.csize) {
                             uint8_t *newout;
@@ -1235,11 +1339,13 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             }
                             buf = newout;
                         }
+
                         if (cli_debug_flag) {
                             if (0 == memcmp(autoit_functions[function_id], "UNKNOWN", MIN(strlen("UNKNOWN"), function_len))) {
                                 cli_dbgmsg("autoit: encountered use of unknown function ID: %s\n", autoit_functions[function_id]);
                             }
                         }
+
                         snprintf((char *)&buf[UNP.cur_output], function_len + 2, "%s ", autoit_functions[function_id]);
                         UNP.cur_output += function_len + 1;
                         break;
@@ -1250,6 +1356,7 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             cli_dbgmsg("autoit: not enough space for an int\n");
                             break;
                         }
+
                         if (UNP.cur_output + 12 >= UNP.csize) {
                             uint8_t *newout;
                             UNP.csize += 512;
@@ -1259,6 +1366,7 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             }
                             buf = newout;
                         }
+
                         snprintf((char *)&buf[UNP.cur_output], 12, "0x%08x ", cli_readint32((char *)&UNP.outputbuf[UNP.cur_input]));
                         UNP.cur_output += 11;
                         UNP.cur_input += 4;
@@ -1272,6 +1380,7 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             cli_dbgmsg("autoit: not enough space for an int64\n");
                             break;
                         }
+
                         if (UNP.cur_output + 20 >= UNP.csize) {
                             uint8_t *newout;
                             UNP.csize += 512;
@@ -1281,6 +1390,7 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             }
                             buf = newout;
                         }
+
                         val = (uint64_t)cli_readint32((char *)&UNP.outputbuf[UNP.cur_input + 4]);
                         val <<= 32;
                         val += (uint64_t)cli_readint32((char *)&UNP.outputbuf[UNP.cur_input]);
@@ -1296,6 +1406,7 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             cli_dbgmsg("autoit: not enough space for a double\n");
                             break;
                         }
+
                         if (UNP.cur_output + 40 >= UNP.csize) {
                             uint8_t *newout;
                             UNP.csize += 512;
@@ -1305,16 +1416,19 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             }
                             buf = newout;
                         }
-                        if (fpu_words == FPU_ENDIAN_LITTLE)
+
+                        if (fpu_words == FPU_ENDIAN_LITTLE) {
                             snprintf((char *)&buf[UNP.cur_output], 39, "%g ", *(double *)&UNP.outputbuf[UNP.cur_input]);
-                        else
+                        } else
                             do {
                                 double x;
                                 uint8_t *j = (uint8_t *)&x;
                                 unsigned int i;
 
-                                for (i = 0; i < 8; i++)
+                                for (i = 0; i < 8; i++) {
                                     j[7 - i] = UNP.outputbuf[UNP.cur_input + i];
+                                }
+
                                 snprintf((char *)&buf[UNP.cur_output], 39, "%g ", x); /* FIXME: check */
                             } while (0);
                         buf[UNP.cur_output + 38] = ' ';
@@ -1339,6 +1453,7 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             cli_dbgmsg("autoit: not enough space for size\n");
                             break;
                         }
+
                         chars  = cli_readint32((char *)&UNP.outputbuf[UNP.cur_input]);
                         dchars = chars * 2;
                         UNP.cur_input += 4;
@@ -1348,6 +1463,7 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             cli_dbgmsg("autoit: size too big - needed %d, total %d, avail %d\n", dchars, UNP.usize, UNP.usize - UNP.cur_input);
                             break;
                         }
+
                         if (UNP.cur_output + chars + 3 >= UNP.csize) {
                             uint8_t *newout;
                             UNP.csize += chars + 512;
@@ -1358,8 +1474,9 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             buf = newout;
                         }
 
-                        if (prefixes[op - 0x30])
+                        if (prefixes[op - 0x30]) {
                             buf[UNP.cur_output++] = prefixes[op - 0x30];
+                        }
 
                         if (chars) {
                             for (i = 0; i < dchars; i += 2) {
@@ -1371,10 +1488,14 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             UNP.cur_output += chars;
                             UNP.cur_input += dchars;
                         }
-                        if (op == 0x36)
+
+                        if (op == 0x36) {
+                            // TODO: Mask possible double quotes inside the string: >Say:"Hi "<  ==> >"Say:""Hi"" "<
                             buf[UNP.cur_output++] = '"';
-                        if (op != 0x34)
+                        }
+                        if (op != 0x34) {
                             buf[UNP.cur_output++] = ' ';
+                        }
                     } break;
 
                     case 0x40: /* , */
@@ -1411,6 +1532,13 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             }
                             buf = newout;
                         }
+
+                        // TODO: Fix Autoit plus bug
+                        //  if (op == 0x49) /* + */ and next op ==0x05 /*int32*/ and that int32 is negative...
+                        //  skip next line (and don't add "+")
+                        //  Background: "$a= (-4)" gets incorrect compiled. Decompiled it will be get "$A= (+ -4)"
+                        //  That doesn't effects the interpreter however when recompiling decompiled output that will result in a syntax error
+
                         UNP.cur_output += snprintf((char *)&buf[UNP.cur_output], 4, "%s ", opers[op - 0x40]);
                         break;
 
@@ -1434,8 +1562,9 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                 }
             }
 
-            if (UNP.error)
+            if (UNP.error) {
                 cli_dbgmsg("autoit: decompilation aborted - partial script may exist\n");
+            }
 
             free(UNP.outputbuf);
         } else {
@@ -1456,40 +1585,51 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
             free(buf);
             return CL_EWRITE;
         }
+
         free(buf);
-        if (ctx->engine->keeptmp)
+
+        if (ctx->engine->keeptmp) {
             cli_dbgmsg("autoit: %s extracted to %s\n", (script) ? "script" : "file", tempfile);
-        else
+        } else {
             cli_dbgmsg("autoit: %s successfully extracted\n", (script) ? "script" : "file");
+        }
+
         if (lseek(i, 0, SEEK_SET) == -1) {
             cli_dbgmsg("autoit: call to lseek() has failed\n");
             close(i);
             return CL_ESEEK;
         }
-        if (cli_magic_scan_desc(i, tempfile, ctx, NULL) == CL_VIRUS) {
-            if (!SCAN_ALLMATCHES) {
-                close(i);
-                if (!ctx->engine->keeptmp)
-                    if (cli_unlink(tempfile)) return CL_EUNLINK;
-                return CL_VIRUS;
+
+        ret = cli_magic_scan_desc(i, tempfile, ctx, NULL, LAYER_ATTRIBUTES_NONE);
+        if (CL_SUCCESS != ret) {
+            close(i);
+            if (!ctx->engine->keeptmp) {
+                if (cli_unlink(tempfile)) {
+                    return CL_EUNLINK;
+                }
             }
-            det = 1;
+            return CL_VIRUS;
         }
+
         close(i);
-        if (!ctx->engine->keeptmp)
-            if (cli_unlink(tempfile)) return CL_EUNLINK;
+
+        if (!ctx->engine->keeptmp) {
+            if (cli_unlink(tempfile)) {
+                return CL_EUNLINK;
+            }
+        }
     }
-    return (det ? CL_VIRUS : ret);
+    return ret;
 }
 
 /*********************
    autoit3 wrapper
 *********************/
 
-int cli_scanautoit(cli_ctx *ctx, off_t offset)
+cl_error_t cli_scanautoit(cli_ctx *ctx, off_t offset)
 {
+    cl_error_t status = CL_SUCCESS;
     const uint8_t *version;
-    int r;
     char *tmpd;
     fmap_t *map = ctx->fmap;
 
@@ -1510,7 +1650,7 @@ int cli_scanautoit(cli_ctx *ctx, off_t offset)
 
     switch (*version) {
         case 0x35:
-            r = ea05(ctx, version + 1, tmpd);
+            status = ea05(ctx, version + 1, tmpd);
             break;
         case 0x36:
             if (fpu_words == FPU_ENDIAN_INITME)
@@ -1518,19 +1658,19 @@ int cli_scanautoit(cli_ctx *ctx, off_t offset)
             if (fpu_words == FPU_ENDIAN_UNKNOWN) {
                 cli_dbgmsg("autoit: EA06 support not available"
                            "(cannot extract ea06 doubles, unknown floating double representation).\n");
-                r = CL_CLEAN;
+                status = CL_CLEAN;
             } else
-                r = ea06(ctx, version + 1, tmpd);
+                status = ea06(ctx, version + 1, tmpd);
             break;
         default:
             /* NOT REACHED */
             cli_dbgmsg("autoit: unknown method\n");
-            r = CL_CLEAN;
+            status = CL_CLEAN;
     }
 
     if (!ctx->engine->keeptmp)
         cli_rmdirs(tmpd);
 
     free(tmpd);
-    return r;
+    return status;
 }
