@@ -685,7 +685,7 @@ static struct {
     {(void *)cli_bcapi_get_pe_section, sizeof(struct cli_exe_section)},
 };
 
-int cli_vm_execute(const struct cli_bc *bc, struct cli_bc_ctx *ctx, const struct cli_bc_func *func, const struct cli_bc_inst *inst)
+cl_error_t cli_vm_execute(const struct cli_bc *bc, struct cli_bc_ctx *ctx, const struct cli_bc_func *func, const struct cli_bc_inst *inst)
 {
     size_t i;
     uint32_t j;
@@ -1162,7 +1162,7 @@ int cli_vm_execute(const struct cli_bc *bc, struct cli_bc_ctx *ctx, const struct
                         READ64(ptr, inst->u.three[1]);
                         off += (ptr & 0x00000000ffffffffULL);
                         iptr = (ptr & 0xffffffff00000000ULL) + (uint64_t)(off);
-                        WRITE64(inst->dest, ptr + off);
+                        WRITE64(inst->dest, iptr);
                     }
                     break;
                 }
@@ -1275,8 +1275,8 @@ int cli_vm_execute(const struct cli_bc *bc, struct cli_bc_ctx *ctx, const struct
                     } else {
                         READ64(ptr, inst->u.three[1]);
                         off *= inst->u.three[0];
-                        off += (ptr & 0x00000000ffffffff);
-                        iptr = (ptr & 0xffffffff00000000) + (uint64_t)(off);
+                        off += (ptr & 0x00000000ffffffffULL);
+                        iptr = (ptr & 0xffffffff00000000ULL) + (uint64_t)(off);
                         WRITE64(inst->dest, iptr);
                     }
                     break;
