@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -33,12 +33,12 @@
 
 /* Certain OSs already use 64bit variables in their stat struct */
 #if (!defined(__FreeBSD__) && !defined(__APPLE__))
-#define STAT64_BLACKLIST 1
+#define STAT64_OK 1
 #else
-#define STAT64_BLACKLIST 0
+#define STAT64_OK 0
 #endif
 
-#if defined(HAVE_STAT64) && STAT64_BLACKLIST
+#if defined(HAVE_STAT64) && STAT64_OK
 
 #include <unistd.h>
 
@@ -47,6 +47,7 @@
 #define LSTAT lstat64
 #define FSTAT fstat64
 #define safe_open(a, b) open(a, b | O_LARGEFILE)
+
 #else
 
 #define STATBUF struct stat
@@ -64,6 +65,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdbool.h>
 
 #include "clamav-types.h"
 #include "clamav-version.h"
@@ -119,6 +121,7 @@ typedef enum cl_error_t {
     CL_ESTATE,
 
     CL_VERIFIED, /* The binary has been deemed trusted */
+    CL_ERROR,    /* Unspecified / generic error */
 
     /* no error codes below this line please */
     CL_ELAST_ERROR
@@ -139,7 +142,7 @@ typedef enum cl_error_t {
 #define CL_DB_OFFICIAL_ONLY     0x1000
 #define CL_DB_BYTECODE          0x2000
 #define CL_DB_SIGNED            0x4000  /* internal */
-#define CL_DB_BYTECODE_UNSIGNED 0x8000
+#define CL_DB_BYTECODE_UNSIGNED 0x8000  /* Caution: You should never run bytecode signatures from untrusted sources. Doing so may result in arbitrary code execution. */
 #define CL_DB_UNSIGNED          0x10000 /* internal */
 #define CL_DB_BYTECODE_STATS    0x20000
 #define CL_DB_ENHANCED          0x40000
@@ -179,17 +182,19 @@ struct cl_scan_options {
 #define CL_SCAN_PARSE_PE                            0x200
 
 /* heuristic alerting options */
-#define CL_SCAN_HEURISTIC_BROKEN                    0x2   /* alert on broken PE and broken ELF files */
-#define CL_SCAN_HEURISTIC_EXCEEDS_MAX               0x4   /* alert when files exceed scan limits (filesize, max scansize, or max recursion depth) */
-#define CL_SCAN_HEURISTIC_PHISHING_SSL_MISMATCH     0x8   /* alert on SSL mismatches */
-#define CL_SCAN_HEURISTIC_PHISHING_CLOAK            0x10  /* alert on cloaked URLs in emails */
-#define CL_SCAN_HEURISTIC_MACROS                    0x20  /* alert on OLE2 files containing macros */
-#define CL_SCAN_HEURISTIC_ENCRYPTED_ARCHIVE         0x40  /* alert if archive is encrypted (rar, zip, etc) */
-#define CL_SCAN_HEURISTIC_ENCRYPTED_DOC             0x80  /* alert if a document is encrypted (pdf, docx, etc) */
-#define CL_SCAN_HEURISTIC_PARTITION_INTXN           0x100 /* alert if partition table size doesn't make sense */
-#define CL_SCAN_HEURISTIC_STRUCTURED                0x200 /* data loss prevention options, i.e. alert when detecting personal information */
-#define CL_SCAN_HEURISTIC_STRUCTURED_SSN_NORMAL     0x400 /* alert when detecting social security numbers */
-#define CL_SCAN_HEURISTIC_STRUCTURED_SSN_STRIPPED   0x800 /* alert when detecting stripped social security numbers */
+#define CL_SCAN_HEURISTIC_BROKEN                    0x2    /* alert on broken PE and broken ELF files */
+#define CL_SCAN_HEURISTIC_EXCEEDS_MAX               0x4    /* alert when files exceed scan limits (filesize, max scansize, or max recursion depth) */
+#define CL_SCAN_HEURISTIC_PHISHING_SSL_MISMATCH     0x8    /* alert on SSL mismatches */
+#define CL_SCAN_HEURISTIC_PHISHING_CLOAK            0x10   /* alert on cloaked URLs in emails */
+#define CL_SCAN_HEURISTIC_MACROS                    0x20   /* alert on OLE2 files containing macros */
+#define CL_SCAN_HEURISTIC_ENCRYPTED_ARCHIVE         0x40   /* alert if archive is encrypted (rar, zip, etc) */
+#define CL_SCAN_HEURISTIC_ENCRYPTED_DOC             0x80   /* alert if a document is encrypted (pdf, docx, etc) */
+#define CL_SCAN_HEURISTIC_PARTITION_INTXN           0x100  /* alert if partition table size doesn't make sense */
+#define CL_SCAN_HEURISTIC_STRUCTURED                0x200  /* data loss prevention options, i.e. alert when detecting personal information */
+#define CL_SCAN_HEURISTIC_STRUCTURED_SSN_NORMAL     0x400  /* alert when detecting social security numbers */
+#define CL_SCAN_HEURISTIC_STRUCTURED_SSN_STRIPPED   0x800  /* alert when detecting stripped social security numbers */
+#define CL_SCAN_HEURISTIC_STRUCTURED_CC             0x1000 /* alert when detecting credit card numbers */
+#define CL_SCAN_HEURISTIC_BROKEN_MEDIA              0x2000 /* alert if a file does not match the identified file format, works with JPEG, TIFF, GIF, PNG */
 
 /* mail scanning options */
 #define CL_SCAN_MAIL_PARTIAL_MESSAGE                0x1
@@ -260,7 +265,7 @@ void cl_cleanup_crypto(void);
  * @param initoptions   Unused.
  * @return cl_error_t   CL_SUCCESS if everything initalized correctly.
  */
-extern int cl_init(unsigned int initoptions);
+extern cl_error_t cl_init(unsigned int initoptions);
 
 /**
  * @brief Allocate a new scanning engine and initialize default settings.
@@ -337,6 +342,8 @@ typedef struct cli_stats_sections {
 /**
  * @brief Set a numerical engine option.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine            An initialized scan engine.
  * @param cl_engine_field   A CL_ENGINE option.
  * @param num               The new engine option value.
@@ -344,7 +351,7 @@ typedef struct cli_stats_sections {
  * @return cl_error_t       CL_EARG if the field number was incorrect.
  * @return cl_error_t       CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_set_num(struct cl_engine *engine, enum cl_engine_field field, long long num);
+extern cl_error_t cl_engine_set_num(struct cl_engine *engine, enum cl_engine_field field, long long num);
 
 /**
  * @brief Get a numerical engine option.
@@ -359,6 +366,11 @@ extern long long cl_engine_get_num(const struct cl_engine *engine, enum cl_engin
 /**
  * @brief Set a string engine option.
  *
+ * If the string option has already been set, the existing string will be free'd
+ * and the new string will replace it.
+ *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine            An initialized scan engine.
  * @param cl_engine_field   A CL_ENGINE option.
  * @param str               The new engine option value.
@@ -367,7 +379,7 @@ extern long long cl_engine_get_num(const struct cl_engine *engine, enum cl_engin
  * @return cl_error_t       CL_EMEM if a memory allocation error occurred.
  * @return cl_error_t       CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_set_str(struct cl_engine *engine, enum cl_engine_field field, const char *str);
+extern cl_error_t cl_engine_set_str(struct cl_engine *engine, enum cl_engine_field field, const char *str);
 
 /**
  * @brief Get a string engine option.
@@ -392,12 +404,14 @@ extern struct cl_settings *cl_engine_settings_copy(const struct cl_engine *engin
 /**
  * @brief Apply settings from a settings structure to a scan engine.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine        A scan engine.
  * @param settings      The settings.
  * @return cl_error_t   CL_SUCCESS if successful.
  * @return cl_error_t   CL_EMEM if a memory allocation error occurred.
  */
-extern int cl_engine_settings_apply(struct cl_engine *engine, const struct cl_settings *settings);
+extern cl_error_t cl_engine_settings_apply(struct cl_engine *engine, const struct cl_settings *settings);
 
 /**
  * @brief Free a settings struct pointer.
@@ -406,19 +420,19 @@ extern int cl_engine_settings_apply(struct cl_engine *engine, const struct cl_se
  * @return cl_error_t   CL_SUCCESS if successful.
  * @return cl_error_t   CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_settings_free(struct cl_settings *settings);
+extern cl_error_t cl_engine_settings_free(struct cl_settings *settings);
 
 /**
  * @brief Prepare the scanning engine.
  *
- * Called this after all required databases have been loaded and settings have
+ * Call this after all required databases have been loaded and settings have
  * been applied.
  *
  * @param engine        A scan engine.
  * @return cl_error_t   CL_SUCCESS if successful.
  * @return cl_error_t   CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_compile(struct cl_engine *engine);
+extern cl_error_t cl_engine_compile(struct cl_engine *engine);
 
 /**
  * @brief Add a reference count to the engine.
@@ -432,7 +446,7 @@ extern int cl_engine_compile(struct cl_engine *engine);
  * @return cl_error_t   CL_SUCCESS if successful.
  * @return cl_error_t   CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_addref(struct cl_engine *engine);
+extern cl_error_t cl_engine_addref(struct cl_engine *engine);
 
 /**
  * @brief Free an engine.
@@ -444,7 +458,7 @@ extern int cl_engine_addref(struct cl_engine *engine);
  * @return cl_error_t   CL_SUCCESS if successful.
  * @return cl_error_t   CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_free(struct cl_engine *engine);
+extern cl_error_t cl_engine_free(struct cl_engine *engine);
 
 /* ----------------------------------------------------------------------------
  * Callback function type definitions.
@@ -461,18 +475,66 @@ extern int cl_engine_free(struct cl_engine *engine);
  * @param type      File type detected via magic - i.e. NOT on the fly - (e.g. "CL_TYPE_MSEXE").
  * @param context   Opaque application provided data.
  * @return          CL_CLEAN = File is scanned.
- * @return          CL_BREAK = Whitelisted by callback - file is skipped and marked as clean.
- * @return          CL_VIRUS = Blacklisted by callback - file is skipped and marked as infected.
+ * @return          CL_BREAK = Allowed by callback - file is skipped and marked as clean.
+ * @return          CL_VIRUS = Blocked by callback - file is skipped and marked as infected.
  */
 typedef cl_error_t (*clcb_pre_cache)(int fd, const char *type, void *context);
 /**
  * @brief Set a custom pre-cache callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
 extern void cl_engine_set_clcb_pre_cache(struct cl_engine *engine, clcb_pre_cache callback);
 
+/*
+ * Attributes of each layer in scan.
+ */
+#define LAYER_ATTRIBUTES_NONE 0x0
+#define LAYER_ATTRIBUTES_NORMALIZED 0x1 /** This layer was modified to make matching more generic, reliable. */
+#define LAYER_ATTRIBUTES_DECRYPTED 0x2  /** Decryption was used to extract this layer. I.e. had to decrypt some previous layer. */
+
+/**
+ * @brief File inspection callback.
+ *
+ * DISCLAIMER: This interface is to be considered unstable while we continue to evaluate it.
+ * We may change this interface in the future.
+ *
+ * Called for each NEW file (inner and outer).
+ * Provides capability to record embedded file information during a scan.
+ *
+ * @param fd                  Current file descriptor which is about to be scanned.
+ * @param type                Current file type detected via magic - i.e. NOT on the fly - (e.g. "CL_TYPE_MSEXE").
+ * @param ancestors           An array of ancestors filenames of size `recursion_level`. filenames may be NULL.
+ * @param parent_file_size    Parent file size.
+ * @param file_name           Current file name, or NULL if the file does not have a name or ClamAV failed to record the name.
+ * @param file_size           Current file size.
+ * @param file_buffer         Current file buffer pointer.
+ * @param recursion_level     Recursion level / depth of the current file.
+ * @param layer_attributes    See LAYER_ATTRIBUTES_* flags.
+ * @param context             Opaque application provided data.
+ * @return                    CL_CLEAN = File is scanned.
+ * @return                    CL_BREAK = Whitelisted by callback - file is skipped and marked as clean.
+ * @return                    CL_VIRUS = Blacklisted by callback - file is skipped and marked as infected.
+ */
+typedef cl_error_t (*clcb_file_inspection)(int fd, const char *type, const char **ancestors, size_t parent_file_size,
+                                           const char *file_name, size_t file_size, const char *file_buffer,
+                                           uint32_t recursion_level, uint32_t layer_attributes, void *context);
+/**
+ * @brief Set a custom file inspection callback function.
+ *
+ * DISCLAIMER: This interface is to be considered unstable while we continue to evaluate it.
+ * We may change this interface in the future.
+ *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
+ * @param engine    The initialized scanning engine.
+ * @param callback  The callback function pointer.
+ */
+extern void cl_engine_set_clcb_file_inspection(struct cl_engine *engine, clcb_file_inspection callback);
+
 /**
  * @brief Pre-scan callback.
  *
@@ -485,13 +547,15 @@ extern void cl_engine_set_clcb_pre_cache(struct cl_engine *engine, clcb_pre_cach
  * @param type      File type detected via magic - i.e. NOT on the fly - (e.g. "CL_TYPE_MSEXE").
  * @param context   Opaque application provided data.
  * @return          CL_CLEAN = File is scanned.
- * @return          CL_BREAK = Whitelisted by callback - file is skipped and marked as clean.
- * @return          CL_VIRUS = Blacklisted by callback - file is skipped and marked as infected.
+ * @return          CL_BREAK = Allowed by callback - file is skipped and marked as clean.
+ * @return          CL_VIRUS = Blocked by callback - file is skipped and marked as infected.
  */
 typedef cl_error_t (*clcb_pre_scan)(int fd, const char *type, void *context);
 /**
  * @brief Set a custom pre-scan callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -510,27 +574,29 @@ extern void cl_engine_set_clcb_pre_scan(struct cl_engine *engine, clcb_pre_scan
  * @param virname   A signature name if there was one or more matches.
  * @param context   Opaque application provided data.
  * @return          Scan result is not overridden.
- * @return          CL_BREAK = Whitelisted by callback - scan result is set to CL_CLEAN.
- * @return          Blacklisted by callback - scan result is set to CL_VIRUS.
+ * @return          CL_BREAK = Allowed by callback - scan result is set to CL_CLEAN.
+ * @return          Blocked by callback - scan result is set to CL_VIRUS.
  */
 typedef cl_error_t (*clcb_post_scan)(int fd, int result, const char *virname, void *context);
 /**
  * @brief Set a custom post-scan callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
 extern void cl_engine_set_clcb_post_scan(struct cl_engine *engine, clcb_post_scan callback);
 
 /**
- * @brief Post-scan callback.
+ * @brief Virus-found callback.
  *
  * Called for each signature match.
  * If all-match is enabled, clcb_virus_found() may be called multiple times per
  * scan.
  *
  * In addition, clcb_virus_found() does not have a return value and thus.
- * can not be used to whitelist the match.
+ * can not be used to ignore the match.
  *
  * @param fd        File descriptor which was scanned.
  * @param virname   Virus name.
@@ -540,6 +606,8 @@ typedef void (*clcb_virus_found)(int fd, const char *virname, void *context);
 /**
  * @brief Set a custom virus-found callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -565,6 +633,8 @@ typedef int (*clcb_sigload)(const char *type, const char *name, unsigned int cus
 /**
  * @brief Set a custom signature-load callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  * @param context   Opaque application provided data.
@@ -578,6 +648,66 @@ enum cl_msg {
     CL_MSG_ERROR        = 128 /* LibClamAV ERROR: */
 };
 
+/**
+ * @brief Progress callback for sig-load, engine-compile, and engine-free.
+ *
+ * Progress is complete when total_items == now_completed.
+ *
+ * Note: The callback should return CL_SUCCESS. We reserve the right to have it
+ *       cancel the operation in the future if you return something else...
+ *       ... but for now, the return value will be ignored.
+ *
+ * @param total_items   Total number of items
+ * @param now_completed Number of items completed
+ * @param context       Opaque application provided data
+ * @return cl_error_t   reserved for future use
+ */
+typedef cl_error_t (*clcb_progress)(size_t total_items, size_t now_completed, void *context);
+
+/**
+ * @brief Set a progress callback function to be called incrementally during a
+ * database load.
+ *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
+ * @param engine    The initialized scanning engine
+ * @param callback  The callback function pointer
+ * @param context   Opaque application provided data
+ */
+extern void cl_engine_set_clcb_sigload_progress(struct cl_engine *engine, clcb_progress callback, void *context);
+
+/**
+ * @brief Set a progress callback function to be called incrementally during an
+ * engine compile.
+ *
+ * Disclaimer: the number of items for this is a rough estimate of the items that
+ * tend to take longest to compile and doesn't represent an accurate number of
+ * things compiled.
+ *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
+ * @param engine    The initialized scanning engine
+ * @param callback  The callback function pointer
+ * @param context   Opaque application provided data
+ */
+extern void cl_engine_set_clcb_engine_compile_progress(struct cl_engine *engine, clcb_progress callback, void *context);
+
+/**
+ * @brief Set a progress callback function to be called incrementally during an
+ * engine free (if the engine is in fact freed).
+ *
+ * Disclaimer: the number of items for this is a rough estimate of the items that
+ * tend to take longest to free and doesn't represent an accurate number of
+ * things freed.
+ *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
+ * @param engine    The initialized scanning engine
+ * @param callback  The callback function pointer
+ * @param context   Opaque application provided data
+ */
+extern void cl_engine_set_clcb_engine_free_progress(struct cl_engine *engine, clcb_progress callback, void *context);
+
 /**
  * @brief Logging message callback for info, warning, and error messages.
  *
@@ -625,6 +755,8 @@ typedef void (*clcb_hash)(int fd, unsigned long long size, const unsigned char *
 /**
  * @brief Set a custom hash stats callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -633,7 +765,7 @@ extern void cl_engine_set_clcb_hash(struct cl_engine *engine, clcb_hash callback
 /**
  * @brief Archive meta matching callback function.
  *
- * May be used to blacklist archive/container samples based on archive metadata.
+ * May be used to block archive/container samples based on archive metadata.
  * Function is invoked multiple times per archive. Typically once per contained file.
  *
  * Note: Used by the --archive-verbose clamscan option. Overriding this will alter
@@ -646,7 +778,7 @@ extern void cl_engine_set_clcb_hash(struct cl_engine *engine, clcb_hash callback
  * @param is_encrypted      Boolean non-zero if the contained file is encrypted.
  * @param filepos_container File index in container.
  * @param context           Opaque application provided data.
- * @return                  CL_VIRUS to blacklist
+ * @return                  CL_VIRUS to block (alert on)
  * @return                  CL_CLEAN to continue scanning
  */
 typedef cl_error_t (*clcb_meta)(const char *container_type, unsigned long fsize_container, const char *filename,
@@ -654,6 +786,8 @@ typedef cl_error_t (*clcb_meta)(const char *container_type, unsigned long fsize_
 /**
  * @brief Set a custom archive metadata matching callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -673,6 +807,8 @@ typedef int (*clcb_file_props)(const char *j_propstr, int rc, void *cbdata);
 /**
  * @brief Set a custom file properties callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -694,6 +830,8 @@ extern void cl_engine_set_clcb_file_props(struct cl_engine *engine, clcb_file_pr
  * The data must persist at least until `clcb_stats_submit()` is called, or
  * `clcb_stats_flush()` is called (optional).
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine The scanning engine.
  * @param cbdata The statistics data. Probably a pointer to a malloc'd struct.
  */
@@ -712,6 +850,8 @@ typedef void (*clcb_stats_add_sample)(const char *virname, const unsigned char *
 /**
  * @brief Set a custom callback function to add sample metadata to a statistics report.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -729,6 +869,8 @@ typedef void (*clcb_stats_remove_sample)(const char *virname, const unsigned cha
 /**
  * @brief Set a custom callback function to remove sample metadata from a statistics report.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -763,6 +905,8 @@ typedef void (*clcb_stats_submit)(struct cl_engine *engine, void *cbdata);
 /**
  * @brief Set a custom callback function to submit the statistics report.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -778,6 +922,8 @@ typedef void (*clcb_stats_flush)(struct cl_engine *engine, void *cbdata);
 /**
  * @brief Set a custom callback function to flush/free the statistics report data.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -792,6 +938,8 @@ typedef size_t (*clcb_stats_get_num)(void *cbdata);
 /**
  * @brief Set a custom callback function to get the number of samples listed in the statistics report.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -806,6 +954,8 @@ typedef size_t (*clcb_stats_get_size)(void *cbdata);
 /**
  * @brief Set a custom callback function to get the size of memory used to store the statistics report.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -820,6 +970,8 @@ typedef char *(*clcb_stats_get_hostid)(void *cbdata);
 /**
  * @brief Set a custom callback function to get the machine's unique host ID.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -847,7 +999,7 @@ extern void cl_engine_stats_enable(struct cl_engine *engine);
  * @param scanoptions       Scanning options.
  * @return cl_error_t       CL_CLEAN, CL_VIRUS, or an error code if an error occured during the scan.
  */
-extern int cl_scandesc(int desc, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions);
+extern cl_error_t cl_scandesc(int desc, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions);
 
 /**
  * @brief Scan a file, given a file descriptor.
@@ -860,10 +1012,10 @@ extern int cl_scandesc(int desc, const char *filename, const char **virname, uns
  * @param[out] scanned      The number of bytes scanned.
  * @param engine            The scanning engine.
  * @param scanoptions       Scanning options.
- * @param[in/out] context   An opaque context structure allowing the caller to record details about the sample being scanned.
+ * @param[in,out] context   An opaque context structure allowing the caller to record details about the sample being scanned.
  * @return cl_error_t       CL_CLEAN, CL_VIRUS, or an error code if an error occured during the scan.
  */
-extern int cl_scandesc_callback(int desc, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
+extern cl_error_t cl_scandesc_callback(int desc, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
 
 /**
  * @brief Scan a file, given a filename.
@@ -875,7 +1027,7 @@ extern int cl_scandesc_callback(int desc, const char *filename, const char **vir
  * @param scanoptions       Scanning options.
  * @return cl_error_t       CL_CLEAN, CL_VIRUS, or an error code if an error occured during the scan.
  */
-extern int cl_scanfile(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions);
+extern cl_error_t cl_scanfile(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions);
 
 /**
  * @brief Scan a file, given a filename.
@@ -887,15 +1039,31 @@ extern int cl_scanfile(const char *filename, const char **virname, unsigned long
  * @param[out] scanned      The number of bytes scanned.
  * @param engine            The scanning engine.
  * @param scanoptions       Scanning options.
- * @param[in/out] context   An opaque context structure allowing the caller to record details about the sample being scanned.
+ * @param[in,out] context   An opaque context structure allowing the caller to record details about the sample being scanned.
  * @return cl_error_t       CL_CLEAN, CL_VIRUS, or an error code if an error occured during the scan.
  */
-extern int cl_scanfile_callback(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
+extern cl_error_t cl_scanfile_callback(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
 
 /* ----------------------------------------------------------------------------
  * Database handling.
  */
-extern int cl_load(const char *path, struct cl_engine *engine, unsigned int *signo, unsigned int dboptions);
+
+/**
+ * @brief Load the signature databases found at the path.
+ *
+ * @param path          May be a file or directory.
+ * @param engine        The engine to load the signatures into
+ * @param[out] signo    The number of signatures loaded
+ * @param dboptions     Database load bitflag field. See the CL_DB_* defines, above.
+ * @return cl_error_t
+ */
+extern cl_error_t cl_load(const char *path, struct cl_engine *engine, unsigned int *signo, unsigned int dboptions);
+
+/**
+ * @brief Get the default database directory path.
+ *
+ * @return const char*
+ */
 extern const char *cl_retdbdir(void);
 
 /* ----------------------------------------------------------------------------
@@ -943,7 +1111,7 @@ extern struct cl_cvd *cl_cvdparse(const char *head);
  * @param file          Filepath of CVD file.
  * @return cl_error_t   CL_SUCCESS if success, else a CL_E* error code.
  */
-extern int cl_cvdverify(const char *file);
+extern cl_error_t cl_cvdverify(const char *file);
 
 /**
  * @brief Free a CVD header struct.
@@ -952,6 +1120,18 @@ extern int cl_cvdverify(const char *file);
  */
 extern void cl_cvdfree(struct cl_cvd *cvd);
 
+/**
+ * @brief Unpack a CVD file.
+ *
+ * Will verify the CVD is correctly signed unless the `dont_verify` parameter is true.
+ *
+ * @param file          Filepath of CVD file.
+ * @param dir           Destination directory.
+ * @param dont_verify   If true, don't verify the CVD.
+ * @return cl_error_t   CL_SUCCESS if success, else a CL_E* error code.
+ */
+extern cl_error_t cl_cvdunpack(const char *file, const char *dir, bool dont_verify);
+
 /* ----------------------------------------------------------------------------
  * DB directory stat functions.
  * Use these functions to watch for database changes.
@@ -973,7 +1153,7 @@ struct cl_stat {
  * @param[out] dbstat   dbstat handle.
  * @return cl_error_t   CL_SUCCESS if successfully initialized.
  */
-extern int cl_statinidir(const char *dirname, struct cl_stat *dbstat);
+extern cl_error_t cl_statinidir(const char *dirname, struct cl_stat *dbstat);
 
 /**
  * @brief Check the database directory for changes.
@@ -991,7 +1171,7 @@ extern int cl_statchkdir(const struct cl_stat *dbstat);
  * @return cl_error_t   CL_SUCCESS
  * @return cl_error_t   CL_ENULLARG
  */
-extern int cl_statfree(struct cl_stat *dbstat);
+extern cl_error_t cl_statfree(struct cl_stat *dbstat);
 
 /**
  * @brief Count the number of signatures in a database file or directory.
@@ -1001,7 +1181,7 @@ extern int cl_statfree(struct cl_stat *dbstat);
  * @param[out] sigs     The number of sigs.
  * @return cl_error_t   CL_SUCCESS if success, else a CL_E* error type.
  */
-extern int cl_countsigs(const char *path, unsigned int countoptions, unsigned int *sigs);
+extern cl_error_t cl_countsigs(const char *path, unsigned int countoptions, unsigned int *sigs);
 
 /* ----------------------------------------------------------------------------
  * Software versions.
@@ -1026,7 +1206,7 @@ extern const char *cl_retver(void);
 /* ----------------------------------------------------------------------------
  * Others.
  */
-extern const char *cl_strerror(int clerror);
+extern const char *cl_strerror(cl_error_t clerror);
 
 /* ----------------------------------------------------------------------------
  * Custom data scanning.
@@ -1062,7 +1242,7 @@ typedef off_t (*clcb_pread)(void *handle, void *buf, size_t count, off_t offset)
  * @brief Open a map given a handle.
  *
  * Open a map for scanning custom data accessed by a handle and pread (lseek +
- * read)-like interface. For example a WIN32 HANDLE.
+ * read)-like interface. For example a file descriptor or a WIN32 HANDLE.
  * By default fmap will use aging to discard old data, unless you tell it not
  * to.
  *
@@ -1071,12 +1251,12 @@ typedef off_t (*clcb_pread)(void *handle, void *buf, size_t count, off_t offset)
  * @param handle        A handle that may be accessed using lseek + read.
  * @param offset        Initial offset to start scanning.
  * @param len           Length of the data from the start (not the offset).
- * @param use_aging     Set to a non-zero value to enable aging.
  * @param pread_cb      A callback function to read data from the handle.
+ * @param use_aging     Set to a non-zero value to enable aging.
  * @return cl_fmap_t*   A map representing the handle interface.
  */
 extern cl_fmap_t *cl_fmap_open_handle(void *handle, size_t offset, size_t len,
-                                      clcb_pread, int use_aging);
+                                      clcb_pread pread_cb, int use_aging);
 
 /**
  * @brief Open a map given a buffer.
@@ -1119,7 +1299,7 @@ extern void cl_fmap_close(cl_fmap_t *);
  *                      signature matched. Another CL_E* error code if an
  *                      error occured.
  */
-extern int cl_scanmap_callback(cl_fmap_t *map, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
+extern cl_error_t cl_scanmap_callback(cl_fmap_t *map, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
 
 /* ----------------------------------------------------------------------------
  * Crypto/hashing functions
