@@ -62,13 +62,9 @@
 #include <libxml/parser.h>
 #endif
 
-#ifdef HAVE_LTDL
-#include "ltdl.h"
-#else // !HAVE_LTDL
 #ifndef _WIN32
 #include <dlfcn.h>
 #endif
-#endif // !HAVE_LTDL
 
 #include "clamav.h"
 #include "others.h"
@@ -84,6 +80,8 @@
 #include "stats.h"
 #include "json_api.h"
 
+#include "clamav_rust.h"
+
 cl_unrar_error_t (*cli_unrar_open)(const char *filename, void **hArchive, char **comment, uint32_t *comment_size, uint8_t debug_flag);
 cl_unrar_error_t (*cli_unrar_peek_file_header)(void *hArchive, unrar_metadata_t *file_metadata);
 cl_unrar_error_t (*cli_unrar_extract_file)(void *hArchive, const char *destPath, char *outputBuffer);
@@ -96,91 +94,17 @@ static int is_rar_inited = 0;
 #define PASTE2(a, b) a #b
 #define PASTE(a, b) PASTE2(a, b)
 
-#ifdef HAVE_LTDL
-
-static int warn_dlerror(const char *msg)
-{
-    const char *err = lt_dlerror();
-    if (err)
-        cli_warnmsg("%s: %s\n", msg, err);
-    else
-        cli_warnmsg("%s\n", err);
-    return 0;
-}
-
-static int lt_init(void)
-{
-    if (lt_dlinit()) {
-        warn_dlerror("Cannot init ltdl - unrar support unavailable");
-        return -1;
-    }
-    return 0;
-}
-
 static void *load_module(const char *name, const char *featurename)
 {
-    static const char *suffixes[] = {
-        LT_MODULE_EXT "." LIBCLAMAV_FULLVER,
-        PASTE(LT_MODULE_EXT ".", LIBCLAMAV_MAJORVER),
-        LT_MODULE_EXT,
-        "." LT_LIBEXT};
-
-    const char *searchpath;
-    const lt_dlinfo *info;
-    char modulename[128];
-    lt_dlhandle rhandle;
-    unsigned i;
-
-    if (lt_dladdsearchdir(SEARCH_LIBDIR)) {
-        cli_dbgmsg("lt_dladdsearchdir failed for %s\n", SEARCH_LIBDIR);
-    }
-
-    searchpath = lt_dlgetsearchpath();
-    if (!searchpath)
-        searchpath = "";
-
-    cli_dbgmsg("searching for %s, user-searchpath: %s\n", featurename, searchpath);
-    for (i = 0; i < sizeof(suffixes) / sizeof(suffixes[0]); i++) {
-        snprintf(modulename, sizeof(modulename), "%s%s", name, suffixes[i]);
-        rhandle = lt_dlopen(modulename);
-        if (rhandle)
-            break;
-        cli_dbgmsg("searching for %s: %s not found\n", featurename, modulename);
-    }
-
-    if (!rhandle) {
-        const char *err = lt_dlerror();
-        if (!err) err = "";
-#ifdef WARN_DLOPEN_FAIL
-        cli_warnmsg("Cannot dlopen %s: %s - %s support unavailable\n", name, err, featurename);
+#ifdef _WIN32
+    static const char *suffixes[] = {LT_MODULE_EXT};
 #else
-        cli_dbgmsg("Cannot dlopen %s: %s - %s support unavailable\n", name, err, featurename);
-#endif
-        return rhandle;
-    }
-
-    info = lt_dlgetinfo(rhandle);
-    if (info)
-        cli_dbgmsg("%s support loaded from %s %s\n", featurename, info->filename ? info->filename : "?", info->name ? info->name : "");
-    return (void *)rhandle;
-}
-
-static void *get_module_function(lt_dlhandle handle, const char *name)
-{
-    return lt_dlsym(handle, name);
-}
-
-#else // !HAVE_LTDL
-
-static void *load_module(const char *name, const char *featurename)
-{
     static const char *suffixes[] = {
-#ifndef _WIN32
         LT_MODULE_EXT "." LIBCLAMAV_FULLVER,
         PASTE(LT_MODULE_EXT ".", LIBCLAMAV_MAJORVER),
         LT_MODULE_EXT,
-#endif
         "." LT_LIBEXT};
+#endif
 
     const char *searchpath;
     char modulename[128];
@@ -191,16 +115,31 @@ static void *load_module(const char *name, const char *featurename)
     void *rhandle;
 #endif
 
-    searchpath = SEARCH_LIBDIR;
-
-    cli_dbgmsg("searching for %s, user-searchpath: %s\n", featurename, searchpath);
-    for (i = 0; i < sizeof(suffixes) / sizeof(suffixes[0]); i++) {
-        snprintf(modulename, sizeof(modulename), "%s%s", name, suffixes[i]);
-
 #ifdef _WIN32
+    /*
+     * First try a standard LoadLibraryA() without specifying a full path.
+     * For more information on the DLL search order, see:
+     *  https://docs.microsoft.com/en-us/windows/desktop/Dlls/dynamic-link-library-search-order
+     */
+    cli_dbgmsg("searching for %s\n", featurename);
+#else
+    /*
+     * First search using the provided SEARCH_LIBDIR (e.g. "<prefix>/lib")
+     * Known issue: If an older clamav version is already installed, the clamav
+     * unit tests using this function will load the older library version from
+     * the install path first.
+     */
+    searchpath = SEARCH_LIBDIR;
+    cli_dbgmsg("searching for %s, user-searchpath: %s\n", featurename, searchpath);
+#endif
+
+    for (i = 0; i < sizeof(suffixes) / sizeof(suffixes[0]); i++) {
+#ifdef _WIN32
+        snprintf(modulename, sizeof(modulename), "%s%s", name, suffixes[i]);
         rhandle = LoadLibraryA(modulename);
 #else  // !_WIN32
-        rhandle         = dlopen(modulename, RTLD_NOW);
+        snprintf(modulename, sizeof(modulename), "%s" PATHSEP "%s%s", searchpath, name, suffixes[i]);
+        rhandle = dlopen(modulename, RTLD_NOW);
 #endif // !_WIN32
         if (rhandle) {
             break;
@@ -209,6 +148,49 @@ static void *load_module(const char *name, const char *featurename)
         cli_dbgmsg("searching for %s: %s not found\n", featurename, modulename);
     }
 
+    if (NULL == rhandle) {
+        char *ld_library_path = NULL;
+        /*
+         * library not found.
+         * Try again using LD_LIBRARY_PATH environment variable for the path.
+         */
+        ld_library_path = getenv("LD_LIBRARY_PATH");
+        if (NULL != ld_library_path) {
+#define MAX_LIBRARY_PATHS 10
+            size_t token_index;
+            size_t tokens_count;
+            const char *tokens[MAX_LIBRARY_PATHS];
+
+            char *tokenized_library_path = NULL;
+
+            tokenized_library_path = strdup(ld_library_path);
+            tokens_count           = cli_strtokenize(tokenized_library_path, ':', MAX_LIBRARY_PATHS, tokens);
+
+            for (token_index = 0; token_index < tokens_count; token_index++) {
+                cli_dbgmsg("searching for %s, LD_LIBRARY_PATH: %s\n", featurename, tokens[token_index]);
+
+                for (i = 0; i < sizeof(suffixes) / sizeof(suffixes[0]); i++) {
+                    snprintf(modulename, sizeof(modulename), "%s" PATHSEP "%s%s", tokens[token_index], name, suffixes[i]);
+#ifdef _WIN32
+                    rhandle = LoadLibraryA(modulename);
+#else  // !_WIN32
+                    rhandle = dlopen(modulename, RTLD_NOW);
+#endif // !_WIN32
+                    if (rhandle) {
+                        break;
+                    }
+
+                    cli_dbgmsg("searching for %s: %s not found\n", featurename, modulename);
+                }
+
+                if (rhandle) {
+                    break;
+                }
+            }
+            free(tokenized_library_path);
+        }
+    }
+
     if (NULL == rhandle) {
 #ifdef _WIN32
         char *err     = NULL;
@@ -287,7 +269,7 @@ static void *get_module_function(HMODULE handle, const char *name)
 static void *get_module_function(void *handle, const char *name)
 {
     void *procAddress = NULL;
-    procAddress = dlsym(handle, name);
+    procAddress       = dlsym(handle, name);
     if (NULL == procAddress) {
         const char *err = dlerror();
         if (NULL == err) {
@@ -300,8 +282,6 @@ static void *get_module_function(void *handle, const char *name)
 }
 #endif // !_WIN32
 
-#endif // !HAVE_LTDL
-
 static void rarload(void)
 {
 #ifndef UNRAR_LINKED
@@ -317,6 +297,13 @@ static void rarload(void)
 
     if (have_rar) return;
 
+#ifdef UNRAR_LINKED
+    cli_unrar_open             = unrar_open;
+    cli_unrar_peek_file_header = unrar_peek_file_header;
+    cli_unrar_extract_file     = unrar_extract_file;
+    cli_unrar_skip_file        = unrar_skip_file;
+    cli_unrar_close            = unrar_close;
+#else
     rhandle = load_module("libclamunrar_iface", "unrar");
     if (NULL == rhandle)
         return;
@@ -332,6 +319,8 @@ static void rarload(void)
         cli_warnmsg("UnRAR support unavailable\n");
         return;
     }
+#endif
+
     have_rar = 1;
 }
 
@@ -350,7 +339,7 @@ unsigned int cl_retflevel(void)
     return CL_FLEVEL;
 }
 
-const char *cl_strerror(int clerror)
+const char *cl_strerror(cl_error_t clerror)
 {
     switch (clerror) {
         /* libclamav specific codes */
@@ -438,16 +427,15 @@ cl_error_t cl_init(unsigned int initoptions)
 
     UNUSEDPARAM(initoptions);
 
+    /* Rust logging initialization */
+    if (!clrs_log_init()) {
+        cli_dbgmsg("Unexpected problem occurred while setting up rust logging... continuing without rust logging. \
+                    Please submit an issue to https://github.com/Cisco-Talos/clamav");
+    }
+
     cl_initialize_crypto();
 
-    /* put dlopen() stuff here, etc. */
-#ifdef HAVE_LTDL
-    if (lt_init() == 0) {
-        rarload();
-    }
-#else
     rarload();
-#endif
 
     gettimeofday(&tv, (struct timezone *)0);
     srand(pid + tv.tv_usec * (pid + 1) + clock());
@@ -634,7 +622,20 @@ cl_error_t cl_engine_set_num(struct cl_engine *engine, enum cl_engine_field fiel
             engine->maxscansize = num;
             break;
         case CL_ENGINE_MAX_FILESIZE:
-            engine->maxfilesize = num;
+            /* We have a limit of around 2GB (INT_MAX - 2). Enforce it here.
+             *
+             * TODO: Large file support is large-ly untested. Remove this restriction and test with a large set of large files of various types.
+             * libclamav's integer type safety has come a long way since 2014, so it's possible we could lift this restriction, but at least one
+             * of the parsers is bound to behave badly with large files. */
+            if ((uint64_t)num > INT_MAX - 2) {
+                if ((uint64_t)num > (uint64_t)2 * 1024 * 1024 * 1024 && num != LLONG_MAX) {
+                    // If greater than 2GB, warn. If exactly at 2GB, don't hassle the user.
+                    cli_warnmsg("Max file-size was set to %lld bytes. Unfortunately, scanning files greater than 2147483647 bytes (2 GiB - 1) is not supported.\n", num);
+                }
+                engine->maxfilesize = INT_MAX - 2;
+            } else {
+                engine->maxfilesize = num;
+            }
             break;
         case CL_ENGINE_MAX_RECURSION:
             if (!num) {
@@ -739,7 +740,7 @@ cl_error_t cl_engine_set_num(struct cl_engine *engine, enum cl_engine_field fiel
             } else {
                 engine->engine_options &= ~(ENGINE_OPTIONS_DISABLE_CACHE);
                 if (!(engine->cache))
-                    cli_cache_init(engine);
+                    clean_cache_init(engine);
             }
             break;
         case CL_ENGINE_DISABLE_PE_STATS:
@@ -972,16 +973,22 @@ struct cl_settings *cl_engine_settings_copy(const struct cl_engine *engine)
     settings->bytecode_mode       = engine->bytecode_mode;
     settings->pua_cats            = engine->pua_cats ? strdup(engine->pua_cats) : NULL;
 
-    settings->cb_pre_cache   = engine->cb_pre_cache;
-    settings->cb_pre_scan    = engine->cb_pre_scan;
-    settings->cb_post_scan   = engine->cb_post_scan;
-    settings->cb_virus_found = engine->cb_virus_found;
-    settings->cb_sigload     = engine->cb_sigload;
-    settings->cb_sigload_ctx = engine->cb_sigload_ctx;
-    settings->cb_hash        = engine->cb_hash;
-    settings->cb_meta        = engine->cb_meta;
-    settings->cb_file_props  = engine->cb_file_props;
-    settings->engine_options = engine->engine_options;
+    settings->cb_pre_cache                   = engine->cb_pre_cache;
+    settings->cb_pre_scan                    = engine->cb_pre_scan;
+    settings->cb_post_scan                   = engine->cb_post_scan;
+    settings->cb_virus_found                 = engine->cb_virus_found;
+    settings->cb_sigload                     = engine->cb_sigload;
+    settings->cb_sigload_ctx                 = engine->cb_sigload_ctx;
+    settings->cb_sigload_progress            = engine->cb_sigload_progress;
+    settings->cb_sigload_progress_ctx        = engine->cb_sigload_progress_ctx;
+    settings->cb_engine_compile_progress     = engine->cb_engine_compile_progress;
+    settings->cb_engine_compile_progress_ctx = engine->cb_engine_compile_progress_ctx;
+    settings->cb_engine_free_progress        = engine->cb_engine_free_progress;
+    settings->cb_engine_free_progress_ctx    = engine->cb_engine_free_progress_ctx;
+    settings->cb_hash                        = engine->cb_hash;
+    settings->cb_meta                        = engine->cb_meta;
+    settings->cb_file_props                  = engine->cb_file_props;
+    settings->engine_options                 = engine->engine_options;
 
     settings->cb_stats_add_sample      = engine->cb_stats_add_sample;
     settings->cb_stats_remove_sample   = engine->cb_stats_remove_sample;
@@ -1047,15 +1054,21 @@ cl_error_t cl_engine_settings_apply(struct cl_engine *engine, const struct cl_se
         engine->pua_cats = NULL;
     }
 
-    engine->cb_pre_cache   = settings->cb_pre_cache;
-    engine->cb_pre_scan    = settings->cb_pre_scan;
-    engine->cb_post_scan   = settings->cb_post_scan;
-    engine->cb_virus_found = settings->cb_virus_found;
-    engine->cb_sigload     = settings->cb_sigload;
-    engine->cb_sigload_ctx = settings->cb_sigload_ctx;
-    engine->cb_hash        = settings->cb_hash;
-    engine->cb_meta        = settings->cb_meta;
-    engine->cb_file_props  = settings->cb_file_props;
+    engine->cb_pre_cache                   = settings->cb_pre_cache;
+    engine->cb_pre_scan                    = settings->cb_pre_scan;
+    engine->cb_post_scan                   = settings->cb_post_scan;
+    engine->cb_virus_found                 = settings->cb_virus_found;
+    engine->cb_sigload                     = settings->cb_sigload;
+    engine->cb_sigload_ctx                 = settings->cb_sigload_ctx;
+    engine->cb_sigload_progress            = settings->cb_sigload_progress;
+    engine->cb_sigload_progress_ctx        = settings->cb_sigload_progress_ctx;
+    engine->cb_engine_compile_progress     = settings->cb_engine_compile_progress;
+    engine->cb_engine_compile_progress_ctx = settings->cb_engine_compile_progress_ctx;
+    engine->cb_engine_free_progress        = settings->cb_engine_free_progress;
+    engine->cb_engine_free_progress_ctx    = settings->cb_engine_free_progress_ctx;
+    engine->cb_hash                        = settings->cb_hash;
+    engine->cb_meta                        = settings->cb_meta;
+    engine->cb_file_props                  = settings->cb_file_props;
 
     engine->cb_stats_add_sample      = settings->cb_stats_add_sample;
     engine->cb_stats_remove_sample   = settings->cb_stats_remove_sample;
@@ -1089,14 +1102,14 @@ cl_error_t cl_engine_settings_free(struct cl_settings *settings)
     return CL_SUCCESS;
 }
 
-void cli_append_virus_if_heur_exceedsmax(cli_ctx *ctx, char *vname)
+void cli_append_potentially_unwanted_if_heur_exceedsmax(cli_ctx *ctx, char *vname)
 {
     if (!ctx->limit_exceeded) {
         ctx->limit_exceeded = true; // guard against adding an alert (or metadata) a million times for non-fatal exceeds-max conditions
                                     // TODO: consider changing this from a bool to a threshold so we could at least see more than 1 limits exceeded
 
         if (SCAN_HEURISTIC_EXCEEDS_MAX) {
-            cli_append_possibly_unwanted(ctx, vname);
+            cli_append_potentially_unwanted(ctx, vname);
             cli_dbgmsg("%s: scanning may be incomplete and additional analysis needed for this file.\n", vname);
         }
 
@@ -1114,46 +1127,56 @@ cl_error_t cli_checklimits(const char *who, cli_ctx *ctx, unsigned long need1, u
     cl_error_t ret = CL_SUCCESS;
     unsigned long needed;
 
-    /* if called without limits, go on, unpack, scan */
-    if (!ctx) return ret;
+    if (!ctx) {
+        /* if called without limits, go on, unpack, scan */
+        goto done;
+    }
 
     needed = (need1 > need2) ? need1 : need2;
     needed = (needed > need3) ? needed : need3;
 
-    /* Enforce timelimit */
-    if (CL_ETIMEOUT == (ret = cli_checktimelimit(ctx))) {
-        /* Abort the scan ... */
-        ret = CL_ETIMEOUT;
+    /* Enforce global time limit, if limit enabled */
+    ret = cli_checktimelimit(ctx);
+    if (CL_SUCCESS != ret) {
+        // Exceeding the time limit will abort the scan.
+        // The logic for this and the possible heuristic is done inside the cli_checktimelimit function.
+        goto done;
     }
 
-    /* Enforce global scan-size limit */
-    if (needed && ctx->engine->maxscansize) {
-        /* if the remaining scansize is too small... */
-        if (ctx->engine->maxscansize - ctx->scansize < needed) {
-            /* Skip this file */
-            cli_dbgmsg("%s: scansize exceeded (initial: %lu, consumed: %lu, needed: %lu)\n", who, (unsigned long int)ctx->engine->maxscansize, (unsigned long int)ctx->scansize, needed);
-            ret = CL_EMAXSIZE;
-            cli_append_virus_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxScanSize");
-        }
+    /* Enforce global scan-size limit, if limit enabled */
+    if (needed && (ctx->engine->maxscansize != 0) && (ctx->engine->maxscansize - ctx->scansize < needed)) {
+        /* The size needed is greater than the remaining scansize ... Skip this file. */
+        cli_dbgmsg("%s: scansize exceeded (initial: %lu, consumed: %lu, needed: %lu)\n", who, (unsigned long int)ctx->engine->maxscansize, (unsigned long int)ctx->scansize, needed);
+        ret = CL_EMAXSIZE;
+        cli_append_potentially_unwanted_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxScanSize");
+        goto done;
     }
 
-    /* Enforce per-file file-size limit */
-    if (needed && ctx->engine->maxfilesize && ctx->engine->maxfilesize < needed) {
-        /* Skip this file */
+    /* Enforce per-file file-size limit, if limit enabled */
+    if (needed && (ctx->engine->maxfilesize != 0) && (ctx->engine->maxfilesize < needed)) {
+        /* The size needed is greater than that limit ... Skip this file. */
         cli_dbgmsg("%s: filesize exceeded (allowed: %lu, needed: %lu)\n", who, (unsigned long int)ctx->engine->maxfilesize, needed);
         ret = CL_EMAXSIZE;
-        cli_append_virus_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFileSize");
+        cli_append_potentially_unwanted_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFileSize");
+        goto done;
     }
 
-    /* Enforce limit on number of embedded files */
-    if (ctx->engine->maxfiles && ctx->scannedfiles >= ctx->engine->maxfiles) {
-        /* Abort the scan ... */
+    /* Enforce limit on number of embedded files, if limit enabled */
+    if ((ctx->engine->maxfiles != 0) && (ctx->scannedfiles >= ctx->engine->maxfiles)) {
+        /* This file would exceed the max # of files ... Skip this file. */
         cli_dbgmsg("%s: files limit reached (max: %u)\n", who, ctx->engine->maxfiles);
         ret = CL_EMAXFILES;
-        cli_append_virus_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFiles");
-        ctx->abort_scan = true;
+        cli_append_potentially_unwanted_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFiles");
+
+        // We don't need to set the `ctx->abort_scan` flag here.
+        // We want `cli_magic_scan()` to finish scanning the current file, but not any future files.
+        // We keep track of the # scanned files with `ctx->scannedfiles`, and that should be sufficient to prevent
+        // additional files from being scanned.
+        goto done;
     }
 
+done:
+
     return ret;
 }
 
@@ -1191,10 +1214,8 @@ cl_error_t cli_checktimelimit(cli_ctx *ctx)
     if (ctx->time_limit.tv_sec != 0) {
         struct timeval now;
         if (gettimeofday(&now, NULL) == 0) {
-            if (now.tv_sec > ctx->time_limit.tv_sec) {
-                ctx->abort_scan = true;
-                ret             = CL_ETIMEOUT;
-            } else if (now.tv_sec == ctx->time_limit.tv_sec && now.tv_usec > ctx->time_limit.tv_usec) {
+            if ((now.tv_sec > ctx->time_limit.tv_sec) ||
+                (now.tv_sec == ctx->time_limit.tv_sec && now.tv_usec > ctx->time_limit.tv_usec)) {
                 ctx->abort_scan = true;
                 ret             = CL_ETIMEOUT;
             }
@@ -1202,7 +1223,10 @@ cl_error_t cli_checktimelimit(cli_ctx *ctx)
     }
 
     if (CL_ETIMEOUT == ret) {
-        cli_append_virus_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxScanTime");
+        cli_append_potentially_unwanted_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxScanTime");
+
+        // abort_scan flag is set so that in cli_magic_scan() we *will* stop scanning, even if we lose the status code.
+        ctx->abort_scan = true;
     }
 
 done:
@@ -1279,70 +1303,88 @@ char *cli_hashfile(const char *filename, int type)
 /* Function: unlink
         unlink() with error checking
 */
-int cli_unlink(const char *pathname)
+cl_error_t cli_unlink(const char *pathname)
 {
     if (unlink(pathname) == -1) {
 #ifdef _WIN32
         /* Windows may fail to unlink a file if it is marked read-only,
-		 * even if the user has permissions to delete the file. */
+         * even if the user has permissions to delete the file. */
         if (-1 == _chmod(pathname, _S_IWRITE)) {
             char err[128];
-            cli_warnmsg("cli_unlink: _chmod failure - %s\n", cli_strerror(errno, err, sizeof(err)));
-            return 1;
+            cli_warnmsg("cli_unlink: _chmod failure for %s - %s\n", pathname, cli_strerror(errno, err, sizeof(err)));
+            return CL_EUNLINK;
         } else if (unlink(pathname) == -1) {
             char err[128];
-            cli_warnmsg("cli_unlink: unlink failure - %s\n", cli_strerror(errno, err, sizeof(err)));
-            return 1;
+            cli_warnmsg("cli_unlink: unlink failure for %s - %s\n", pathname, cli_strerror(errno, err, sizeof(err)));
+            return CL_EUNLINK;
         }
-        return 0;
+        return CL_SUCCESS;
 #else
         char err[128];
-        cli_warnmsg("cli_unlink: unlink failure - %s\n", cli_strerror(errno, err, sizeof(err)));
-        return 1;
+        cli_warnmsg("cli_unlink: unlink failure for %s - %s\n", pathname, cli_strerror(errno, err, sizeof(err)));
+        return CL_EUNLINK;
 #endif
     }
-    return 0;
+    return CL_SUCCESS;
 }
 
-void cli_virus_found_cb(cli_ctx *ctx)
+void cli_virus_found_cb(cli_ctx *ctx, const char *virname)
 {
-    if (ctx->engine->cb_virus_found)
-        ctx->engine->cb_virus_found(fmap_fd(ctx->fmap), (const char *)*ctx->virname, ctx->cb_ctx);
-}
-
-cl_error_t cli_append_possibly_unwanted(cli_ctx *ctx, const char *virname)
-{
-    if (SCAN_ALLMATCHES) {
-        return cli_append_virus(ctx, virname);
-    } else if (SCAN_HEURISTIC_PRECEDENCE) {
-        return cli_append_virus(ctx, virname);
-    } else if (ctx->num_viruses == 0 && ctx->virname != NULL && *ctx->virname == NULL) {
-        ctx->found_possibly_unwanted = 1;
-        ctx->num_viruses++;
-        *ctx->virname = virname;
+    if (ctx->engine->cb_virus_found) {
+        ctx->engine->cb_virus_found(
+            fmap_fd(ctx->fmap),
+            virname,
+            ctx->cb_ctx);
     }
-    return CL_CLEAN;
 }
 
-cl_error_t cli_append_virus(cli_ctx *ctx, const char *virname)
+/**
+ * @brief Add an indicator to the scan evidence.
+ *
+ * @param ctx
+ * @param virname Name of the indicator
+ * @param type Type of the indicator
+ * @return Returns CL_SUCCESS if added and IS in ALLMATCH mode, or if was PUA and not in HEURISTIC-PRECEDENCE-mode.
+ * @return Returns CL_VIRUS if added and NOT in ALLMATCH mode, or if was PUA and not in ALLMATCH but IS in HEURISTIC-PRECEDENCE-mode.
+ * @return Returns some other error code like CL_ERROR or CL_EMEM if something went wrong.
+ */
+static cl_error_t append_virus(cli_ctx *ctx, const char *virname, IndicatorType type)
 {
-    if (ctx->virname == NULL) {
-        return CL_CLEAN;
+    cl_error_t status             = CL_ERROR;
+    FFIError *add_indicator_error = NULL;
+    bool add_successful;
+
+    char *location = NULL;
+
+    if (ctx->evidence == NULL) {
+        // evidence storage not initialized, cannot continue.
+        status = CL_SUCCESS;
+        goto done;
     }
+
     if ((ctx->fmap != NULL) &&
         (ctx->recursion_stack != NULL) &&
         (CL_VIRUS != cli_check_fp(ctx, virname))) {
-        return CL_CLEAN;
-    }
-    if (!SCAN_ALLMATCHES && ctx->num_viruses != 0) {
-        if (SCAN_HEURISTIC_PRECEDENCE) {
-            return CL_CLEAN;
-        }
+        // FP signature found for one of the layers. Ignore indicator.
+        status = CL_SUCCESS;
+        goto done;
     }
 
-    ctx->num_viruses++;
-    *ctx->virname = virname;
-    cli_virus_found_cb(ctx);
+    add_successful = evidence_add_indicator(
+        ctx->evidence,
+        virname,
+        type,
+        &add_indicator_error);
+    if (!add_successful) {
+        cli_errmsg("Failed to add indicator to scan evidence: %s\n", ffierror_fmt(add_indicator_error));
+        status = CL_ERROR;
+        goto done;
+    }
+
+    if (type == IndicatorType_Strong) {
+        // Run that virus callback which in clamscan says "<signature name> FOUND"
+        cli_virus_found_cb(ctx, virname);
+    }
 
 #if HAVE_JSON
     if (SCAN_COLLECT_METADATA && ctx->wrkproperty) {
@@ -1351,37 +1393,92 @@ cl_error_t cli_append_virus(cli_ctx *ctx, const char *virname)
             arrobj = json_object_new_array();
             if (NULL == arrobj) {
                 cli_errmsg("cli_append_virus: no memory for json virus array\n");
-                return CL_EMEM;
+                status = CL_EMEM;
+                goto done;
             }
             json_object_object_add(ctx->wrkproperty, "Viruses", arrobj);
         }
         virobj = json_object_new_string(virname);
         if (NULL == virobj) {
             cli_errmsg("cli_append_virus: no memory for json virus name object\n");
-            return CL_EMEM;
+            status = CL_EMEM;
+            goto done;
         }
         json_object_array_add(arrobj, virobj);
     }
 #endif
-    return CL_VIRUS;
+
+    if (SCAN_ALLMATCHES) {
+        // Never break.
+        status = CL_SUCCESS;
+    } else {
+        // Usually break.
+        switch (type) {
+            case IndicatorType_Strong: {
+                status = CL_VIRUS;
+                // abort_scan flag is set so that in cli_magic_scan() we *will* stop scanning, even if we lose the status code.
+                ctx->abort_scan = true;
+                break;
+            }
+            case IndicatorType_PotentiallyUnwanted: {
+                status = CL_SUCCESS;
+                break;
+            }
+            default: {
+                status = CL_SUCCESS;
+            }
+        }
+    }
+
+done:
+    if (NULL != location) {
+        free(location);
+    }
+
+    return status;
+}
+
+cl_error_t cli_append_potentially_unwanted(cli_ctx *ctx, const char *virname)
+{
+    if (SCAN_HEURISTIC_PRECEDENCE) {
+        return append_virus(ctx, virname, IndicatorType_Strong);
+    } else {
+        return append_virus(ctx, virname, IndicatorType_PotentiallyUnwanted);
+    }
+}
+
+cl_error_t cli_append_virus(cli_ctx *ctx, const char *virname)
+{
+    if ((strncmp(virname, "PUA.", 4) == 0) ||
+        (strncmp(virname, "Heuristics.", 11) == 0) ||
+        (strncmp(virname, "BC.Heuristics.", 14) == 0)) {
+        return cli_append_potentially_unwanted(ctx, virname);
+    } else {
+        return append_virus(ctx, virname, IndicatorType_Strong);
+    }
 }
 
 const char *cli_get_last_virus(const cli_ctx *ctx)
 {
-    if (!ctx || !ctx->virname || !(*ctx->virname))
+    if (!ctx || !ctx->evidence) {
         return NULL;
-    return *ctx->virname;
+    }
+
+    return evidence_get_last_alert(ctx->evidence);
 }
 
 const char *cli_get_last_virus_str(const cli_ctx *ctx)
 {
     const char *ret;
-    if ((ret = cli_get_last_virus(ctx)))
+
+    if (NULL != (ret = cli_get_last_virus(ctx))) {
         return ret;
+    }
+
     return "";
 }
 
-cl_error_t cli_recursion_stack_push(cli_ctx *ctx, cl_fmap_t *map, cli_file_t type, bool is_new_buffer)
+cl_error_t cli_recursion_stack_push(cli_ctx *ctx, cl_fmap_t *map, cli_file_t type, bool is_new_buffer, uint32_t attributes)
 {
     cl_error_t status = CL_SUCCESS;
 
@@ -1389,7 +1486,7 @@ cl_error_t cli_recursion_stack_push(cli_ctx *ctx, cl_fmap_t *map, cli_file_t typ
     recursion_level_t *new_container     = NULL;
 
     // Check the regular limits
-    if (CL_SUCCESS != (status = cli_checklimits("cli_updatelimits", ctx, map->len, 0, 0))) {
+    if (CL_SUCCESS != (status = cli_checklimits("cli_recursion_stack_push", ctx, map->len, 0, 0))) {
         cli_dbgmsg("cli_recursion_stack_push: Some content was skipped. The scan result will not be cached.\n");
         emax_reached(ctx); // Disable caching for all recursion layers.
         goto done;
@@ -1400,7 +1497,7 @@ cl_error_t cli_recursion_stack_push(cli_ctx *ctx, cl_fmap_t *map, cli_file_t typ
         cli_dbgmsg("cli_recursion_stack_push: Archive recursion limit exceeded (%u, max: %u)\n", ctx->recursion_level, ctx->engine->max_recursion_level);
         cli_dbgmsg("cli_recursion_stack_push: Some content was skipped. The scan result will not be cached.\n");
         emax_reached(ctx); // Disable caching for all recursion layers.
-        cli_append_virus_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxRecursion");
+        cli_append_potentially_unwanted_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxRecursion");
         status = CL_EMAXREC;
         goto done;
     }
@@ -1422,17 +1519,23 @@ cl_error_t cli_recursion_stack_push(cli_ctx *ctx, cl_fmap_t *map, cli_file_t typ
         new_container->recursion_level_buffer_fmap = current_container->recursion_level_buffer_fmap + 1;
     }
 
-    if (ctx->next_layer_is_normalized) {
-        // Normalized layers should be ignored when using the get_type() and get_intermediate_type()
-        // functions so that signatures that specify the container or intermediates need not account
-        // for normalized layers "contained in" HTML / Javascript / etc.
-        new_container->is_normalized_layer = true;
-        ctx->next_layer_is_normalized      = false;
+    // Apply the requested next-layer attributes.
+    //
+    // Note that this is how we also keep track of normalized layers.
+    // Normalized layers should be ignored when using the get_type() and get_intermediate_type()
+    // functions so that signatures that specify the container or intermediates need not account
+    // for normalized layers "contained in" HTML / Javascript / etc.
+    new_container->attributes = attributes;
+
+    // If the current layer is marked "decrypted", all child-layers are also marked "decrypted".
+    if (current_container->attributes & LAYER_ATTRIBUTES_DECRYPTED) {
+        new_container->attributes |= LAYER_ATTRIBUTES_DECRYPTED;
     }
 
     ctx->fmap = new_container->fmap;
 
 done:
+
     return status;
 }
 
@@ -1497,7 +1600,7 @@ static int recursion_stack_get(cli_ctx *ctx, int index)
     }
 
     while (current_layer >= desired_layer && current_layer > 0) {
-        if (ctx->recursion_stack[current_layer].is_normalized_layer) {
+        if (ctx->recursion_stack[current_layer].attributes & LAYER_ATTRIBUTES_NORMALIZED) {
             // The current layer is normalized, so we should step back an extra layer
             // It's okay if desired_layer goes negative.
             desired_layer--;
@@ -1778,6 +1881,11 @@ void cl_engine_set_clcb_pre_cache(struct cl_engine *engine, clcb_pre_cache callb
     engine->cb_pre_cache = callback;
 }
 
+void cl_engine_set_clcb_file_inspection(struct cl_engine *engine, clcb_file_inspection callback)
+{
+    engine->cb_file_inspection = callback;
+}
+
 void cl_engine_set_clcb_pre_scan(struct cl_engine *engine, clcb_pre_scan callback)
 {
     engine->cb_pre_scan = callback;
@@ -1799,6 +1907,24 @@ void cl_engine_set_clcb_sigload(struct cl_engine *engine, clcb_sigload callback,
     engine->cb_sigload_ctx = callback ? context : NULL;
 }
 
+void cl_engine_set_clcb_sigload_progress(struct cl_engine *engine, clcb_progress callback, void *context)
+{
+    engine->cb_sigload_progress     = callback;
+    engine->cb_sigload_progress_ctx = callback ? context : NULL;
+}
+
+void cl_engine_set_clcb_engine_compile_progress(struct cl_engine *engine, clcb_progress callback, void *context)
+{
+    engine->cb_engine_compile_progress     = callback;
+    engine->cb_engine_compile_progress_ctx = callback ? context : NULL;
+}
+
+void cl_engine_set_clcb_engine_free_progress(struct cl_engine *engine, clcb_progress callback, void *context)
+{
+    engine->cb_engine_free_progress     = callback;
+    engine->cb_engine_free_progress_ctx = callback ? context : NULL;
+}
+
 void cl_engine_set_clcb_hash(struct cl_engine *engine, clcb_hash callback)
 {
     engine->cb_hash = callback;
@@ -1813,3 +1939,16 @@ void cl_engine_set_clcb_file_props(struct cl_engine *engine, clcb_file_props cal
 {
     engine->cb_file_props = callback;
 }
+
+uint8_t cli_get_debug_flag()
+{
+    return cli_debug_flag;
+}
+
+uint8_t cli_set_debug_flag(uint8_t debug_flag)
+{
+    uint8_t was    = cli_debug_flag;
+    cli_debug_flag = debug_flag;
+
+    return was;
+}
