@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014, 2017-2022 Cisco and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2014-2022 Cisco and/or its affiliates. All rights reserved.
  *
  *  Author: Shawn Webb
  *
@@ -231,7 +231,7 @@ static char *pdf_decrypt_string(struct pdf_struct *pdf, struct pdf_obj *obj, con
     enum enc_method enc;
 
     /* handled only once in cli_pdf() */
-    //pdf_handle_enc(pdf);
+    // pdf_handle_enc(pdf);
     if (pdf->flags & (1 << DECRYPTABLE_PDF)) {
         enc = get_enc_method(pdf, obj);
         return decrypt_any(pdf, obj->id, in, length, enc);
@@ -254,7 +254,7 @@ char *pdf_finalize_string(struct pdf_struct *pdf, struct pdf_obj *obj, const cha
         return NULL;
     memcpy(wrkstr, in, len);
 
-    //cli_errmsg("pdf_final: start(%d):   %s\n", wrklen, wrkstr);
+    // cli_errmsg("pdf_final: start(%d):   %s\n", wrklen, wrkstr);
 
     /* convert PDF specific escape sequences, like octal sequences */
     /* TODO: replace the escape sequences directly in the wrkstr   */
@@ -331,7 +331,7 @@ char *pdf_finalize_string(struct pdf_struct *pdf, struct pdf_obj *obj, const cha
         wrklen = outlen;
     }
 
-    //cli_errmsg("pdf_final: escaped(%d): %s\n", wrklen, wrkstr);
+    // cli_errmsg("pdf_final: escaped(%d): %s\n", wrklen, wrkstr);
 
     /* check for encryption and decrypt */
     if (pdf->flags & (1 << ENCRYPTED_PDF)) {
@@ -353,7 +353,7 @@ char *pdf_finalize_string(struct pdf_struct *pdf, struct pdf_obj *obj, const cha
         }
     }
 
-    //cli_errmsg("pdf_final: decrypt(%d): %s\n", wrklen, wrkstr);
+    // cli_errmsg("pdf_final: decrypt(%d): %s\n", wrklen, wrkstr);
 
     /* check for UTF-* and convert to UTF-8 */
     for (i = 0; i < wrklen; ++i) {
@@ -369,7 +369,7 @@ char *pdf_finalize_string(struct pdf_struct *pdf, struct pdf_obj *obj, const cha
         wrkstr = output;
     }
 
-    //cli_errmsg("pdf_final: postutf(%d): %s\n", wrklen, wrkstr);
+    // cli_errmsg("pdf_final: postutf(%d): %s\n", wrklen, wrkstr);
 
     return wrkstr;
 }
@@ -382,6 +382,11 @@ char *pdf_parse_string(struct pdf_struct *pdf, struct pdf_obj *obj, const char *
     char *res = NULL;
     uint32_t objid;
 
+    if (PDF_OBJECT_RECURSION_LIMIT < pdf->parse_recursion_depth) {
+        cli_dbgmsg("pdf_parse_string: Recursion limit reached.\n");
+        return NULL;
+    }
+
     if (obj->objstm) {
         if (objsize > (size_t)(obj->objstm->streambuf_len - (objstart - obj->objstm->streambuf))) {
             /* Possible attempt to exploit bb11980 */
@@ -439,6 +444,7 @@ char *pdf_parse_string(struct pdf_struct *pdf, struct pdf_obj *obj, const char *
 
     p2 = (char *)(q + objsize);
     if (is_object_reference(p1, &p2, &objid)) {
+        cl_error_t ret;
         struct pdf_obj *newobj;
         char *begin, *p3;
         STATBUF sb;
@@ -450,6 +456,11 @@ char *pdf_parse_string(struct pdf_struct *pdf, struct pdf_obj *obj, const char *
         if (!(newobj))
             return NULL;
 
+        if (!CLI_ISCONTAINED(pdf->map, pdf->size, newobj->start, newobj->size)) {
+            cli_dbgmsg("pdf_parse_string: object not contained in PDF\n");
+            return NULL;
+        }
+
         if (newobj == obj)
             return NULL;
 
@@ -464,15 +475,19 @@ char *pdf_parse_string(struct pdf_struct *pdf, struct pdf_obj *obj, const char *
         objflags = newobj->flags;
         newobj->flags |= (1 << OBJ_FORCEDUMP);
 
-        if (pdf_extract_obj(pdf, newobj, PDF_EXTRACT_OBJ_NONE) != CL_SUCCESS)
+        pdf->parse_recursion_depth++;
+        ret = pdf_extract_obj(pdf, newobj, PDF_EXTRACT_OBJ_NONE);
+        pdf->parse_recursion_depth--;
+        if (ret != CL_SUCCESS) {
             return NULL;
+        }
 
         newobj->flags = objflags;
 
         if (!(newobj->path))
             return NULL;
 
-        fd = open(newobj->path, O_RDONLY);
+        fd = open(newobj->path, O_RDONLY | O_BINARY);
         if (fd == -1) {
             cli_unlink(newobj->path);
             free(newobj->path);
@@ -517,7 +532,9 @@ char *pdf_parse_string(struct pdf_struct *pdf, struct pdf_obj *obj, const char *
             switch (*p3) {
                 case '(':
                 case '<':
+                    pdf->parse_recursion_depth++;
                     res = pdf_parse_string(pdf, obj, p3, objsize2, NULL, NULL, meta);
+                    pdf->parse_recursion_depth--;
                     break;
                 default:
                     res = pdf_finalize_string(pdf, obj, begin, objsize2);
@@ -664,6 +681,11 @@ struct pdf_dict *pdf_parse_dict(struct pdf_struct *pdf, struct pdf_obj *obj, siz
     if (!(pdf) || !(obj) || !(begin))
         return NULL;
 
+    if (PDF_OBJECT_RECURSION_LIMIT < pdf->parse_recursion_depth) {
+        cli_dbgmsg("pdf_parse_dict: Recursion limit reached\n");
+        return NULL;
+    }
+
     objstart = (obj->objstm) ? (const char *)(obj->start + obj->objstm->streambuf)
                              : (const char *)(obj->start + pdf->map);
 
@@ -810,23 +832,31 @@ struct pdf_dict *pdf_parse_dict(struct pdf_struct *pdf, struct pdf_obj *obj, siz
 
         switch (begin[0]) {
             case '(':
-                val   = pdf_parse_string(pdf, obj, begin, end - objstart, NULL, &p1, NULL);
+                pdf->parse_recursion_depth++;
+                val = pdf_parse_string(pdf, obj, begin, end - objstart, NULL, &p1, NULL);
+                pdf->parse_recursion_depth--;
                 begin = p1 + 2;
                 break;
             case '[':
-                arr   = pdf_parse_array(pdf, obj, end - objstart, begin, &p1);
+                pdf->parse_recursion_depth++;
+                arr = pdf_parse_array(pdf, obj, end - objstart, begin, &p1);
+                pdf->parse_recursion_depth--;
                 begin = p1 + 1;
                 break;
             case '<':
                 if ((size_t)(begin - objstart) < objsize - 2) {
                     if (begin[1] == '<') {
-                        dict  = pdf_parse_dict(pdf, obj, end - objstart, begin, &p1);
+                        pdf->parse_recursion_depth++;
+                        dict = pdf_parse_dict(pdf, obj, end - objstart, begin, &p1);
+                        pdf->parse_recursion_depth--;
                         begin = p1 + 2;
                         break;
                     }
                 }
 
-                val   = pdf_parse_string(pdf, obj, begin, end - objstart, NULL, &p1, NULL);
+                pdf->parse_recursion_depth++;
+                val = pdf_parse_string(pdf, obj, begin, end - objstart, NULL, &p1, NULL);
+                pdf->parse_recursion_depth--;
                 begin = p1 + 2;
                 break;
             default:
@@ -933,6 +963,11 @@ struct pdf_array *pdf_parse_array(struct pdf_struct *pdf, struct pdf_obj *obj, s
     if (!(pdf) || !(obj) || !(begin))
         return NULL;
 
+    if (PDF_OBJECT_RECURSION_LIMIT < pdf->parse_recursion_depth) {
+        cli_dbgmsg("pdf_parse_array: Recursion limit reached\n");
+        return NULL;
+    }
+
     objstart = (obj->objstm) ? (const char *)(obj->start + obj->objstm->streambuf)
                              : (const char *)(obj->start + pdf->map);
 
@@ -1002,19 +1037,25 @@ struct pdf_array *pdf_parse_array(struct pdf_struct *pdf, struct pdf_obj *obj, s
         switch (begin[0]) {
             case '<':
                 if ((size_t)(begin - objstart) < objsize - 2 && begin[1] == '<') {
+                    pdf->parse_recursion_depth++;
                     dict = pdf_parse_dict(pdf, obj, end - objstart, begin, &begin);
+                    pdf->parse_recursion_depth--;
                     begin += 2;
                     break;
                 }
 
                 /* Not a dictionary. Intentionally fall through. */
+                /* fall-through */
             case '(':
+                pdf->parse_recursion_depth++;
                 val = pdf_parse_string(pdf, obj, begin, end - objstart, NULL, &begin, NULL);
+                pdf->parse_recursion_depth--;
                 begin += 2;
                 break;
             case '[':
-                /* XXX We should have a recursion counter here */
+                pdf->parse_recursion_depth++;
                 arr = pdf_parse_array(pdf, obj, end - objstart, begin, &begin);
+                pdf->parse_recursion_depth--;
                 begin += 1;
                 break;
             default:
