@@ -1,7 +1,7 @@
 /*
  *  Detect phishing, based on URL spoofing detection.
  *
- *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -40,7 +40,7 @@
 #include "htmlnorm.h"
 #include "phishcheck.h"
 #include "phish_domaincheck_db.h"
-#include "phish_whitelist.h"
+#include "phish_allow_list.h"
 #include "regex_list.h"
 #include "iana_tld.h"
 #include "iana_cctld.h"
@@ -82,25 +82,25 @@ Steps:
 - convert hostname to lowercase
 - normalize \ to /
 
-3. Matched the urls against a _whitelist_:
-a _realLink_, _displayedLink_ pair is matched against the _whitelist_.
-the _whitelist_ is a list of pairs of realLink, displayedLink. Any of the elements of those pairs can be a _regex_.
- if url *is found* in _whitelist_ --> *CLEAN*
+3. Matched the urls against an _allow_list_:
+a _realLink_, _displayedLink_ pair is matched against the _allow_list_.
+the _allow_list_ is a list of pairs of realLink, displayedLink. Any of the elements of those pairs can be a _regex_.
+ if url *is found* in _allow_list_ --> *CLEAN*
 
-4. URL is looked up in the _domainlist_
-The _domainlist_ is a list of pairs of realLink, displayedLink (any of which can be regex).
+4. URL is looked up in the _domain_list_
+The _domain_list_ is a list of pairs of realLink, displayedLink (any of which can be regex).
 This is the list of domains we do phishing detection for (such as ebay,paypal,chase,....)
 We can't decide to stop processing here or not, so we just set a flag.
 
-Note(*!*): the flags are modified by the the domainlist checker. If domain is found, then the flags associated with it filter the default compile-time flags.
+Note(*!*): the flags are modified by the the domain list checker. If domain is found, then the flags associated with it filter the default compile-time flags.
 
 5. _Hostname_ is extracted from the _displayed URL_.
-It is checked against the _whitelist_, and _domainlist_.
+It is checked against the _allow_list_, and _domain_list_.
 
 6. Now we know if we want to stop processing.
-If we are only scanning domains in the _domainlist_ (default behaviour), and the url/domain
+If we are only scanning domains in the _domain_list_ (default behaviour), and the url/domain
 isn't found in it, we return (and mark url as not_list/clean).
-If we scan all domains, then the domainlist isn't even checked.
+If we scan all domains, then the domain list isn't even checked.
 
 7. URL cloak check.
 check for %00, and hex-encoded IPs in URL.
@@ -127,7 +127,7 @@ if not -> clean
 
 16. Do DNS lookups/reverse lookups. Disabled now (too much load/too many lookups). *
 
-For the Whitelist(.wdb)/Domainlist(.pdb) format see regex_list.c (search for Flags)
+For the AllowList(.wdb)/DomainList(.pdb) format see regex_list.c (search for Flags)
  *
  */
 
@@ -386,7 +386,7 @@ static int get_host(const char* URL, int isReal, int* phishy, const char** hstar
                 return rc;
             if (rc)
                 *phishy |= PHISHY_USERNAME_IN_URL; /* if the url contains a username that is there just to fool people,
-			     					     like http://banksite@example.com/ */
+                                                                     like http://banksite@example.com/ */
             start = realhost + 1;                  /*skip the username*/
         } while (realhost);                        /*skip over multiple @ characters, text following last @ character is the real host*/
     } else if (ismailto && isReal)
@@ -526,7 +526,7 @@ str_strip(char** begin, const char** end, const char* what, size_t what_len)
         return;
 
     /*if(str_end < (sbegin + what_len))
-		return;*/
+                return;*/
     if (strlen(sbegin) < what_len)
         return;
 
@@ -633,7 +633,7 @@ cleanupURL(struct string* URL, struct string* pre_URL, int isReal)
 
     clear_msb(begin);
     /*if(begin == NULL)
-		return;*/
+                return;*/
     /*TODO: handle hex-encoded IPs*/
     while (isspace(*begin))
         begin++;
@@ -665,9 +665,9 @@ cleanupURL(struct string* URL, struct string* pre_URL, int isReal)
 
         str_replace(begin, end, '\\', '/');
         /* find beginning of hostname, because:
-		 * - we want to keep only protocol, host, and
-		 *  strip path & query parameter(s)
-		 * - we want to make hostname lowercase*/
+         * - we want to keep only protocol, host, and
+         *  strip path & query parameter(s)
+         * - we want to make hostname lowercase*/
         host_begin = strchr(begin, ':');
         while (host_begin && (host_begin < end) && (host_begin[1] == '/')) host_begin++;
         if (!host_begin)
@@ -677,8 +677,8 @@ cleanupURL(struct string* URL, struct string* pre_URL, int isReal)
         host_len = strcspn(host_begin, ":/?");
         if (host_begin + host_len > end + 1) {
             /* prevent hostname extending beyond end, it can happen
-			 * if we have spaces at the end, we don't want those part of
-			 * the hostname */
+             * if we have spaces at the end, we don't want those part of
+             * the hostname */
             host_len = end - host_begin + 1;
         } else {
             /* cut the URL after the hostname */
@@ -689,7 +689,7 @@ cleanupURL(struct string* URL, struct string* pre_URL, int isReal)
         /* convert hostname to lowercase, but only hostname! */
         str_make_lowercase(host_begin, host_len);
         /* some broken MUAs put > in the href, and then
-		 * we get a false positive, so remove them */
+         * we get a false positive, so remove them */
         str_replace(begin, end, '<', ' ');
         str_replace(begin, end, '>', ' ');
         str_replace(begin, end, '\"', ' ');
@@ -727,14 +727,11 @@ cl_error_t phishingScan(cli_ctx* ctx, tag_arguments_t* hrefs)
     /* TODO: get_host and then apply regex, etc. */
     int i;
     struct phishcheck* pchk = (struct phishcheck*)ctx->engine->phishcheck;
-    /* check for status of whitelist fatal error, etc. */
+    /* check for status of allow list fatal error, etc. */
     if (!pchk || pchk->is_disabled) {
         goto done;
     }
 
-    if (!ctx->found_possibly_unwanted && !SCAN_ALLMATCHES)
-        *ctx->virname = NULL;
-
     for (i = 0; i < hrefs->count; i++) {
         struct url_check urls;
         enum phish_status phishing_verdict;
@@ -776,32 +773,32 @@ cl_error_t phishingScan(cli_ctx* ctx, tag_arguments_t* hrefs)
             case CL_PHISH_CLEAN:
                 continue;
             case CL_PHISH_NUMERIC_IP:
-                status = cli_append_possibly_unwanted(ctx, "Heuristics.Phishing.Email.Cloaked.NumericIP");
+                status = cli_append_potentially_unwanted(ctx, "Heuristics.Phishing.Email.Cloaked.NumericIP");
                 break;
             case CL_PHISH_CLOAKED_NULL:
-                status = cli_append_possibly_unwanted(ctx, "Heuristics.Phishing.Email.Cloaked.Null"); /*fakesite%01%00@fake.example.com*/
+                status = cli_append_potentially_unwanted(ctx, "Heuristics.Phishing.Email.Cloaked.Null"); /*fakesite%01%00@fake.example.com*/
                 break;
             case CL_PHISH_SSL_SPOOF:
-                status = cli_append_possibly_unwanted(ctx, "Heuristics.Phishing.Email.SSL-Spoof");
+                status = cli_append_potentially_unwanted(ctx, "Heuristics.Phishing.Email.SSL-Spoof");
                 break;
             case CL_PHISH_CLOAKED_UIU:
-                status = cli_append_possibly_unwanted(ctx, "Heuristics.Phishing.Email.Cloaked.Username"); /*http://banksite@fake.example.com*/
+                status = cli_append_potentially_unwanted(ctx, "Heuristics.Phishing.Email.Cloaked.Username"); /*http://banksite@fake.example.com*/
                 break;
             case CL_PHISH_HASH0:
-                status = cli_append_possibly_unwanted(ctx, "Heuristics.Safebrowsing.Suspected-malware_safebrowsing.clamav.net");
+                status = cli_append_potentially_unwanted(ctx, "Heuristics.Safebrowsing.Suspected-malware_safebrowsing.clamav.net");
                 break;
             case CL_PHISH_HASH1:
-                status = cli_append_possibly_unwanted(ctx, "Heuristics.Phishing.URL.Blacklisted");
+                status = cli_append_potentially_unwanted(ctx, "Heuristics.Phishing.URL.Blocked");
                 break;
             case CL_PHISH_HASH2:
-                status = cli_append_possibly_unwanted(ctx, "Heuristics.Safebrowsing.Suspected-phishing_safebrowsing.clamav.net");
+                status = cli_append_potentially_unwanted(ctx, "Heuristics.Safebrowsing.Suspected-phishing_safebrowsing.clamav.net");
                 break;
             case CL_PHISH_NOMATCH:
             default:
-                status = cli_append_possibly_unwanted(ctx, "Heuristics.Phishing.Email.SpoofedDomain");
+                status = cli_append_potentially_unwanted(ctx, "Heuristics.Phishing.Email.SpoofedDomain");
                 break;
         }
-        if (CL_CLEAN != status && !SCAN_ALLMATCHES) {
+        if (CL_SUCCESS != status) {
             goto done;
         }
     }
@@ -813,7 +810,7 @@ done:
 static char hex2int(const unsigned char* src)
 {
     return (src[0] == '0' && src[1] == '0') ? 0x1 : /* don't convert %00 to \0, use 0x1
- 		      * this value is also used by cloak check*/
+                                                     * this value is also used by cloak check*/
                hextable[src[0]] << 4 | hextable[src[1]];
 }
 
@@ -863,8 +860,8 @@ void phishing_done(struct cl_engine* engine)
     if (pchk && !pchk->is_disabled) {
         free_regex(&pchk->preg_numeric);
     }
-    whitelist_done(engine);
-    domainlist_done(engine);
+    allow_list_done(engine);
+    domain_list_done(engine);
     if (pchk) {
         cli_dbgmsg("Freeing phishcheck struct\n");
         MPOOL_FREE(engine->mempool, pchk);
@@ -1013,7 +1010,7 @@ static int isURL(char* URL, int accept_anyproto)
 
     if (!has_proto && (q = memchr(p, '@', end - p))) {
         /* don't phishcheck if displayed URL is email, but do phishcheck if
-	     * foo.TLD@host is used */
+         * foo.TLD@host is used */
         const char* q2 = q - 1;
         while (q2 > p && *q2 != '.') q2--;
         if (q2 == p || !in_tld_set(q2 + 1, q - q2 - 1))
@@ -1105,11 +1102,11 @@ static int url_get_host(struct url_check* url, struct url_check* host_url, int i
 
     if (!host->data || (isReal && (host->data[0] == '\0' || strstr(host->data, ".."))) || *phishy & REAL_IS_MAILTO || strchr(host->data, ' ')) {
         /* no host,
-		 * link without domain, such as: href="/isapi.dll?...
-		 * mailto:
-		 * spaces in hostname
-		 * double dots
-		 */
+         * link without domain, such as: href="/isapi.dll?...
+         * mailto:
+         * spaces in hostname
+         * double dots
+         */
         cli_dbgmsg("Phishcheck:skipping invalid host\n");
         return CL_PHISH_CLEAN;
     }
@@ -1141,9 +1138,9 @@ static enum phish_status phishy_map(int phishy, enum phish_status fallback)
         return fallback;
 }
 
-static cl_error_t whitelist_check(const struct cl_engine* engine, struct url_check* urls, int hostOnly)
+static cl_error_t allow_list_check(const struct cl_engine* engine, struct url_check* urls, int hostOnly)
 {
-    return whitelist_match(engine, urls->realLink.data, urls->displayLink.data, hostOnly);
+    return allow_list_match(engine, urls->realLink.data, urls->displayLink.data, hostOnly);
 }
 
 static cl_error_t hash_match(const struct regex_matcher* rlist,
@@ -1204,7 +1201,7 @@ static cl_error_t hash_match(const struct regex_matcher* rlist,
             cli_dbgmsg("This hash matched: %s\n", h);
             switch (*virname) {
                 case 'W':
-                    cli_dbgmsg("Hash is whitelisted, skipping\n");
+                    cli_dbgmsg("Hash is allowed, skipping\n");
                     break;
                 case '1':
                     *phishing_verdict = CL_PHISH_HASH1;
@@ -1351,7 +1348,7 @@ static cl_error_t url_hash_match(
 
     if (!rlist || !rlist->sha256_hashes.bm_patterns) {
         /* no hashes loaded -> don't waste time canonicalizing and
-		 * looking up */
+         * looking up */
         goto done;
     }
     if ((NULL == inurl) || (NULL == phishing_verdict)) {
@@ -1406,7 +1403,7 @@ static cl_error_t url_hash_match(
         --ki;
         for (ji = COMPONENTS + 1; ji > j;) {
             /* lookup last 2 and 3 components of host, as hostkey prefix,
-		 * if not matched, shortcircuit lookups */
+             * if not matched, shortcircuit lookups */
             int need_prefixmatch = (count < 2 && !prefix_matched) &&
                                    rlist->hostkey_prefix.bm_patterns;
             --ji;
@@ -1479,7 +1476,7 @@ static enum phish_status phishingCheck(cli_ctx* ctx, struct url_check* urls)
         goto done;
     }
 
-    if (CL_SUCCESS != (status = url_hash_match(ctx->engine->domainlist_matcher,
+    if (CL_SUCCESS != (status = url_hash_match(ctx->engine->domain_list_matcher,
                                                urls->realLink.data,
                                                strlen(urls->realLink.data),
                                                &phishing_verdict))) {
@@ -1523,11 +1520,11 @@ static enum phish_status phishingCheck(cli_ctx* ctx, struct url_check* urls)
     }
 
     /*
-     * Whitelist X-type WDB signatures:  X:RealURL:DisplayedURL
+     * AllowList X-type WDB signatures:  X:RealURL:DisplayedURL
      * Eg:
      *      X:.+\.benign\.com([/?].*)?:.+\.benign\.de
      */
-    if (whitelist_check(ctx->engine, urls, 0)) { /* if url is whitelisted don't perform further checks */
+    if (allow_list_check(ctx->engine, urls, 0)) { /* if url is allowed don't perform further checks */
         phishing_verdict = CL_PHISH_CLEAN;
         goto done;
     }
@@ -1537,7 +1534,7 @@ static enum phish_status phishingCheck(cli_ctx* ctx, struct url_check* urls)
      * Eg:
      *      R:.+\.malicious\.net([/?].*)?:.+\.benign\.com
      */
-    /* Provide copies of the oirinal URL's, because domainlist_match() may modify the buffer,
+    /* Provide copies of the oirinal URL's, because domain_list_match() may modify the buffer,
        and we don't want that to happen in this case. */
     realData = cli_strdup(urls->realLink.data);
     if (!realData) {
@@ -1551,7 +1548,7 @@ static enum phish_status phishingCheck(cli_ctx* ctx, struct url_check* urls)
         phishing_verdict = CL_PHISH_CLEAN;
         goto done;
     }
-    if (domainlist_match(ctx->engine, realData, displayData, &urls->pre_fixup, 0)) {
+    if (domain_list_match(ctx->engine, realData, displayData, &urls->pre_fixup, 0)) {
         phishy |= DOMAIN_LISTED;
     }
 
@@ -1576,11 +1573,11 @@ static enum phish_status phishingCheck(cli_ctx* ctx, struct url_check* urls)
     }
 
     /*
-     * Whitelist M-type WDB signatures: M:RealHostname:DisplayedHostname
+     * Allow List M-type WDB signatures: M:RealHostname:DisplayedHostname
      * Eg:
      *      M:email.isbenign.com:benign.com
      */
-    if (whitelist_check(ctx->engine, &host_url, 1)) {
+    if (allow_list_check(ctx->engine, &host_url, 1)) {
         phishing_verdict = CL_PHISH_CLEAN;
         goto done;
     }
@@ -1590,14 +1587,14 @@ static enum phish_status phishingCheck(cli_ctx* ctx, struct url_check* urls)
      * Eg:
      *      H:malicious.com
      */
-    if (domainlist_match(ctx->engine, host_url.displayLink.data, host_url.realLink.data, &urls->pre_fixup, 1)) {
+    if (domain_list_match(ctx->engine, host_url.displayLink.data, host_url.realLink.data, &urls->pre_fixup, 1)) {
         phishy |= DOMAIN_LISTED;
     } else {
         urls->flags &= urls->always_check_flags;
         /* don't return, we may need to check for ssl/cloaking */
     }
 
-    /* link type filtering must occur after last domainlist_match */
+    /* link type filtering must occur after last domain_list_match */
     if (urls->link_type & LINKTYPE_IMAGE && !(urls->flags & CHECK_IMG_URL)) {
         /* its listed, but this link type is filtered */
         phishing_verdict = CL_PHISH_CLEAN;
@@ -1607,8 +1604,8 @@ static enum phish_status phishingCheck(cli_ctx* ctx, struct url_check* urls)
     if (urls->flags & CHECK_CLOAKING) {
         /*
          * Checks if URL is cloaked.
-		 * Should we check if it contains another http://, https://?
-		 * No because we might get false positives from redirect services.
+         * Should we check if it contains another http://, https://?
+         * No because we might get false positives from redirect services.
          */
         if (strchr(urls->realLink.data, 0x1)) {
             phishing_verdict = CL_PHISH_CLOAKED_NULL;
@@ -1675,7 +1672,7 @@ static const char* phishing_ret_toString(enum phish_status phishing_verdict)
         case CL_PHISH_HASH0:
         case CL_PHISH_HASH1:
         case CL_PHISH_HASH2:
-            return "Blacklisted";
+            return "Blocked";
         default:
             return "Unknown return code";
     }
