@@ -41,14 +41,15 @@
 #include "matcher.h"
 
 /* NOTE: Photoshop stores data in BIG ENDIAN format, this is the opposite
-	to virtually everything else */
+        to virtually everything else */
 
 #define special_endian_convert_16(v) be16_to_host(v)
 #define special_endian_convert_32(v) be32_to_host(v)
 
 int cli_check_mydoom_log(cli_ctx *ctx)
 {
-    const uint32_t *record;
+    uint32_t record[16];
+    const uint32_t *ptr;
     uint32_t check, key;
     fmap_t *map         = ctx->fmap;
     unsigned int blocks = map->len / (8 * 4);
@@ -59,14 +60,24 @@ int cli_check_mydoom_log(cli_ctx *ctx)
     if (blocks > 5)
         blocks = 5;
 
-    record = fmap_need_off_once(map, 0, 8 * 4 * blocks);
-    if (!record)
+    /*
+     * The following pointer might not be properly aligned. There there is
+     * memcmp() + memcpy() workaround to avoid performing an unaligned access
+     * while reading the uint32_t.
+     */
+    ptr = fmap_need_off_once(map, 0, 8 * 4 * blocks);
+    if (!ptr)
         return CL_CLEAN;
+
     while (blocks) { /* This wasn't probably intended but that's what the current code does anyway */
-        if (record[--blocks] == 0xffffffff)
+        const uint32_t marker_ff = 0xffffffff;
+
+	if (!memcmp(&ptr[--blocks], &marker_ff, sizeof(uint32_t)))
             return CL_CLEAN;
     }
 
+    memcpy(record, ptr, sizeof(record));
+
     key   = ~be32_to_host(record[0]);
     check = (be32_to_host(record[1]) ^ key) +
             (be32_to_host(record[2]) ^ key) +
@@ -89,7 +100,7 @@ int cli_check_mydoom_log(cli_ctx *ctx)
     if ((~check) != key)
         return CL_CLEAN;
 
-    return cli_append_virus(ctx, "Heuristics.Worm.Mydoom.M.log");
+    return cli_append_potentially_unwanted(ctx, "Heuristics.Worm.Mydoom.M.log");
 }
 
 static uint32_t riff_endian_convert_32(uint32_t value, int big_endian)
@@ -148,10 +159,10 @@ static int riff_read_chunk(fmap_t *map, off_t *offset, int big_endian, int rec_l
         return 0;
     }
     /* FIXME: WTF!?
-	if (lseek(fd, offset, SEEK_SET) != offset) {
-		return 2;
-	}
-	*/
+        if (lseek(fd, offset, SEEK_SET) != offset) {
+                return 2;
+        }
+        */
     return 1;
 }
 
@@ -324,11 +335,11 @@ int cli_detect_swizz(struct swizz_stats *stats)
             uint32_t v = gn[i];
             gn[i]      = (v << 15) / all;
             if (cli_debug_flag)
-                fprintf(stderr, "%lu, ", (unsigned long)gn[i]);
+                cli_eprintf("%lu, ", (unsigned long)gn[i]);
         }
         global_swizz = swizz_j48_global(gn) ? CL_VIRUS : CL_CLEAN;
         if (cli_debug_flag) {
-            fprintf(stderr, "\n");
+            cli_eprintf("\n");
             cli_dbgmsg("cli_detect_swizz: global: %s\n", global_swizz ? "suspicious" : "clean");
         }
     }
