@@ -1,7 +1,7 @@
 /*
  *  Unit tests for bytecode functions.
  *
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -33,13 +33,17 @@
 #include <fcntl.h>
 #include <errno.h>
 
-#include "../libclamav/clamav.h"
-#include "../libclamav/others.h"
-#include "../libclamav/bytecode.h"
+// libclamav
+#include "clamav.h"
+#include "others.h"
+#include "bytecode.h"
+#include "dconf.h"
+#include "bytecode_priv.h"
+#include "pe.h"
+#include "clamav_rust.h"
+
 #include "checks.h"
-#include "../libclamav/dconf.h"
-#include "../libclamav/bytecode_priv.h"
-#include "../libclamav/pe.h"
+
 #ifdef CL_THREAD_SAFE
 #include <pthread.h>
 #endif
@@ -51,7 +55,7 @@ static void runtest(const char *file, uint64_t expected, int fail, int nojit,
 {
     fmap_t *map = NULL;
     int rc;
-    int fd = open_testfile(file);
+    int fd = open_testfile(file, O_RDONLY); /* CBC databases should be opened in text mode (not binary), or else CRLF line indexing with `fgets()` will fail. */
     FILE *f;
     struct cli_bc bc;
     cli_ctx cctx;
@@ -69,13 +73,23 @@ static void runtest(const char *file, uint64_t expected, int fail, int nojit,
     cctx.options = &options;
 
     cctx.options->general |= CL_SCAN_GENERAL_ALLMATCHES;
-    cctx.virname = &virname;
     cctx.engine = engine = cl_engine_new();
     ck_assert_msg(!!cctx.engine, "cannot create engine");
     rc = cl_engine_compile(engine);
     ck_assert_msg(!rc, "cannot compile engine");
-    cctx.fmap = cli_calloc(sizeof(fmap_t *), engine->maxreclevel + 2);
-    ck_assert_msg(!!cctx.fmap, "cannot allocate fmap");
+
+    cctx.evidence = evidence_new();
+
+    cctx.dconf = cctx.engine->dconf;
+
+    cctx.recursion_stack_size = cctx.engine->max_recursion_level;
+    cctx.recursion_stack      = cli_calloc(sizeof(recursion_level_t), cctx.recursion_stack_size);
+    ck_assert_msg(!!cctx.recursion_stack, "cli_calloc() for recursion_stack failed");
+
+    // ctx was memset, so recursion_level starts at 0.
+    cctx.recursion_stack[cctx.recursion_level].fmap = NULL;
+
+    cctx.fmap = cctx.recursion_stack[cctx.recursion_level].fmap;
 
     ck_assert_msg(fd >= 0, "retmagic open failed");
     f = fdopen(fd, "r");
@@ -97,13 +111,13 @@ static void runtest(const char *file, uint64_t expected, int fail, int nojit,
     ck_assert_msg(rc == CL_SUCCESS, "cli_bytecode_load failed");
     fclose(f);
 
-    if (testmode && have_clamjit)
+    if (testmode && have_clamjit())
         engine->bytecode_mode = CL_BYTECODE_MODE_TEST;
 
     rc = cli_bytecode_prepare2(engine, &bcs, BYTECODE_ENGINE_MASK);
     ck_assert_msg(rc == CL_SUCCESS, "cli_bytecode_prepare failed");
 
-    if (have_clamjit && !nojit && nojit != -1 && !testmode) {
+    if (have_clamjit() && !nojit && !testmode) {
         ck_assert_msg(bc.state == bc_jit, "preparing for JIT failed");
     }
 
@@ -113,12 +127,12 @@ static void runtest(const char *file, uint64_t expected, int fail, int nojit,
 
     ctx->ctx = &cctx;
     if (infile) {
-        snprintf(filestr, sizeof(filestr), OBJDIR "/%s", infile);
-        fdin = open(filestr, O_RDONLY);
+        snprintf(filestr, sizeof(filestr), OBJDIR PATHSEP "%s", infile);
+        fdin = open(filestr, O_RDONLY | O_BINARY);
         if (fdin < 0 && errno == ENOENT)
-            fdin = open_testfile(infile);
+            fdin = open_testfile(infile, O_RDONLY | O_BINARY);
         ck_assert_msg(fdin >= 0, "failed to open infile");
-        map = fmap(fdin, 0, 0);
+        map = fmap(fdin, 0, 0, filestr);
         ck_assert_msg(!!map, "unable to fmap infile");
         if (pedata)
             ctx->hooks.pedata = pedata;
@@ -129,25 +143,26 @@ static void runtest(const char *file, uint64_t expected, int fail, int nojit,
     cli_bytecode_context_setfuncid(ctx, &bc, 0);
     rc = cli_bytecode_run(&bcs, &bc, ctx);
     ck_assert_msg(rc == fail, "cli_bytecode_run failed, expected: %u, have: %u\n",
-                    fail, rc);
+                  fail, rc);
 
     if (rc == CL_SUCCESS) {
         v = cli_bytecode_context_getresult_int(ctx);
-        ck_assert_msg(v == expected, "Invalid return value from bytecode run, expected: %llx, have: %llx\n",
-                        expected, v);
+        ck_assert_msg(v == expected, "Invalid return value from bytecode run, expected: " STDx64 ", have: " STDx64 "\n",
+                      expected, v);
     }
     if (infile && expectedvirname) {
         ck_assert_msg(ctx->virname &&
-                        !strcmp(ctx->virname, expectedvirname),
-                    "Invalid virname, expected: %s\n", expectedvirname);
+                          !strcmp(ctx->virname, expectedvirname),
+                      "Invalid virname, expected: %s\n", expectedvirname);
     }
     cli_bytecode_context_destroy(ctx);
     if (map)
         funmap(map);
     cli_bytecode_destroy(&bc);
     cli_bytecode_done(&bcs);
-    free(cctx.fmap);
+    free(cctx.recursion_stack);
     cl_engine_free(engine);
+    evidence_free(cctx.evidence);
     if (fdin >= 0)
         close(fdin);
 }
@@ -155,56 +170,56 @@ static void runtest(const char *file, uint64_t expected, int fail, int nojit,
 START_TEST(test_retmagic_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/retmagic.cbc", 0x1234f00d, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
-    runtest("input/retmagic.cbc", 0x1234f00d, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 1);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "retmagic.cbc", 0x1234f00d, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "retmagic.cbc", 0x1234f00d, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 1);
 }
 END_TEST
 
 START_TEST(test_retmagic_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/retmagic.cbc", 0x1234f00d, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "retmagic.cbc", 0x1234f00d, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_arith_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/arith.cbc", 0xd5555555, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "arith.cbc", 0xd5555555, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_arith_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/arith.cbc", 0xd5555555, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "arith.cbc", 0xd5555555, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_apicalls_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/apicalls.cbc", 0xf00d, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "apicalls.cbc", 0xf00d, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_apicalls_int)
 {
-    runtest("input/apicalls.cbc", 0xf00d, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "apicalls.cbc", 0xf00d, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_apicalls2_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/apicalls2.cbc", 0xf00d, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "apicalls2.cbc", 0xf00d, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_apicalls2_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/apicalls2.cbc", 0xf00d, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "apicalls2.cbc", 0xf00d, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
@@ -212,41 +227,41 @@ START_TEST(test_div0_jit)
 {
     cl_init(CL_INIT_DEFAULT);
     /* must not crash on div#0 but catch it */
-    runtest("input/div0.cbc", 0, CL_EBYTECODE, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "div0.cbc", 0, CL_EBYTECODE, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_div0_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/div0.cbc", 0, CL_EBYTECODE, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "div0.cbc", 0, CL_EBYTECODE, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_lsig_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/lsig.cbc", 0, 0, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "lsig.cbc", 0, 0, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_lsig_int)
 {
-    runtest("input/lsig.cbc", 0, 0, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "lsig.cbc", 0, 0, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_inf_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/inf.cbc", 0, CL_ETIMEOUT, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "inf.cbc", 0, CL_ETIMEOUT, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_inf_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/inf.cbc", 0, CL_ETIMEOUT, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "inf.cbc", 0, CL_ETIMEOUT, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
@@ -268,8 +283,9 @@ START_TEST(test_matchwithread_jit)
     sect.uvsz        = 4096;
     sect.uraw        = 1;
     sect.ursz        = 512;
-    runtest("input/matchwithread.cbc", 0, 0, 0, "../test/clam.exe", &pedata,
-            &sect, "ClamAV-Test-File-detected-via-bytecode", 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "matchwithread.cbc", 0, 0, 0,
+            "input" PATHSEP "clamav_hdb_scanfiles" PATHSEP "clam.exe",
+            &pedata, &sect, "ClamAV-Test-File-detected-via-bytecode", 0);
 }
 END_TEST
 
@@ -291,190 +307,195 @@ START_TEST(test_matchwithread_int)
     sect.uvsz        = 4096;
     sect.uraw        = 1;
     sect.ursz        = 512;
-    runtest("input/matchwithread.cbc", 0, 0, 1, "../test/clam.exe", &pedata,
-            &sect, "ClamAV-Test-File-detected-via-bytecode", 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "matchwithread.cbc", 0, 0, 1,
+            "input" PATHSEP "clamav_hdb_scanfiles" PATHSEP "clam.exe",
+            &pedata, &sect, "ClamAV-Test-File-detected-via-bytecode", 0);
 }
 END_TEST
 
 START_TEST(test_pdf_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/pdf.cbc", 0, 0, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "pdf.cbc", 0, 0, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_pdf_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/pdf.cbc", 0, 0, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "pdf.cbc", 0, 0, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_bswap_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/bswap.cbc", 0xbeef, 0, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "bswap.cbc", 0xbeef, 0, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_bswap_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/bswap.cbc", 0xbeef, 0, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "bswap.cbc", 0xbeef, 0, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_inflate_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/inflate.cbc", 0xbeef, 0, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "inflate.cbc", 0xbeef, 0, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_inflate_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/inflate.cbc", 0xbeef, 0, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "inflate.cbc", 0xbeef, 0, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_api_extract_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/api_extract_7.cbc", 0xf00d, 0, 0, "input/apitestfile", NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "api_extract_7.cbc", 0xf00d, 0, 0,
+            "input" PATHSEP "bytecode_scanfiles" PATHSEP "apitestfile", NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_api_files_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/api_files_7.cbc", 0xf00d, 0, 0, "input/apitestfile", NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "api_files_7.cbc", 0xf00d, 0, 0,
+            "input" PATHSEP "bytecode_scanfiles" PATHSEP "apitestfile", NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_apicalls2_7_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/apicalls2_7.cbc", 0xf00d, 0, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "apicalls2_7.cbc", 0xf00d, 0, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_apicalls_7_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/apicalls_7.cbc", 0xf00d, 0, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "apicalls_7.cbc", 0xf00d, 0, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_arith_7_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/arith_7.cbc", 0xd55555dd, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "arith_7.cbc", 0xd55555dd, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_debug_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/debug_7.cbc", 0xf00d, 0, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "debug_7.cbc", 0xf00d, 0, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_inf_7_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/inf_7.cbc", 0, CL_ETIMEOUT, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "inf_7.cbc", 0, CL_ETIMEOUT, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_lsig_7_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/lsig_7.cbc", 0, 0, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "lsig_7.cbc", 0, 0, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_retmagic_7_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/retmagic_7.cbc", 0x1234f00d, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "retmagic_7.cbc", 0x1234f00d, CL_SUCCESS, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_testadt_jit)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/testadt_7.cbc", 0xf00d, 0, 0, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "testadt_7.cbc", 0xf00d, 0, 0, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_api_extract_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/api_extract_7.cbc", 0xf00d, 0, 1, "input/apitestfile", NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "api_extract_7.cbc", 0xf00d, 0, 1,
+            "input" PATHSEP "bytecode_scanfiles" PATHSEP "apitestfile", NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_api_files_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/api_files_7.cbc", 0xf00d, 0, 1, "input/apitestfile", NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "api_files_7.cbc", 0xf00d, 0, 1,
+            "input" PATHSEP "bytecode_scanfiles" PATHSEP "apitestfile", NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_apicalls2_7_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/apicalls2_7.cbc", 0xf00d, 0, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "apicalls2_7.cbc", 0xf00d, 0, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_apicalls_7_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/apicalls_7.cbc", 0xf00d, 0, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "apicalls_7.cbc", 0xf00d, 0, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_arith_7_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/arith_7.cbc", 0xd55555dd, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "arith_7.cbc", 0xd55555dd, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_debug_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/debug_7.cbc", 0xf00d, 0, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "debug_7.cbc", 0xf00d, 0, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_inf_7_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/inf_7.cbc", 0, CL_ETIMEOUT, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "inf_7.cbc", 0, CL_ETIMEOUT, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_lsig_7_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/lsig_7.cbc", 0, 0, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "lsig_7.cbc", 0, 0, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_retmagic_7_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/retmagic_7.cbc", 0x1234f00d, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "retmagic_7.cbc", 0x1234f00d, CL_SUCCESS, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
 START_TEST(test_testadt_int)
 {
     cl_init(CL_INIT_DEFAULT);
-    runtest("input/testadt_7.cbc", 0xf00d, 0, 1, NULL, NULL, NULL, NULL, 0);
+    runtest("input" PATHSEP "bytecode_sigs" PATHSEP "testadt_7.cbc", 0xf00d, 0, 1, NULL, NULL, NULL, NULL, 0);
 }
 END_TEST
 
@@ -488,20 +509,20 @@ static void runload(const char *dbname, struct cl_engine *engine, unsigned signo
         /* when run from automake srcdir is set, but if run manually then not */
         srcdir = SRCDIR;
     }
-    str = cli_malloc(strlen(dbname) + strlen(srcdir) + 2);
+    str = cli_malloc(strlen(srcdir) + 1 + strlen(dbname) + 1);
     ck_assert_msg(!!str, "cli_malloc");
-    sprintf(str, "%s/%s", srcdir, dbname);
+    sprintf(str, "%s" PATHSEP "%s", srcdir, dbname);
 
     rc = cl_load(str, engine, &signo, CL_DB_STDOPT);
     ck_assert_msg(rc == CL_SUCCESS, "failed to load %s: %s\n",
-                    dbname, cl_strerror(rc));
+                  str, cl_strerror(rc));
     ck_assert_msg(signo == signoexp, "different number of signatures loaded, expected %u, got %u\n",
-                    signoexp, signo);
+                  signoexp, signo);
     free(str);
 
     rc = cl_engine_compile(engine);
     ck_assert_msg(rc == CL_SUCCESS, "failed to load %s: %s\n",
-                    dbname, cl_strerror(rc));
+                  str, cl_strerror(rc));
 }
 
 START_TEST(test_load_bytecode_jit)
@@ -511,7 +532,7 @@ START_TEST(test_load_bytecode_jit)
     engine = cl_engine_new();
     ck_assert_msg(!!engine, "failed to create engine\n");
 
-    runload("input/bytecode.cvd", engine, 5);
+    runload("input" PATHSEP "bytecode_sigs" PATHSEP "bytecode.cvd", engine, 5);
 
     cl_engine_free(engine);
 }
@@ -525,7 +546,7 @@ START_TEST(test_load_bytecode_int)
     engine->dconf->bytecode = BYTECODE_INTERPRETER;
     ck_assert_msg(!!engine, "failed to create engine\n");
 
-    runload("input/bytecode.cvd", engine, 5);
+    runload("input" PATHSEP "bytecode_sigs" PATHSEP "bytecode.cvd", engine, 5);
 
     cl_engine_free(engine);
 }
@@ -545,7 +566,7 @@ static void *thread(void *arg)
     /* run all cl_load at once, to maximize chance of a crash
      * in case of a race condition */
     pthread_barrier_wait(&barrier);
-    runload("input/bytecode.cvd", engine, 5);
+    runload("input" PATHSEP "bytecode_sigs" PATHSEP "bytecode.cvd", engine, 5);
     cl_engine_free(engine);
     return NULL;
 }
