@@ -1,7 +1,7 @@
 /*
  *  Unit tests for JS normalizer.
  *
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  *  Authors: aCaB <acab@clamav.net>
@@ -29,19 +29,26 @@
 #include <check.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <string.h>
 #include <sys/stat.h>
+#include <fcntl.h>
+
+// libclamav
+#include "clamav.h"
+#include "others.h"
+#include "disasm.h"
 
-#include "../libclamav/clamav.h"
-#include "../libclamav/others.h"
-#include "../libclamav/disasm.h"
 #include "checks.h"
 
 START_TEST(test_disasm_basic)
 {
-    char file[]   = "disasmXXXXXX";
-    int fd        = mkstemp(file), ref;
+    int fd               = -1;
+    char *temp_file_path = NULL;
+    cli_gentempfd_with_prefix(NULL, "disasm", &temp_file_path, &fd);
+    int ref;
     uint8_t buf[] = {
         /* m00/rm000 - add [eax], al */
         0x00,
@@ -208,25 +215,26 @@ START_TEST(test_disasm_basic)
 
     };
     uint8_t *d;
-    off_t size;
+    ssize_t size;
     STATBUF st;
 
     ck_assert_msg(fd != -1, "mkstemp failed");
-    ref = open_testfile("input/disasmref.bin");
+    ref = open_testfile("input" PATHSEP "other_scanfiles" PATHSEP "disasmref.bin", O_RDONLY | O_BINARY);
     ck_assert_msg(FSTAT(ref, &st) != -1, "fstat failed");
     disasmbuf(buf, sizeof(buf), fd);
     size = lseek(fd, 0, SEEK_CUR);
-    ck_assert_msg(size == st.st_size, "disasm size mismatch(value %u, expected: %u)", size, st.st_size);
+    ck_assert_msg(size == st.st_size, "disasm size mismatch(value %zd, expected: %zd)", size, (ssize_t)st.st_size);
     lseek(fd, 0, SEEK_SET);
     d = malloc(size * 2);
-    ck_assert_msg(d != NULL, "disasm malloc(%u) failed", size);
+    ck_assert_msg(d != NULL, "disasm malloc(%zd) failed", size * 2);
     ck_assert_msg(read(ref, d, size) == size, "disasm reference read failed");
     ck_assert_msg(read(fd, d + size, size) == size, "disasm read failed");
     close(fd);
     close(ref);
     ck_assert_msg(!memcmp(d, d + size, size), "disasm data doesn't match the reference");
     free(d);
-    unlink(file);
+    unlink(temp_file_path);
+    free(temp_file_path);
 }
 END_TEST
 
